/* libvisual plugin for livido
   authors: Salsaman (G. Finch) <salsaman@xs4all.nl>
            Dennis Smit  <synap@yourbase.nl>
            Duilio J. Protti <dprotti@users.sourceforge.net>

   Released under the Lesser Gnu Public License (LGPL)

 (c) 2004, project authors
*/




#include "../src/plugins-livido.h"
#include <stdio.h>
#include <string.h>
#include <libvisual/lv_actor.h>
#include <libvisual/lv_input.h>

livido_setup_f livido_setup;

livido_init_f libvis_init;
livido_deinit_f libvis_deinit;
livido_process_f libvis_process;

int libvis_host_audio_callback (VisInput *, VisAudio *, void *user_data);

typedef struct {
  VisVideo *video;
  VisActor *actor;
  VisInput *input;
  void *audio_data;
} livido_libvis_t;


/* declare our init function */
livido_instance_t *libvis_init(livido_instance_t *inst) {
  
  livido_libvis_t *libvis=(livido_libvis_t *)malloc (sizeof(livido_libvis_t));

  libvis->actor = visual_actor_new (inst->class->name);

  // TODO - get list of inputs, add list param with values for inputs




  // host supplied audio buffers
  //libvis->input = visual_input_new (NULL);
  //visual_input_set_callback (libvis->input, libvis_host_audio_callback, (void *)libvis);

  libvis->input = visual_input_new ("alsa");
  //libvis->input = visual_input_new ("esd");
  //libvis->input = visual_input_new ("mplayer");
  libvis->video = visual_video_new();

  // Force RGB 24
  visual_video_set_depth (libvis->video, VISUAL_VIDEO_DEPTH_24BIT);
  visual_video_set_dimension (libvis->video, inst->out_channels[0].width, inst->out_channels[0].height);

  // assume pitch is normal

  //visual_video_set_pitch (video, inst->out_);


  visual_actor_realize (libvis->actor);
  visual_actor_set_video (libvis->actor, libvis->video);
  visual_actor_video_negotiate (libvis->actor, 0, FALSE, FALSE);
  visual_input_realize (libvis->input);
  libvis->audio_data=NULL;

  inst->internal=(void *)libvis;
  return inst;
}


int libvis_process
(livido_instance_t *env, livido_frame_t **in, livido_frame_t **out) {
  // host should have returned one output channel and no input channels

  livido_libvis_t *libvis=(livido_libvis_t *)env->internal;
  visual_input_run (libvis->input);
  visual_video_set_buffer (libvis->video, out[0]->pixel_data);
  visual_actor_run (libvis->actor, libvis->input->audio);
  return 1;
}


int libvis_deinit (livido_instance_t *inst) {

  livido_libvis_t *libvis=(livido_libvis_t *)inst->internal;
  if (libvis!=NULL) {
    if (libvis->video!=NULL) visual_video_free (libvis->video);
    if (libvis->actor!=NULL) visual_actor_destroy (libvis->actor);
    if (libvis->input!=NULL) visual_input_destroy (libvis->input);
    free (libvis);
    }
  return 1; // success
}


// TODO - change to RGB24
livido_channel_t in_channels[]={LIVIDO_CHANNEL_END};

livido_palette_t out_palettes[]={LVD_PALETTE_BGR24,LVD_PALETTE_END};
livido_channel_t out_channels[]={{"Output Channel 1",0,out_palettes,0,0,0,0,0,0,0,0,NULL},LIVIDO_CHANNEL_END};


livido_class_t *livido_setup(void) {
  char *name=NULL,*desc;
  int count=0;
  livido_class_t *fx=NULL;

  if (visual_init (NULL,NULL)<0) {
    printf ("Unable to load libvisual plugins\n");
    fx=malloc (sizeof(livido_class_t));
    fx->name=NULL;
    return fx;
  }

  for (;(name=visual_actor_get_next_by_name_nogl (name))!=NULL;count++);
  fx=malloc ((count+1)*sizeof(livido_class_t));
  for (count=0;(name=visual_actor_get_next_by_name_nogl (name))!=NULL;count++) {
    fx[count].name=strdup (name);
    fx[count].author=strdup ("libvisual");
    fx[count].description=strdup (name);
    fx[count].version=1;
    fx[count].livido_api_version=1.7;
    fx[count].properties=LVD_PROPERTY_REALTIME;
    fx[count].parameters=NULL;
    fx[count].in_channels=in_channels;
    fx[count].out_channels=out_channels;
    fx[count].init=&libvis_init;
    fx[count].deinit=&libvis_deinit;
    fx[count].process=&libvis_process;
  }
  fx[count].name=NULL;
  return fx;
}




///////////// callback so host can supply its own audio sammples //////////


int libvis_host_audio_callback (VisInput *input, VisAudio *audio, void *user_data) {
  // audio_data is 16bit signed, little endian
  // two channels non-interlaced, 512 samples in length
  // a rate of 44100Hz is recommended

  // since we have no in channels, we have to use the audio_data of the 
  // output channel (!)

  // return -1 on failure, 0 on success

  livido_libvis_t *libvis=(livido_libvis_t *)user_data;
  if (libvis->audio_data!=NULL) {
    memcpy (audio->plugpcm,libvis->audio_data,2048);
  }
  return 0;
}
