#include "../src/plugins-livido.h"
#include <stdio.h>

livido_setup_f livido_setup;

livido_init_f mirror_init;
livido_deinit_f mirror_deinit;
livido_process_f mirror_process;



/* declare our init function */
livido_instance_t *mirror_init(livido_instance_t *inst) {
  return inst;
}


int mirror_process
(livido_instance_t *env, livido_frame_t **in, livido_frame_t **out) {
  // host should have returned one input channel and one output channel
  // (which can be NULL, since we exported the property INPLACE)

  // we also set our input/output channels same as, so we can assume both frame
  // sizes and palettes match

  int i,j,offset=0;
  int rowstride=env->in_channels[0].rowstrides[0];
  int width=env->in_channels[0].width*3;
  int size=env->in_channels[0].height*rowstride;

  unsigned char *old_pixel_data=in[0]->pixel_data, *new_pixel_data;
  livido_palette_t palette=env->in_channels[0].palettes[0];

  int inplace=(out==NULL||out[0]==NULL);

  if (palette==LVD_PALETTE_BGR24) offset=-1;

  if (inplace) {
    new_pixel_data=old_pixel_data;
  }
  else new_pixel_data=out[0]->pixel_data;


  for (i=0;i<size;i+=rowstride) {
    for (j=0;j<width/2;j++) {
      new_pixel_data[i+width-1-j+offset*2]=old_pixel_data[i+j];
      if (!inplace) new_pixel_data[i+j]=old_pixel_data[i+j];
      if (palette==LVD_PALETTE_BGR24) offset=++offset<2?offset:-1;
    }
  } 
  return 1;
}


int mirror_deinit (livido_instance_t *inst) {
  return 1; // success
}


// create 1 input and one matching output channel, no masks

// allow RGB24 and BGR24 palettes

livido_palette_t in_palettes[]={LVD_PALETTE_RGB24,LVD_PALETTE_BGR24,LVD_PALETTE_END};
livido_palette_t out_palettes[]={LVD_PALETTE_RGB24,LVD_PALETTE_BGR24,LVD_PALETTE_END};

livido_channel_t in_channels[]={{"Input Channel 1",0,in_palettes,0,0,0,0,0,0,0,0,NULL},LIVIDO_CHANNEL_END};

livido_channel_t out_channels[]={{"Output Channel 1",0,out_palettes,1,0,0,1,0,0,0,0,NULL},LIVIDO_CHANNEL_END};

livido_class_t fx[]={{"mirror","salsaman","mirror example",1,1.7,LVD_PROPERTY_REALTIME|LVD_PROPERTY_INPLACE,NULL,in_channels,out_channels,mirror_init,mirror_deinit,mirror_process},LIVIDO_CLASS_END};


livido_class_t *livido_setup(void) {
  return fx;
}

