#include "../src/plugins-livido.h"
#include <stdio.h>

livido_setup_f livido_setup;

livido_init_f posterise_init;
livido_deinit_f posterise_deinit;
livido_process_f posterise_process;

/* declare our init function */
livido_instance_t *posterise_init(livido_instance_t *inst) {
  return inst;
}


int posterise_process
(livido_instance_t *env, livido_frame_t **in, livido_frame_t **out) {
  // host should have returned one input channel and one output channel
  // (which can be NULL, since we exported the property INPLACE)

  // we also set our input/output channels same as, so we can assume both frame
  // sizes and palettes match

  int i,j,offset=0;
  int rowstride=env->in_channels[0].rowstrides[0];
  int width=env->in_channels[0].width*3;
  int size=env->in_channels[0].height*rowstride;

  unsigned char *old_pixel_data=in[0]->pixel_data, *new_pixel_data;

  int inplace=(out==NULL||out[0]==NULL);

  if (inplace) {
    new_pixel_data=old_pixel_data;
  }
  else new_pixel_data=out[0]->pixel_data;

  for (i=0;i<size;i+=rowstride) {
    for (j=0;j<width;j++) {
      new_pixel_data[i+j]=new_pixel_data[i+j]>128?255:0;
    }
  } 
  return 1;
}


int posterise_deinit (livido_instance_t *inst) {
  return 1; // success
}


// create 1 input and one matching output channel, no masks

// allow RGB24 and BGR24 palettes

livido_palette_t in_palettes[]={LVD_PALETTE_RGB24,LVD_PALETTE_BGR24,LVD_PALETTE_END};
livido_palette_t out_palettes[]={LVD_PALETTE_RGB24,LVD_PALETTE_BGR24,LVD_PALETTE_END};

livido_channel_t in_channels[]={{"Input Channel 1",0,in_palettes,0,0,0,0,0,0,0,0,NULL},LIVIDO_CHANNEL_END};

livido_channel_t out_channels[]={{"Output Channel 1",0,out_palettes,1,0,0,1,0,0,0,0,NULL},LIVIDO_CHANNEL_END};

livido_class_t fx[]={{"posterise","salsaman","posterise",1,1.7,LVD_PROPERTY_REALTIME|LVD_PROPERTY_INPLACE,NULL,in_channels,out_channels,posterise_init,posterise_deinit,posterise_process},LIVIDO_CLASS_END};


livido_class_t *livido_setup(void) {
  return fx;
}

