#include "../src/plugins-livido.h"
#include <stdio.h>

livido_setup_f livido_setup;

livido_init_f simple_blend_init;
livido_deinit_f simple_blend_deinit;
livido_process_f simple_blend_process;


/* precomputed tables */
#define FP_BITS 18

static int Y_R[256];
static int Y_G[256];
static int Y_B[256];
static int Cb_R[256];
static int Cb_G[256];
static int Cb_B[256];
static int Cr_R[256];
static int Cr_G[256];
static int Cr_B[256];
static int conv_RY_inited = 0;

static int RGB_Y[256];
static int R_Cr[256];
static int G_Cb[256];
static int G_Cr[256];
static int B_Cb[256];
static int conv_YR_inited = 0;


static int myround(double n)
{
  if (n >= 0) 
    return (int)(n + 0.5);
  else
    return (int)(n - 0.5);
}



static void init_RGB_to_YCbCr_tables(void)
{
  int i;

  /*
   * Q_Z[i] =   (coefficient * i
   *             * (Q-excursion) / (Z-excursion) * fixed-pogint-factor)
   *
   * to one of each, add the following:
   *             + (fixed-pogint-factor / 2)         --- for rounding later
   *             + (Q-offset * fixed-pogint-factor)  --- to add the offset
   *             
   */
  for (i = 0; i < 256; i++) {
    Y_R[i] = myround(0.299 * (double)i 
		     * 219.0 / 255.0 * (double)(1<<FP_BITS));
    Y_G[i] = myround(0.587 * (double)i 
		     * 219.0 / 255.0 * (double)(1<<FP_BITS));
    Y_B[i] = myround((0.114 * (double)i 
		      * 219.0 / 255.0 * (double)(1<<FP_BITS))
		     + (double)(1<<(FP_BITS-1))
		     + (16.0 * (double)(1<<FP_BITS)));

    Cb_R[i] = myround(-0.168736 * (double)i 
		      * 224.0 / 255.0 * (double)(1<<FP_BITS));
    Cb_G[i] = myround(-0.331264 * (double)i 
		      * 224.0 / 255.0 * (double)(1<<FP_BITS));
    Cb_B[i] = myround((0.500 * (double)i 
		       * 224.0 / 255.0 * (double)(1<<FP_BITS))
		      + (double)(1<<(FP_BITS-1))
		      + (128.0 * (double)(1<<FP_BITS)));

    Cr_R[i] = myround(0.500 * (double)i 
		      * 224.0 / 255.0 * (double)(1<<FP_BITS));
    Cr_G[i] = myround(-0.418688 * (double)i 
		      * 224.0 / 255.0 * (double)(1<<FP_BITS));
    Cr_B[i] = myround((-0.081312 * (double)i 
		       * 224.0 / 255.0 * (double)(1<<FP_BITS))
		      + (double)(1<<(FP_BITS-1))
		      + (128.0 * (double)(1<<FP_BITS)));
  }
  conv_RY_inited = 1;
}




static void init_YCbCr_to_RGB_tables(void)
{
  int i;

  int startclamp_y=16;
  int endclamp_y=235;

  int startclamp_crcb=16;
  int endclamp_crcb=240;

  /* clip Y values under 16 */
    for (i = 0; i < startclamp_y; i++) {
    RGB_Y[i] = myround((1.0 * (double)(16 - 16) 
		     * 255.0 / 219.0 * (double)(1<<FP_BITS))
		    + (double)(1<<(FP_BITS-1)));
		    }
  for (i = startclamp_y; i <= endclamp_y; i++) {
    RGB_Y[i] = myround((1.0 * (double)(i - 16) 
		     * 255.0 / 219.0 * (double)(1<<FP_BITS))
		    + (double)(1<<(FP_BITS-1)));
  }
  /* clip Y values above 235 */
    for (i = endclamp_y+1; i < 256; i++) {
    RGB_Y[i] = myround((1.0 * (double)(235 - 16) 
		     * 255.0 / 219.0 * (double)(1<<FP_BITS))
		    + (double)(1<<(FP_BITS-1)));
  }
  
  /* clip Cb/Cr values below 16 */	 
    for (i = 0; i < startclamp_crcb; i++) {
    R_Cr[i] = myround(1.402 * (double)(-112)
		   * 255.0 / 224.0 * (double)(1<<FP_BITS));
    G_Cr[i] = myround(-0.714136 * (double)(-112)
		   * 255.0 / 224.0 * (double)(1<<FP_BITS));
    G_Cb[i] = myround(-0.344136 * (double)(-112)
		   * 255.0 / 224.0 * (double)(1<<FP_BITS));
    B_Cb[i] = myround(1.772 * (double)(-112)
		   * 255.0 / 224.0 * (double)(1<<FP_BITS));
		   }
  for (i = startclamp_crcb; i <= endclamp_crcb; i++) {
    R_Cr[i] = myround(1.402 * (double)(i - 128)
		   * 255.0 / 224.0 * (double)(1<<FP_BITS));
    G_Cr[i] = myround(-0.714136 * (double)(i - 128)
		   * 255.0 / 224.0 * (double)(1<<FP_BITS));
    G_Cb[i] = myround(-0.344136 * (double)(i - 128)
		   * 255.0 / 224.0 * (double)(1<<FP_BITS));
    B_Cb[i] = myround(1.772 * (double)(i - 128)
		   * 255.0 / 224.0 * (double)(1<<FP_BITS));
  }
  /* clip Cb/Cr values above 240 */	 
    for (i = endclamp_crcb+1; i < 256; i++) {
    R_Cr[i] = myround(1.402 * (double)(112)
		   * 255.0 / 224.0 * (double)(1<<FP_BITS));
    G_Cr[i] = myround(-0.714136 * (double)(112)
		   * 255.0 / 224.0 * (double)(1<<FP_BITS));
    G_Cb[i] = myround(-0.344136 * (double)(i - 128)
		   * 255.0 / 224.0 * (double)(1<<FP_BITS));
    B_Cb[i] = myround(1.772 * (double)(112)
		   * 255.0 / 224.0 * (double)(1<<FP_BITS));
		   }
  conv_YR_inited = 1;
}


uint8_t 
calc_luma (uint8_t *pixel) {
  return (Y_R[pixel[0]] + Y_G[pixel[1]]+ Y_B[pixel[2]]) >> FP_BITS;
}





/* declare our init function */
livido_instance_t *common_init(livido_instance_t *inst) {
  return inst;
}


int common_process
(int type, livido_instance_t *env, livido_frame_t **in, livido_frame_t **out) {
  // host should have returned one input channel and one output channel
  // (which can be NULL, since we exported the property INPLACE)

  // we also set our input/output channels same as, so we can assume both frame
  // sizes and palettes match

  int i,j,k,offset=0;
  int rowstride0=env->in_channels[0].rowstrides[0];
  int rowstride1=env->in_channels[1].rowstrides[0];
  int width=env->in_channels[0].width*3;
  int size=env->in_channels[0].height*rowstride0;

  int bf;
  uint8_t blend_factor;

  unsigned char *old_pixel_data=in[0]->pixel_data, *new_pixel_data, *blend_data=in[1]->pixel_data;
  int inplace=(out==NULL||out[0]==NULL);

  memcpy(&bf,env->parameter[0],sizeof(int));
  blend_factor=(uint8_t)bf;

  if (inplace) {
    new_pixel_data=old_pixel_data;
  }
  else new_pixel_data=out[0]->pixel_data;

  for (i=0;i<size;i+=rowstride0) {
    k=i/rowstride0*rowstride1;
    for (j=0;j<width;j++) {
      switch (type) {
      case 0:
	// chroma blend
	new_pixel_data[i+j]=(uint8_t)((new_pixel_data[i+j]*blend_factor+blend_data[k+j]*(256-blend_factor))>>8);
	new_pixel_data[i+j+1]=(uint8_t)((new_pixel_data[i+j+1]*blend_factor+blend_data[k+j+1]*(256-blend_factor))>>8);
	new_pixel_data[i+j+2]=(uint8_t)((new_pixel_data[i+j+2]*blend_factor+blend_data[k+j+2]*(256-blend_factor))>>8);
	break;
      case 1:
	if (calc_luma (&new_pixel_data[i+j])<(blend_factor)) {
	  new_pixel_data[i+j]=blend_data[k+j];
	  new_pixel_data[i+j+1]=blend_data[k+j+1];
	  new_pixel_data[i+j+2]=blend_data[k+j+2];
	}
	break;	
      case 2:
	// threshold luma blend (foreground)
	if (calc_luma (&blend_data[k+j])>(blend_factor)) {
	  new_pixel_data[i+j]=blend_data[k+j];
	  new_pixel_data[i+j+1]=blend_data[k+j+1];
	  new_pixel_data[i+j+2]=blend_data[k+j+2];
	}
	break;
      }
    }
  }
  return 1;
}


int common_deinit (livido_instance_t *inst) {
  return 1; // success
}

int chroma_process
(livido_instance_t *env, livido_frame_t **in, livido_frame_t **out) {
  return common_process(0,env,in,out);
}

int lumo_process
(livido_instance_t *env, livido_frame_t **in, livido_frame_t **out) {
  return common_process(1,env,in,out);
}

int lumu_process
(livido_instance_t *env, livido_frame_t **in, livido_frame_t **out) {
  return common_process(2,env,in,out);
}

// create 1 input and one matching output channel, no masks

// allow RGB24 and BGR24 palettes

livido_palette_t in_palettes[]={LVD_PALETTE_RGB24,LVD_PALETTE_BGR24,LVD_PALETTE_END};
livido_palette_t out_palettes[]={LVD_PALETTE_RGB24,LVD_PALETTE_BGR24,LVD_PALETTE_END};

livido_channel_t in_channels[]={{"Input Channel 1",0,in_palettes,0,0,0,0,0,0,0,0,NULL},{"Input Channel 2",0,in_palettes,0,0,0,0,0,0,0,0,NULL},LIVIDO_CHANNEL_END};

livido_channel_t out_channels[]={{"Output Channel 1",0,out_palettes,1,0,0,1,0,0,0,0,NULL},LIVIDO_CHANNEL_END};

int paramdef1=200;
int paramdef2=0;

livido_parameter_t params[]={{"amount","blend amount",NULL,LVD_PARAM_NUMBER,0,0.,256.,&paramdef1,0,0},{"mode","fx mode",NULL,LVD_PARAM_NUMBER,0,0.,2.,&paramdef2,0,0},LIVIDO_PARAM_END};

livido_class_t fx[]={{"chroma blend","salsaman","chroma blend",1,1.7,LVD_PROPERTY_REALTIME|LVD_PROPERTY_INPLACE,params,in_channels,out_channels,common_init,common_deinit,chroma_process},{"luma overlay","salsaman","luma overlay",1,1.7,LVD_PROPERTY_REALTIME|LVD_PROPERTY_INPLACE,params,in_channels,out_channels,common_init,common_deinit,lumo_process},{"luma underlay","salsaman","luma underlay",1,1.7,LVD_PROPERTY_REALTIME|LVD_PROPERTY_INPLACE,params,in_channels,out_channels,common_init,common_deinit,lumu_process},LIVIDO_CLASS_END};


livido_class_t *livido_setup(void) {
  init_RGB_to_YCbCr_tables();
  return fx;
}

