// WARNING - THIS IS NOT AN OFFICIAL RELEASE OF LIVIDO
// This is only for test purposese in LiVES
// For official releases, see www.dyne.org

// effects-livido.c
// LiVES
// (c) G. Finch 2004 <salsaman@xs4all.nl>
// see file COPYING for licensing details

#include "support.h"
#include "main.h"

#ifdef USE_LIVIDO

#include "dlfcn.h"

// on a standard keyboard, ctrl-1 to ctrl-9 control effects, ctrl-0 clears fx
#define MAX_MODES_PER_KEY 8
#define MAX_LIVIDO_INSTANCES (FX_KEYS*MAX_MODES_PER_KEY)

// dummy channel size when no clip is open
#define DEFAULT_CHANNEL_HSIZE 320
#define DEFAULT_CHANNEL_VSIZE 240

// fixed size for generators - TODO - make configurable
#define DEF_GEN_WIDTH 320
#define DEF_GEN_HEIGHT 240

static livido_class_t *livido_fx[MAX_LIVIDO_PLUGINS];
static livido_instance_t *livido_instance[MAX_LIVIDO_INSTANCES];
static void *livido_handles[MAX_LIVIDO_PLUGINS];

// each 'hotkey' will control n instances, selectable as 'modes' or banks
static int key_to_instance[FX_KEYS>0?FX_KEYS-1:1][MAX_MODES_PER_KEY];
static int key_to_fx[FX_KEYS>0?FX_KEYS-1:1][MAX_MODES_PER_KEY];
static int key_modes[FX_KEYS>0?FX_KEYS-1:1];

static int next_free_instance=0;

gint num_livido_plugins=0;

// subtract this from num_livido_plugins to give physical number of plugins
static gint num_modes;


// arrays of in/out channels, these get created when the plugins are loaded
// each pointer is to a NULL terminated array of channels
static livido_channel_t *my_in_channels[MAX_LIVIDO_PLUGINS];
static livido_channel_t *my_out_channels[MAX_LIVIDO_PLUGINS];

// store keys so we now eg, which rte mask entry to xor when deiniting
static gint fg_generator_key;
static gint bg_generator_key;

#define DEBUG_LIVIDO

inline int get_next_free_instance(void) {
  int i;
  for (i=next_free_instance+1;i!=next_free_instance;i++) if (livido_instance[(i=(i==MAX_LIVIDO_INSTANCES?0:i))]==NULL) return i;
  return -1;
}

gboolean check_lvd_palette_list (livido_channel_t *chan, livido_palette_t palette) {
  int i;

  if (chan==NULL||chan->palettes==NULL) return FALSE;
  if (palette==LVD_PALETTE_END) return TRUE;

  for (i=0;chan->palettes[i]!=LVD_PALETTE_END;i++)
      if (chan->palettes[i]==palette) return TRUE;
  return FALSE;
}


void set_in_channel_palettes (gint idx, gint num_channels) {
  // set in channel palettes for filter[idx]
  // in this case we fill first the mandatory channels,
  // then if necessary the optional channels
  int i;


  // fill mandatory channels
  for (i=0;my_in_channels[idx][i].palettes!=NULL;i++) {
    if (!my_in_channels[idx][i].optional) {
      my_in_channels[idx][i].palettes[0]=LVD_PALETTE_BGR24;
      num_channels--;
    }
  }

  if (num_channels) {
    // OK we need to use some optional channels
    for (i=0;my_in_channels[idx][i].palettes!=NULL;i++) {
      if (my_in_channels[idx][i].optional) {
	// got one !
	if (num_channels--) {
	  // use this one
	  my_in_channels[idx][i].optional=0; // indicate we will use it
	  my_in_channels[idx][i].palettes[0]=LVD_PALETTE_BGR24;
	}
      }
    }
  }
}




void set_out_channel_palettes (gint idx, gint num_channels) {
  // same as set in channels, but this time we cross ref the "same as"
  // palettes
  int i;

  // fill mandatory channels
  for (i=0;my_out_channels[idx][i].palettes!=NULL;i++) {
    if (!my_out_channels[idx][i].optional) {
      my_out_channels[idx][i].palettes[0]=LVD_PALETTE_BGR24;
      num_channels--;
    }
  }

  if (num_channels) {
    // OK we need to use some optional channels
    for (i=0;my_out_channels[idx][i].palettes!=NULL;i++) {
      if (my_out_channels[idx][i].optional) {
	// got one !
	if (num_channels--) {
	  my_out_channels[idx][i].optional=0;
	  my_out_channels[idx][i].palettes[0]=LVD_PALETTE_BGR24;
	}
      }
    }
  }


  // now we set same as palettes
  for (i=0;my_in_channels[idx][i].palettes!=NULL;i++) {
    // in channels

    // palette
    if (my_in_channels[idx][i].same_palette>0) my_in_channels[idx][i].palettes[0]=my_in_channels[idx][my_in_channels[idx][i].same_palette].palettes[0];
    else if (my_in_channels[idx][i].same_palette<0) my_in_channels[idx][i].palettes[0]=my_out_channels[idx][-my_out_channels[idx][i].same_palette].palettes[0];
  }

  for (i=0;my_out_channels[idx][i].palettes!=NULL;i++) {
    // in channels

    // palette
    if (my_out_channels[idx][i].same_palette>0) my_out_channels[idx][i].palettes[0]=my_in_channels[idx][my_in_channels[idx][i].same_palette].palettes[0];
    else if (my_out_channels[idx][i].same_palette<0) my_out_channels[idx][i].palettes[0]=my_out_channels[idx][-my_out_channels[idx][i].same_palette].palettes[0];
  }

  // same_size will be checked in init function

}




gboolean check_for_lives(livido_class_t *filter, gint idx) {
  // for LiVES, currently:
  // all effects must take 0, 1 or 2 inputs and provide 
  // at least 1 output (for now)

  // all channels used must support BGR24 (for now)

  gint chans_in_mand=0;
  gint chans_in_opt_max=0;
  gint chans_out_mand=0;
  gint chans_out_opt_max=0;

  int i;


  my_in_channels[idx]=my_out_channels[idx]=NULL;

  // for now we will only load realtime effects

  if (!(filter->properties&LVD_PROPERTY_REALTIME)) return FALSE;

  // count number of mandatory and optional in_channels
  // restrict it just to BGR24
  for (i=0;filter->in_channels[i].palettes!=NULL;i++) {
    if (!filter->in_channels[i].optional) {
      // is mandatory
      if (!check_lvd_palette_list (&filter->in_channels[i],LVD_PALETTE_BGR24)) return FALSE;
      chans_in_mand++;
    }
    else {
      if (check_lvd_palette_list (&filter->in_channels[i],LVD_PALETTE_BGR24)) chans_in_opt_max++;
    }
  }


  // count number of mandatory and optional in_channels
  // restrict it just to BGR24
  for (i=0;filter->out_channels[i].palettes!=NULL;i++) {
    if (!filter->out_channels[i].optional) {
      // is mandatory
      if (!check_lvd_palette_list (&filter->out_channels[i],LVD_PALETTE_BGR24)) return FALSE;
      chans_out_mand++;
    }
    else {
      if (check_lvd_palette_list (&filter->out_channels[i],LVD_PALETTE_BGR24)) chans_out_opt_max++;
    }
  }


#ifdef DEBUG_LIVIDO
  g_print ("got in channs %d m %d o out channs %d m %d o\n",chans_in_mand,chans_in_opt_max,chans_out_mand,chans_out_opt_max);
#endif

  if (chans_in_mand>2) return FALSE;
  if (chans_out_mand>1||!(chans_out_mand+chans_out_opt_max)) return FALSE;

  // create our copy of the channels
  my_in_channels[idx]=(livido_channel_t *)g_malloc ((chans_in_mand+chans_in_opt_max+1)*sizeof(livido_channel_t));
  my_out_channels[idx]=(livido_channel_t *)g_malloc ((chans_out_mand+chans_out_opt_max+1)*sizeof(livido_channel_t));
  
  for (i=0;i<chans_in_mand+chans_in_opt_max;i++) {
    my_in_channels[idx][i].palettes=(livido_palette_t *)g_malloc(sizeof(livido_palette_t));
    my_in_channels[idx][i].palettes[0]=LVD_PALETTE_END;
    g_snprintf(my_in_channels[idx][i].name,512,"%s",filter->in_channels[i].name);
    my_in_channels[idx][i].same_palette=filter->in_channels[i].same_palette;
  }
  my_in_channels[idx][i].palettes=NULL;
  
  for (i=0;i<chans_out_mand+chans_out_opt_max;i++) {
    my_out_channels[idx][i].palettes=(livido_palette_t *)g_malloc(sizeof(livido_palette_t));
    my_out_channels[idx][i].palettes[0]=LVD_PALETTE_END;
    g_snprintf(my_out_channels[idx][i].name,512,"%s",filter->out_channels[i].name);
    my_out_channels[idx][i].same_palette=filter->out_channels[i].same_palette;
  }
  my_out_channels[idx][i].palettes=NULL;

  return TRUE;
}


void livido_init_all (void) {
// get list of plugins from directory and create our fx
  int i,j,idx=0,plugin_idx;
  gchar *plugin_name;
  GList *livido_fx_list=get_plugin_list(PLUGIN_LIVIDO_FX_BUILTIN,TRUE);
  gchar *plugin_path=g_strdup_printf("%s%s",PLUGIN_DIR,PLUGIN_LIVIDO_FX_BUILTIN);
  gchar *fx_path;
  gchar *msg;
  gint mode=0;
  gint skipped=0;
  gint key=-1;

  livido_setup_f *setup_fn;
  livido_class_t *fx_array;

#ifdef DEBUG_LIVIDO
  g_printerr("In livido init\n");
#endif
  num_modes=0;

  // danger Will Robinson !
  fg_generator_key=-1;
  bg_generator_key=-1;

  // reset all instances to NULL
  for (i=0;i<MAX_LIVIDO_INSTANCES;livido_instance[i++]=NULL);
  for (i=0;i<FX_KEYS-1;i++) {
    key_modes[i]=0;
    for (j=0;j<MAX_MODES_PER_KEY;j++) {
      key_to_fx[i][j]=-1;
    }
  }

  // walk list and create fx structures
  for (plugin_idx=0;plugin_idx<g_list_length(livido_fx_list);plugin_idx++) {
    plugin_name=g_strdup(g_list_nth_data(livido_fx_list,plugin_idx));

    // TODO - replace with livido library fn
    // TODO - add error checking
    fx_path=g_strdup_printf("%s/%s/%s",PREFIX,plugin_path,plugin_name);

#ifdef DEBUG_LIVIDO
    g_printerr("Checking plugin %s\n",fx_path);
#endif

    if ((livido_handles[plugin_idx-skipped]=dlopen(fx_path,RTLD_LAZY))) {
      setup_fn=dlsym(livido_handles[plugin_idx-skipped],"livido_setup");
      mode=0;
      if (!(fx_array=(*setup_fn)())) g_printerr("bang, plugin exploded !\n");
      else {
	for (;fx_array->name!=NULL&&idx<MAX_LIVIDO_PLUGINS;fx_array++) {
	  if (check_for_lives(fx_array,idx)) {
#ifdef DEBUG_LIVIDO

	    if (!mode) key++;
	    if (key<FX_KEYS-1) msg=g_strdup_printf("Loaded effect \"%s\" in plugin \"%s\"; assigned to key ctrl-%d, mode %d.\n",fx_array->name,plugin_name,key+1,mode);
	    else msg=g_strdup_printf("Loaded effect \"%s\" in plugin \"%s\", no key assigned",fx_array->name,plugin_name);
	    d_print(msg);
	    g_free(msg);
#endif
	   
	    // TODO - load effects with variable number of in channels multiple
	    // times

	    if (my_in_channels[idx][0].palettes!=NULL) {
	      if (my_in_channels[idx][1].palettes==NULL) {
		// effect1
		set_in_channel_palettes (idx,1);
	      }
	      else {
		set_in_channel_palettes (idx,2);
	      }
	    }
	    set_out_channel_palettes (idx,1);
	    livido_fx[idx]=fx_array;
	    if (key<FX_KEYS-1) {
	      key_to_fx[key][mode]=idx;
	    }
	    mode++;
	    idx++;
	  }}}
      if (mode>0) num_modes+=--mode;
    }
    else {
      g_printerr(_ ("Unable to load plugin.\n"));
      skipped++;
    }
    g_free(plugin_name);
  }
  num_livido_plugins=idx-skipped;
  msg=g_strdup_printf(_ ("Loaded %d Livido effects\n"),idx-skipped);
  d_print(msg);
  g_free(msg);

  // TODO - add any rendered effects to fx submenu
}



GdkPixbuf *gdk_pixbuf_new_from_generator (void *src, GError **error) {
  GdkPixbuf *pixbuf=NULL;
  guchar *pixels;
  livido_frame_t *ofp[1];
  livido_instance_t *inst=(livido_instance_t *)src;
  livido_timecode_t tc=LIVIDO_TIMECODE_UNDEFINED;
  
  // construct our gdk_pixbuf
  if ((pixbuf=gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, inst->out_channels[0].width, inst->out_channels[0].height))==NULL) return NULL;

  pixels=gdk_pixbuf_get_pixels (pixbuf);

  // WARNING WARNING !
  if (gdk_pixbuf_get_rowstride (pixbuf)!=inst->out_channels[0].width*3) return NULL;

  ofp[0]=(livido_frame_t *)g_malloc(sizeof(livido_frame_t));
  ofp[0]->timecode=tc;
  ofp[0]->x=0.;
  ofp[0]->y=0.;
  ofp[0]->pixel_data=pixels;
  ofp[0]->audio_data=NULL;
  ofp[0]->extra=NULL;
  (*inst->class->process)(inst,NULL,ofp);
  g_free (ofp[0]);
  return pixbuf;
}

void livido_generator_end (livido_instance_t *inst) {
  // generator has stopped for one of the following reasons:
  // efect was de-inited; clip (bg/fg) was changed; playback stopped with fg
  gboolean is_bg=FALSE;
  gint current_file=mainw->current_file;

  if (mainw->blend_file!=-1&&mainw->blend_file!=current_file&&mainw->files[mainw->blend_file]->ext_src==inst) is_bg=TRUE;
  
  (*(inst->class)->deinit)(inst);

  // TODO - use instance->class
  LIVIDO_FREE_INSTANCE(inst->class,inst);
  inst=NULL;

  if (is_bg) {
    bg_generator_key=-1;
    mainw->current_file=mainw->blend_file;
    mainw->blend_file=-1;
  }
  else {
    fg_generator_key=-1;
    if (mainw->rte&=(1<<fg_generator_key)) mainw->rte^=(1<<fg_generator_key);
  }
  cfile->ext_src=NULL;
  cfile->disk_frames--;

  if (cfile->disk_frames<1) {
    close_current_file();
    if (is_bg) mainw->current_file=current_file;
    return;
  }
  
  // some frames were recorded
  g_snprintf(cfile->name,256,g_strdup_printf(_ ("Untitled%d"),mainw->untitled_number++));
  set_menu_text (cfile->menuentry,cfile->name,FALSE);
  add_to_winmenu();
  cfile->is_untitled=TRUE;
  cfile->end=cfile->frames=cfile->disk_frames;
  cfile->clip_type=CLIP_TYPE_DISK;
  cfile->changed=TRUE;
  cfile->bpp=24;
  if (is_bg) mainw->current_file=current_file;
  else if (mainw->playing_file==-1) {
    switch_to_file ((mainw->current_file=0),current_file);
  }
  return;
}



gboolean livido_generator_start (livido_instance_t *inst) {
  // make an "ephemeral clip"

  // cf. yuv4mpeg.c
  // start "playing" but receive frames from a plugin

  gint old_file=mainw->current_file;

  // create a virtual clip
  gint new_file=mainw->first_free_file;

  if (mainw->is_processing) return FALSE;
  if ((mainw->preview||(mainw->current_file>-1&&cfile->opening))&&!mainw->num_tr_applied) return FALSE;

  if (!get_new_handle(new_file,g_strdup_printf (_ ("generator:%s"),inst->class->name))) {
    mainw->error=TRUE;
    return FALSE;
  }

  if (mainw->pre_src_file==-1) mainw->pre_src_file=old_file;

  mainw->current_file=new_file;
  cfile->clip_type=CLIP_TYPE_GENERATOR;
  cfile->ext_src=inst;

  cfile->achans=0;
  cfile->asampsize=0;

  // open as a clip with 1 frame
  cfile->start=cfile->end=cfile->frames=1;
  cfile->disk_frames=0;
  cfile->arps=cfile->arate=0;
  cfile->pb_fps=cfile->fps=prefs->default_fps;

  cfile->hsize=inst->out_channels[0].width;
  cfile->vsize=inst->out_channels[0].height;

  cfile->opening=FALSE;
  cfile->proc_ptr=NULL;

  cfile->changed=FALSE;

  // allow clip switching
  cfile->is_loaded=TRUE;

  g_snprintf(cfile->type,40,"generator:%s",inst->class->name);

  // if not playing, start playing
  if (mainw->playing_file==-1) {
    if (!mainw->num_tr_applied||old_file==-1) {
      switch_to_file((mainw->current_file=old_file),new_file);
      set_main_title(cfile->file_name,0);
      mainw->play_start=1;
      mainw->play_end=INT_MAX;
      if (mainw->num_tr_applied) mainw->blend_file=mainw->current_file;
    }
    else {
      mainw->current_file=old_file;
      mainw->blend_file=new_file;
      mainw->play_start=cfile->start;
      mainw->play_end=cfile->end;
      if (mainw->rte&(1<<bg_generator_key)) mainw->rte^=(1<<bg_generator_key);

      g_print ("pt zulu %d\n",mainw->rte);

      // TODO - allow switch of bg_generator mode
      if (mainw->last_grabable_effect==bg_generator_key+1) mainw->last_grabable_effect=EFFECT_NONE;
    }
    if (mainw->play_window!=NULL&&old_file==-1) {
      // usually preview or load_preview_frame would do this
      g_signal_handler_block(mainw->play_window,mainw->pw_exp_func);
    }

    play_file();

    inst=cfile->ext_src; // user may have switched generators during pb

    if (mainw->blend_file==-1||mainw->blend_file==mainw->current_file||mainw->files[mainw->blend_file]->ext_src!=inst) {
      if (fg_generator_key!=-1) {
	g_print ("ok end!!\n");
	livido_generator_end (inst);
      }
    }
    if (mainw->pre_src_file!=-1) {
      // switch back to where we started
      switch_to_file ((mainw->current_file=0),mainw->pre_src_file);
      mainw->pre_src_file=-1;
    }
  }
  else {
    mainw->current_file=old_file;
    if (!mainw->num_tr_applied) {
      if (mainw->current_file==-1) mainw->current_file=new_file;
      g_print ("dqs %d %d\n",mainw->current_file,new_file);
      do_quick_switch (new_file);
    }
    else {
      mainw->blend_file=new_file;
      if (mainw->rte&(1<<bg_generator_key)) mainw->rte^=(1<<bg_generator_key);
    }
  }
  return TRUE;
}



gboolean livido_init_effect(int hotkey, gint rowstride) {
  livido_class_t *fx;
  livido_instance_t *instance;
  livido_channel_t *inc,*outc;
  gint num_tr_applied=mainw->num_tr_applied;

  if (hotkey<0) {
    hotkey=-hotkey;
    mainw->num_tr_applied=0;
  }

  if (key_to_fx[hotkey][key_modes[hotkey]]==-1) {
    mainw->num_tr_applied=num_tr_applied;
    return FALSE;
  }

  g_print ("init %d %d\n",hotkey,mainw->rte);

  if (next_free_instance==-1) {
    d_print(_ ("Warning - no more effect slots left !\n"));
    mainw->num_tr_applied=num_tr_applied;
    return FALSE;
  }

  if (my_in_channels[key_to_fx[hotkey][key_modes[hotkey]]]->palettes==NULL) {
    // starting a new generator, we must stop the old one
    // warning = in future, bg *and* fg generators could change
    if (mainw->num_tr_applied&&mainw->blend_file!=-1&&mainw->blend_file!=mainw->current_file&&mainw->files[mainw->blend_file]->clip_type==CLIP_TYPE_GENERATOR) {
      livido_generator_end (mainw->files[mainw->blend_file]->ext_src);
      bg_generator_key=-1;
    }
    else if (mainw->current_file>0&&cfile->clip_type==CLIP_TYPE_GENERATOR) {
      g_print ("ending %d\n",fg_generator_key);
      livido_generator_end (cfile->ext_src);
      fg_generator_key=-1;
    }
  }
    
  fx=livido_fx[key_to_fx[hotkey][key_modes[hotkey]]];

  mainw->last_grabable_effect=hotkey;

  inc=my_in_channels[key_to_fx[hotkey][key_modes[hotkey]]];
  outc=my_out_channels[key_to_fx[hotkey][key_modes[hotkey]]];

  if (inc[0].palettes!=NULL) {
    if (inc[1].palettes!=NULL) {
      num_tr_applied++; // increase trans count
      if (num_tr_applied==1) {
	mainw->blend_file=mainw->current_file;
	mainw->blend_file_step=(mainw->current_file>-1&&cfile->pb_fps<0)?-1:1;
      }
      if (mainw->current_file>-1) {
	inc[1].width=cfile->hsize;
	inc[1].height=cfile->vsize;
	inc[1].rowstrides[0]=3*cfile->hsize; // guess
      }
      else {
	inc[1].width=DEFAULT_CHANNEL_HSIZE;
	inc[1].height=DEFAULT_CHANNEL_VSIZE;
	inc[1].rowstrides[0]=DEFAULT_CHANNEL_HSIZE*3;
      }
    }
    
    if (mainw->current_file>-1) {
      inc[0].width=cfile->hsize;
      inc[0].height=cfile->vsize;
      inc[0].rowstrides[0]=rowstride;
    }
    else {
      inc[0].width=DEFAULT_CHANNEL_HSIZE;
      inc[0].height=DEFAULT_CHANNEL_VSIZE;
      inc[0].rowstrides[0]=DEFAULT_CHANNEL_HSIZE*3;
    }
      
    if (mainw->current_file>-1) {
      outc[0].width=cfile->hsize;
      outc[0].height=cfile->vsize;
      outc[0].rowstrides[0]=cfile->hsize*3;
    }
    else {
      outc[0].width=DEFAULT_CHANNEL_HSIZE;
      outc[0].height=DEFAULT_CHANNEL_VSIZE;
      outc[0].rowstrides[0]=DEFAULT_CHANNEL_HSIZE*3;
    }
  }
  else {
    // aha - a generator
    outc[0].width=DEF_GEN_WIDTH;
    outc[0].height=DEF_GEN_HEIGHT;
    outc[0].rowstrides[0]=outc[0].width*3;

    // TODO 

  }
  
  LIVIDO_CREATE_INSTANCE(fx,instance,inc,outc);

  livido_instance[next_free_instance]=instance;
  key_to_instance[hotkey][key_modes[hotkey]]=next_free_instance;
  
  instance=(*fx->init)(instance);
  next_free_instance=get_next_free_instance();

  if (inc[0].palettes==NULL) {
    // generator start
    if (mainw->num_tr_applied) {
      // transition is on, make into bg clip
      bg_generator_key=hotkey;
    }
    else {
      // no transition, make fg (or kb was grabbed for fg generator)
      fg_generator_key=hotkey;
    }
    // start the generator and start playing
    if (!livido_generator_start(instance)) {
      d_print (g_strdup_printf (_ ("Failed to start generator %s\n"),instance->class->name));
    if (mainw->num_tr_applied) {
      bg_generator_key=-1;
    }
    else {
      fg_generator_key=-1;
    }
    mainw->num_tr_applied=num_tr_applied;
    return FALSE;
    }
  }
  mainw->num_tr_applied=num_tr_applied;
  return TRUE;
}

#define KEYSCALE 256


// TODO - fix crash when min==max

gint livido_get_blend_factor(int hotkey) {

  livido_instance_t *instance=livido_instance[key_to_instance[hotkey][key_modes[hotkey]]];

  // TODO - check why this happens from init_resize
  if (instance==NULL) return 0;

  livido_class_t *fx=instance->class;
  if (fx->parameters==NULL||mainw->files[mainw->blend_file]==NULL) return 0;
  switch (fx->parameters[0].type) {
  case LVD_PARAM_NUMBER:
    if (fx->parameters[0].decimals) {
      gfloat val=get_float_param(instance->parameter[0]);
      return (gint)((val-fx->parameters[0].min)*KEYSCALE/(fx->parameters[0].max-fx->parameters[0].min)+.5);
    }
    else {
      gint val=get_int_param(instance->parameter[0]);
      return (gint)((val-fx->parameters[0].min)*KEYSCALE/(fx->parameters[0].max-fx->parameters[0].min)+.5);
    }
  default:
    return 0;
  }
}


void livido_set_blend_factor(int hotkey) {
  livido_instance_t *instance=livido_instance[key_to_instance[hotkey][key_modes[hotkey]]];
  livido_class_t *fx=instance->class;
  if (fx->parameters==NULL||mainw->files[mainw->blend_file]==NULL) return;
  switch (fx->parameters[0].type) {
  case LVD_PARAM_NUMBER:
    if (fx->parameters[0].decimals) {
      set_float_param(instance->parameter[0],(gfloat)(fx->parameters[0].min+((gfloat)mainw->blend_factor/KEYSCALE*(fx->parameters[0].max-fx->parameters[0].min))));
    }
    else {
      set_int_param(instance->parameter[0],(gint)(fx->parameters[0].min+((gfloat)mainw->blend_factor/KEYSCALE*(fx->parameters[0].max-fx->parameters[0].min))+.5));
    }
    break;
  }
}

void livido_deinit_effect(int hotkey) {
  livido_class_t *fx;
  livido_instance_t *instance;

  if (hotkey>=1<<(num_livido_plugins-1)) return;

  g_print ("deinit %d\n",hotkey);

  fx=livido_fx[key_to_fx[hotkey][key_modes[hotkey]]];
  instance=livido_instance[key_to_instance[hotkey][key_modes[hotkey]]];
  if (my_in_channels[key_to_fx[hotkey][key_modes[hotkey]]][0].palettes==NULL) {
    // is generator
    if (mainw->playing_file>-1) {
      mainw->cancelled=1;
    }
    else livido_generator_end (instance);
    return;
  }
  else (*fx->deinit)(instance);

  // TODO - use instance->class
  LIVIDO_FREE_INSTANCE(fx,instance);
  instance=NULL;
  livido_instance[key_to_instance[hotkey][key_modes[hotkey]]]=NULL;
  if (my_in_channels[key_to_fx[hotkey][key_modes[hotkey]]][0].palettes!=NULL&&my_in_channels[key_to_fx[hotkey][key_modes[hotkey]]][1].palettes!=NULL) {
    g_print ("deinit trans\n");
    if (!(--mainw->num_tr_applied)) {
      if (mainw->blend_file!=-1&&mainw->blend_file!=mainw->current_file&&mainw->files[mainw->blend_file]->clip_type==CLIP_TYPE_GENERATOR) {
	livido_generator_end (mainw->files[mainw->blend_file]->ext_src);
	g_print ("subs end\n");
      }
      mainw->blend_file=-1;
    }
  }
}


void **store_livido_param_values (gint i) {
  // TODO - eventually use store_rfx_param_values, etc.
  // requires stabilisation of livido parameters
  return NULL;
}


void set_livido_params_from_store (gint i, void **store) {
}



void livido_params_store_free (gint i, void **store) {
}


void livido_init_resize(gint rowstride) {
  // this is called a) at the start of playback, and b) when the frame size of
  // the current file changes
  void **param_store=NULL;
  int i;
  gint num_tr_applied=mainw->num_tr_applied;
  gint blend_file=mainw->blend_file;
  if (num_tr_applied==1) mainw->num_tr_applied=2; // nasty things happen
  // otherwise...
  for (i=0;i<FX_KEYS-1;i++) {
    if (mainw->rte&(1<<i)) {
      // re-init only active effects
      if (i<num_livido_plugins&&my_in_channels[key_to_fx[i][key_modes[i]]][0].palettes!=NULL) {
	// and don't re-init generators
	param_store=store_livido_param_values (i);
	livido_deinit_effect(i);
	// reset all channel sizes
	livido_init_effect(i,rowstride);
	set_livido_params_from_store (i,param_store);
	// TODO - remove next line ?
	if (i==mainw->rte_keys-1) livido_set_blend_factor(i);
	livido_params_store_free (i,param_store);
	g_free (param_store);
      }
    }
  }
  mainw->num_tr_applied=num_tr_applied;
  mainw->blend_file=blend_file;
  if (mainw->blend_file!=-1) mainw->blend_factor=livido_get_blend_factor(mainw->rte_keys);
}




void livido_apply_effects(int height, int rowstride, guchar **in_pixel_data, livido_palette_t in_palette) {
  // for now, the in_palette will always come in as RGB24LE, this can be changed back and forth, but the final output should be RGB24LE again
  
  // there could be a small optimisation, if mainw->ext_playback is TRUE
  // then the final output could be left as mainw->ext_palette
  // sometimes this can be UYVY

  // palettes need converting

  int i;
  guchar *out_pixel_data=g_malloc(height*rowstride);

  livido_class_t *fx;
  livido_instance_t *instance;

  livido_timecode_t tc=LIVIDO_TIMECODE_UNDEFINED;

  livido_frame_t *ifp[2];
  livido_frame_t *ofp[1];

  livido_frame_t output_frame[]={{tc,0.0,0.0,out_pixel_data,NULL,NULL}};

  gint in_channs=1;
  livido_frame_t *bf_data=NULL;

  ifp[0]=(livido_frame_t *)g_malloc(sizeof(livido_frame_t));
  ifp[0]->timecode=tc;
  ifp[0]->x=0.;
  ifp[0]->y=0.;
  ifp[0]->pixel_data=in_pixel_data[0];
  ifp[0]->audio_data=NULL;
  ifp[0]->extra=NULL;

  if (in_pixel_data[1]!=NULL) {
    // transition
    ifp[1]=(livido_frame_t *)g_malloc(sizeof(livido_frame_t));
    ifp[1]->timecode=tc;
    ifp[1]->x=0.;
    ifp[1]->y=0.;
    ifp[1]->pixel_data=in_pixel_data[1];
    ifp[1]->audio_data=NULL;
    ifp[1]->extra=NULL;
    in_channs=2;
  }
  // plugin should know how many channels, thus how many elements in arrays

  // use inplace if we can

  for (i=0;i<FX_KEYS-1;i++) {
    if (mainw->rte&(1<<i)&&my_in_channels[key_to_fx[i][key_modes[i]]][0].palettes!=NULL) {
      // ignore generators
      fx=livido_fx[key_to_fx[i][key_modes[i]]];
      instance=livido_instance[key_to_instance[i][key_modes[i]]];

      if (my_in_channels[key_to_fx[i][key_modes[i]]][1].palettes==NULL) {
	ofp[0]=NULL;
	// single channel plugin
	if (!(fx->properties&LVD_PROPERTY_INPLACE)) {

	  // TODO ** fix !!
	  ofp[0]=output_frame;
	}
	if (in_channs==2) {
	  // save ref to bg frame
	  bf_data=ifp[1];
	}
	ifp[1]=NULL;
	(*fx->process)(instance,ifp,ofp);
	if (!(fx->properties&LVD_PROPERTY_INPLACE)) {
	  memcpy(in_pixel_data,out_pixel_data,height*rowstride);
	}
	if (in_channs==2) {
	  ifp[1]=bf_data;
	}
      }
      else {
	ofp[0]=NULL;
	// two channel in effect
	if (in_channs==2) {
	  if (!(fx->properties&LVD_PROPERTY_INPLACE)) {

	    // TODO - ** fix !!
	    ofp[0]=output_frame; // this is our output
	  }
	  (*fx->process)(instance,ifp,ofp);
	  if (!(fx->properties&LVD_PROPERTY_INPLACE)) {
	    memcpy(in_pixel_data,out_pixel_data,height*rowstride);
	  }
	}
      }
    }
  }
  g_free (out_pixel_data);

  // note: ->pixel_data is not freed here, should be ref'ed elsewhere
  g_free(ifp[0]);
  if (in_pixel_data[1]!=NULL) {
    g_free(ifp[1]);
  }
}



void livido_deinit_all(void) {
  // deinit all except generators
  // this is called on ctrl-0 or on shutdown
  int i;
  g_print ("in deinit all %d\n",mainw->rte);
  for (i=0;i<FX_KEYS-1;i++) {
    if ((mainw->rte&(1<<i))&&my_in_channels[key_to_fx[i][key_modes[i]]][0].palettes!=NULL) {
      g_print ("in deinit %d\n",i);
      livido_deinit_effect(i);
      mainw->rte^=(1<<i);
    }
  }
}

void livido_end(void) {
  int i;
  mainw->num_tr_applied=0; // don't bother with all that
  livido_deinit_all();
  for (i=0;i<num_livido_plugins-num_modes;i++) {
    dlclose(livido_handles[i]);
  }
}


void livido_key_nextmode (void) {
  gint newmode,oldmode=key_modes[mainw->rte_keys];
  gint blend_file;

  // TODO - handle case where generator is switched off

  if (oldmode==MAX_MODES_PER_KEY||key_to_fx[mainw->rte_keys][oldmode+1]==-1) {
    newmode=0;
  }
  else {
    newmode=key_modes[mainw->rte_keys]+1;
  }


  // we will do some special cheats with generators, to make them more sticky
  if (my_in_channels[key_to_fx[mainw->rte_keys][oldmode]][0].palettes!=NULL&&my_in_channels[key_to_fx[mainw->rte_keys][oldmode]][1].palettes!=NULL&&my_in_channels[key_to_fx[mainw->rte_keys][newmode]][0].palettes!=NULL&&my_in_channels[key_to_fx[mainw->rte_keys][newmode]][1].palettes!=NULL) {
    // transition --> transition, allow any bg generator to survive
    mainw->num_tr_applied++;
  }

  if (oldmode!=newmode) {
    blend_file=mainw->blend_file;
    if (my_in_channels[key_to_fx[mainw->rte_keys][oldmode]][0].palettes!=NULL) {
      livido_deinit_effect (mainw->rte_keys);
    }
    key_modes[mainw->rte_keys]=newmode;
    livido_init_effect(-mainw->rte_keys,mainw->frame_image==NULL?0:gdk_pixbuf_get_rowstride(mainw->frame_image));
    mainw->blend_file=blend_file;

    if (mainw->blend_file!=-1) mainw->blend_factor=livido_get_blend_factor(mainw->rte_keys);
  }
  if (my_in_channels[key_to_fx[mainw->rte_keys][oldmode]][0].palettes!=NULL&&my_in_channels[key_to_fx[mainw->rte_keys][oldmode]][1].palettes!=NULL&&my_in_channels[key_to_fx[mainw->rte_keys][newmode]][0].palettes!=NULL&&my_in_channels[key_to_fx[mainw->rte_keys][newmode]][1].palettes!=NULL) {
    // transition --> transition, allow any bg generator to survive
    mainw->num_tr_applied--;
  }
}


void livido_swap_fg_bg (void) {
  gint key=fg_generator_key;
  if (key!=-1) {
    if (mainw->rte&(1<<key)) mainw->rte^=(1<<key);
  }
  fg_generator_key=bg_generator_key;
  if (fg_generator_key!=-1) {
    if (!(mainw->rte&(1<<fg_generator_key))) mainw->rte^=(1<<fg_generator_key);
  }
  bg_generator_key=key;
}



#endif
