// interface.c
// LiVES
// (c) G. Finch 2003 <salsaman@xs4all.nl>
// Released under the GPL 2.0
// see file ../COPYING for licensing details

// TODO - use gtk_widget_showall where poss.
// and don't forget gtk_box_pack_end (doh)
// and just use label instead of labelnn, etc.

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include "main.h"
#include "callbacks.h"
#include "interface.h"
#include "merge.h"
#include "paramwindow.h"
#include "support.h"




GtkWidget*
create_fileselection1 (const gchar *title, gint preview_type)
{
  GtkWidget *fileselection1;
  GtkWidget *ok_button1;
  GtkWidget *cancel_button1;

  gchar title1[256];
  g_snprintf(title1,256,"LiVES: - %s",title);

  fileselection1 = gtk_file_selection_new (title1);
  gtk_widget_modify_bg(fileselection1, GTK_STATE_NORMAL, &palette->normal_back);

  gtk_container_set_border_width (GTK_CONTAINER (fileselection1), 10);
  gtk_window_set_position (GTK_WINDOW (fileselection1), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (fileselection1), TRUE);

  gtk_window_set_transient_for(GTK_WINDOW(fileselection1),GTK_WINDOW(mainw->LiVES));

  if (prefs->fileselmax==TRUE) {
    gtk_window_maximize (GTK_WINDOW(fileselection1));
  }

  ok_button1 = GTK_FILE_SELECTION (fileselection1)->ok_button;
  gtk_widget_show (ok_button1);
  GTK_WIDGET_SET_FLAGS (ok_button1, GTK_CAN_DEFAULT);

  cancel_button1 = GTK_FILE_SELECTION (fileselection1)->cancel_button;
  gtk_widget_show (cancel_button1);
  GTK_WIDGET_SET_FLAGS (cancel_button1, GTK_CAN_DEFAULT);

  gtk_widget_grab_focus (fileselection1);
  
  if (preview_type>0) widget_add_preview(GTK_BOX (GTK_FILE_SELECTION(fileselection1)->main_vbox),GTK_BOX (GTK_FILE_SELECTION (fileselection1)->action_area),preview_type);

  g_signal_connect (GTK_FILE_SELECTION(fileselection1)->cancel_button, "clicked",G_CALLBACK (on_cancel_button1_clicked),NULL);
  return fileselection1;
}


void
widget_add_preview(GtkBox *for_preview, GtkBox *for_button, gint preview_type) {
  // preview type 1 - video and audio, fileselector
  // preview type 2 - audio only, fileselector
  // preview type 3 - a/v, open selection

  GtkWidget *preview_button;
  GtkWidget *fs_label;

  mainw->fs_playframe = gtk_frame_new (NULL);
  mainw->fs_playarea = gtk_socket_new ();

  if (preview_type==1||preview_type==3) {

    gtk_widget_show (mainw->fs_playframe);

    gtk_container_set_border_width (GTK_CONTAINER(mainw->fs_playframe), 10);
    gtk_frame_set_shadow_type (GTK_FRAME(mainw->fs_playframe), GTK_SHADOW_IN);
    gtk_widget_modify_bg (mainw->fs_playframe, GTK_STATE_NORMAL, &palette->normal_back);
    
    fs_label = gtk_label_new (_ ("Preview"));
    gtk_widget_show (fs_label);
    gtk_frame_set_label_widget (GTK_FRAME (mainw->fs_playframe), fs_label);
    gtk_label_set_justify (GTK_LABEL (fs_label), GTK_JUSTIFY_RIGHT);

    if (palette->style&STYLE_1) {
      gtk_widget_modify_fg(fs_label, GTK_STATE_NORMAL, &palette->normal_fore);
    }
    
    gtk_box_pack_start (for_preview, mainw->fs_playframe, FALSE, FALSE, TRUE);
    gtk_widget_set_size_request (mainw->fs_playarea, DEFAULT_FRAME_HSIZE, DEFAULT_FRAME_VSIZE);

    gtk_container_add (GTK_CONTAINER (mainw->fs_playframe), mainw->fs_playarea);

    gtk_widget_modify_bg (mainw->fs_playarea, GTK_STATE_NORMAL, &palette->normal_back);
    gtk_widget_show(mainw->fs_playarea);
   }

  preview_button = gtk_button_new_with_mnemonic (_ ("_Preview"));
  gtk_widget_show (preview_button);

  if (palette->style&STYLE_1) {
    gtk_widget_modify_bg(preview_button, GTK_STATE_NORMAL, &palette->menu_and_bars);
  }

  gtk_box_pack_start (for_button, preview_button, TRUE, TRUE, 0);

  g_signal_connect (GTK_OBJECT (preview_button), "clicked",
		    G_CALLBACK (on_fs_preview_clicked),
		    GINT_TO_POINTER (preview_type));

}




process* 
create_processing (const gchar *text)
{

  GtkWidget *dialog_vbox1;
  GtkWidget *vbox2;
  GtkWidget *vbox3;
  GtkWidget *dialog_action_area1;
  process *procw=(process*)(g_malloc(sizeof(process)));
  gchar tmp_label[256];

  procw->processing = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER (procw->processing), 10);
  gtk_window_add_accel_group (GTK_WINDOW (procw->processing), mainw->accel_group);

  gtk_widget_set_size_request (procw->processing, 320, 185);
  gtk_window_set_resizable (GTK_WINDOW (procw->processing), FALSE);
  
  gtk_widget_modify_bg(procw->processing, GTK_STATE_NORMAL, &palette->normal_back);
  gtk_window_set_title (GTK_WINDOW (procw->processing), _("LiVES: - Processing..."));
  gtk_window_set_transient_for(GTK_WINDOW(procw->processing),GTK_WINDOW(mainw->LiVES));
  gtk_window_set_position (GTK_WINDOW (procw->processing), GTK_WIN_POS_CENTER);

  dialog_vbox1 = GTK_DIALOG (procw->processing)->vbox;
  gtk_widget_show (dialog_vbox1);

  gtk_dialog_set_has_separator(GTK_DIALOG(procw->processing),FALSE);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox2);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox2, TRUE, TRUE, 0);

  vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox3);
  gtk_box_pack_start (GTK_BOX (vbox2), vbox3, TRUE, TRUE, 0);

  g_snprintf(tmp_label,256,"%s...",text);
  procw->label = gtk_label_new (tmp_label);
  gtk_widget_show (procw->label);
  gtk_box_pack_start (GTK_BOX (vbox3), procw->label, FALSE, FALSE, 0);
  gtk_widget_modify_fg(procw->label, GTK_STATE_NORMAL, &palette->normal_fore);
  gtk_label_set_justify (GTK_LABEL (procw->label), GTK_JUSTIFY_LEFT);

  procw->progressbar = gtk_progress_bar_new ();
  gtk_widget_show (procw->progressbar);
  gtk_box_pack_start (GTK_BOX (vbox3), procw->progressbar, FALSE, FALSE, 0);
  gtk_widget_modify_fg(procw->progressbar, GTK_STATE_NORMAL, &palette->normal_fore);

  if (mainw->internal_messaging) procw->label2 = gtk_label_new (_("\n\nPlease Wait\n\nRemember to switch off effects (ctrl-0) afterwards !"));
  else procw->label2 = gtk_label_new (_("\nPlease Wait"));
  gtk_widget_show (procw->label2);
  gtk_box_pack_start (GTK_BOX (vbox3), procw->label2, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (procw->label2), GTK_JUSTIFY_CENTER);
  gtk_widget_modify_fg(procw->label2, GTK_STATE_NORMAL, &palette->normal_fore);


  dialog_action_area1 = GTK_DIALOG (procw->processing)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  procw->stop_button = gtk_button_new_with_mnemonic (_ ("_Enough"));
  procw->preview_button = gtk_button_new_with_mnemonic (_ ("_Preview"));
  procw->pause_button = gtk_button_new_with_mnemonic (_ ("Pau_se"));

  gtk_dialog_add_action_widget (GTK_DIALOG (procw->processing), procw->preview_button, 1);
  gtk_widget_hide(procw->preview_button);
  GTK_WIDGET_SET_FLAGS (procw->preview_button, GTK_CAN_DEFAULT);
    
    gtk_dialog_add_action_widget (GTK_DIALOG (procw->processing), procw->pause_button, 0);
    gtk_widget_hide(procw->pause_button);
    GTK_WIDGET_SET_FLAGS (procw->pause_button, GTK_CAN_DEFAULT);


  if (cfile->opening==TRUE&&(cfile->frames>0||cfile->opening_loc==TRUE)) {
  // the "enough" button for opening
   gtk_dialog_add_action_widget (GTK_DIALOG (procw->processing), procw->stop_button, 0);
   gtk_widget_show(procw->stop_button);
   GTK_WIDGET_SET_FLAGS (procw->stop_button, GTK_CAN_DEFAULT);
  }
  
  procw->cancel_button = gtk_button_new_with_mnemonic (_ ("_Cancel"));
  gtk_dialog_add_action_widget (GTK_DIALOG (procw->processing), procw->cancel_button, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (procw->cancel_button, GTK_CAN_DEFAULT);

  gtk_widget_add_accelerator (procw->cancel_button, "activate", mainw->accel_group,
                              GDK_Escape, 0, 0);

  g_signal_connect (GTK_OBJECT (procw->stop_button), "clicked",
		    G_CALLBACK (on_stop_clicked),
		    NULL);

  g_signal_connect (GTK_OBJECT (procw->pause_button), "clicked",
                      G_CALLBACK (on_effects_paused),
                      NULL);

  g_signal_connect (GTK_OBJECT (procw->preview_button), "clicked",
                      G_CALLBACK (on_preview_clicked),
                      NULL);


  g_signal_connect (GTK_OBJECT (procw->cancel_button), "clicked",
                      G_CALLBACK (on_button3_clicked),
                      NULL);

  g_signal_connect (GTK_OBJECT (procw->processing), "delete_event",
                      G_CALLBACK (return_true),
                      NULL);

  gtk_widget_grab_default (procw->preview_button);

  return procw;
}



fileinfo*
create_info_window (gint audio_channels)
{
 

  GtkWidget *vbox5;
  GtkWidget *frame4;
  GtkWidget *fixed3;
  GtkWidget *fixed5;
  GtkWidget *fixed6;
  GtkWidget *label45;
  GtkWidget *label46;
  GtkWidget *label47;
  GtkWidget *label48;
  GtkWidget *label44;
  GtkWidget *label43;
  GtkWidget *label40;
  GtkWidget *label50;
  GtkWidget *label51;
  GtkWidget *label52;
  GtkWidget *label53;
  GtkWidget *frame5;
  GtkWidget *left;
  GtkWidget *frame6;
  GtkWidget *right;
  GtkWidget *button8;

  fileinfo *filew=(fileinfo*)(g_malloc(sizeof(fileinfo)));

  filew->info_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_position (GTK_WINDOW (filew->info_window), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (filew->info_window), TRUE);

  gtk_widget_modify_bg(filew->info_window, GTK_STATE_NORMAL, &palette->normal_back);

  vbox5 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox5);
  gtk_container_add (GTK_CONTAINER (filew->info_window), vbox5);

  if (mainw->fs==TRUE&&mainw->sep_win==TRUE&&mainw->playing_file>-1) {
    gtk_window_set_transient_for(GTK_WINDOW(filew->info_window),GTK_WINDOW(mainw->play_window));
  }
  else {
    gtk_window_set_transient_for(GTK_WINDOW(filew->info_window),GTK_WINDOW(mainw->LiVES));
  }

  if (cfile->frames>0) {
    frame4 = gtk_frame_new (NULL);
    gtk_widget_set_size_request (frame4, 460, 340);
    gtk_widget_show (frame4);
    gtk_box_pack_start (GTK_BOX (vbox5), frame4, TRUE, TRUE, 0);
    gtk_widget_modify_bg(frame4, GTK_STATE_NORMAL, &palette->normal_back);
    
    fixed3 = gtk_fixed_new ();
    gtk_widget_show (fixed3);
    gtk_container_add (GTK_CONTAINER (frame4), fixed3);
    
    filew->textview24 = gtk_text_view_new ();
    gtk_widget_show (filew->textview24);
    gtk_fixed_put (GTK_FIXED (fixed3), filew->textview24, 96, 48);
    gtk_widget_set_size_request (filew->textview24, 120, 50);
    gtk_text_view_set_editable (GTK_TEXT_VIEW (filew->textview24), FALSE);
    gtk_text_view_set_justification (GTK_TEXT_VIEW (filew->textview24), GTK_JUSTIFY_CENTER);
    gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (filew->textview24), FALSE);
    
    filew->textview25 = gtk_text_view_new ();
    gtk_widget_show (filew->textview25);
    gtk_fixed_put (GTK_FIXED (fixed3), filew->textview25, 288, 48);
    gtk_widget_set_size_request (filew->textview25, 120, 50);
    gtk_text_view_set_editable (GTK_TEXT_VIEW (filew->textview25), FALSE);
    gtk_text_view_set_justification (GTK_TEXT_VIEW (filew->textview25), GTK_JUSTIFY_CENTER);
    gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (filew->textview25), FALSE);
    
    filew->textview26 = gtk_text_view_new ();
    gtk_widget_show (filew->textview26);
    gtk_fixed_put (GTK_FIXED (fixed3), filew->textview26, 96, 136);
    gtk_widget_set_size_request (filew->textview26, 120, 50);
    gtk_text_view_set_editable (GTK_TEXT_VIEW (filew->textview26), FALSE);
    gtk_text_view_set_justification (GTK_TEXT_VIEW (filew->textview26), GTK_JUSTIFY_CENTER);
    gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (filew->textview26), FALSE);
    
    filew->textview27 = gtk_text_view_new ();
    gtk_widget_show (filew->textview27);
    gtk_fixed_put (GTK_FIXED (fixed3), filew->textview27, 288, 136);
    gtk_widget_set_size_request (filew->textview27, 120, 50);
    gtk_text_view_set_editable (GTK_TEXT_VIEW (filew->textview27), FALSE);
    gtk_text_view_set_justification (GTK_TEXT_VIEW (filew->textview27), GTK_JUSTIFY_CENTER);
    gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (filew->textview27), FALSE);
    
    filew->textview28 = gtk_text_view_new ();
    gtk_widget_show (filew->textview28);
    gtk_fixed_put (GTK_FIXED (fixed3), filew->textview28, 288, 224);
    gtk_widget_set_size_request (filew->textview28, 120, 50);
    gtk_text_view_set_editable (GTK_TEXT_VIEW (filew->textview28), FALSE);
    gtk_text_view_set_justification (GTK_TEXT_VIEW (filew->textview28), GTK_JUSTIFY_CENTER);
    gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (filew->textview28), FALSE);
    
    filew->textview29 = gtk_text_view_new ();
    gtk_widget_show (filew->textview29);
    gtk_fixed_put (GTK_FIXED (fixed3), filew->textview29, 96, 224);
    gtk_widget_set_size_request (filew->textview29, 120, 50);
    gtk_text_view_set_editable (GTK_TEXT_VIEW (filew->textview29), FALSE);
    gtk_text_view_set_justification (GTK_TEXT_VIEW (filew->textview29), GTK_JUSTIFY_CENTER);
    gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (filew->textview29), FALSE);
    
    label45 = gtk_label_new (_("Format"));
    gtk_widget_show (label45);
    gtk_fixed_put (GTK_FIXED (fixed3), label45, 44, 64);
    gtk_widget_set_size_request (label45, 45, 16);
    gtk_label_set_justify (GTK_LABEL (label45), GTK_JUSTIFY_LEFT);
    gtk_widget_modify_fg(label45, GTK_STATE_NORMAL, &palette->normal_fore);
    
    label46 = gtk_label_new (_("Size"));
    gtk_widget_show (label46);
    gtk_fixed_put (GTK_FIXED (fixed3), label46, 52, 152);
    gtk_widget_set_size_request (label46, 39, 16);
    gtk_label_set_justify (GTK_LABEL (label46), GTK_JUSTIFY_LEFT);
    gtk_widget_modify_fg(label46, GTK_STATE_NORMAL, &palette->normal_fore);
    
    label47 = gtk_label_new (_("File Size"));
    gtk_widget_show (label47);
    gtk_fixed_put (GTK_FIXED (fixed3), label47, 28, 240);
    gtk_widget_set_size_request (label47, 65, 16);
    gtk_label_set_justify (GTK_LABEL (label47), GTK_JUSTIFY_LEFT);
    gtk_widget_modify_fg(label47, GTK_STATE_NORMAL, &palette->normal_fore);
    
    label48 = gtk_label_new (_("Total time"));
    gtk_widget_show (label48);
    gtk_fixed_put (GTK_FIXED (fixed3), label48, 220, 240);
    gtk_widget_set_size_request (label48, 63, 16);
    gtk_label_set_justify (GTK_LABEL (label48), GTK_JUSTIFY_LEFT);
    gtk_widget_modify_fg(label48, GTK_STATE_NORMAL, &palette->normal_fore);
    
    label44 = gtk_label_new (_("FPS"));
    gtk_widget_show (label44);
    gtk_fixed_put (GTK_FIXED (fixed3), label44, 246, 64);
    gtk_widget_set_size_request (label44, 39, 16);
    gtk_label_set_justify (GTK_LABEL (label44), GTK_JUSTIFY_LEFT);
    gtk_widget_modify_fg(label44, GTK_STATE_NORMAL, &palette->normal_fore);
    
    label43 = gtk_label_new (_("Frames"));
    gtk_widget_show (label43);
    gtk_fixed_put (GTK_FIXED (fixed3), label43, 236, 152);
    gtk_widget_set_size_request (label43, 45, 16);
    gtk_label_set_justify (GTK_LABEL (label43), GTK_JUSTIFY_LEFT);
    gtk_widget_modify_fg(label43, GTK_STATE_NORMAL, &palette->normal_fore);
    
    label40 = gtk_label_new (_("Video"));
    gtk_widget_show (label40);
    gtk_frame_set_label_widget (GTK_FRAME (frame4), label40);
    gtk_label_set_justify (GTK_LABEL (label40), GTK_JUSTIFY_LEFT);
    gtk_widget_modify_fg(label40, GTK_STATE_NORMAL, &palette->normal_fore);
  }

  if (audio_channels>0) {
    frame5 = gtk_frame_new (NULL);
    gtk_widget_set_size_request (frame5, 460, 100);
    gtk_widget_show (frame5);
    gtk_box_pack_start (GTK_BOX (vbox5), frame5, TRUE, TRUE, 0);
    gtk_widget_modify_bg(frame5, GTK_STATE_NORMAL, &palette->normal_back);

    if (audio_channels>1) {
      left = gtk_label_new (_("Left Audio"));
    }
    else {
      left = gtk_label_new (_("Audio"));
    }

    gtk_widget_show (left);
    gtk_frame_set_label_widget (GTK_FRAME (frame5), left);
    gtk_label_set_justify (GTK_LABEL (left), GTK_JUSTIFY_LEFT);
    gtk_widget_modify_fg(left, GTK_STATE_NORMAL, &palette->normal_fore);

    fixed5 = gtk_fixed_new ();
    gtk_widget_show (fixed5);
    gtk_container_add (GTK_CONTAINER (frame5), fixed5);


    filew->textview_ltime = gtk_text_view_new ();
    gtk_widget_show (filew->textview_ltime);
    gtk_fixed_put (GTK_FIXED (fixed5), filew->textview_ltime, 288, 16);
    gtk_widget_set_size_request (filew->textview_ltime, 120, 50);
    gtk_text_view_set_editable (GTK_TEXT_VIEW (filew->textview_ltime), FALSE);
    gtk_text_view_set_justification (GTK_TEXT_VIEW (filew->textview_ltime), GTK_JUSTIFY_CENTER);
    gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (filew->textview_ltime), FALSE);
    
    label50 = gtk_label_new (_("Total time"));
    gtk_widget_show (label50);
    gtk_fixed_put (GTK_FIXED (fixed5), label50, 220, 32);
    gtk_widget_set_size_request (label50, 63, 16);
    gtk_label_set_justify (GTK_LABEL (label50), GTK_JUSTIFY_LEFT);
    gtk_widget_modify_fg(label50, GTK_STATE_NORMAL, &palette->normal_fore);

    filew->textview_lrate = gtk_text_view_new ();
    gtk_widget_show (filew->textview_lrate);
    gtk_fixed_put (GTK_FIXED (fixed5), filew->textview_lrate, 96, 16);
    gtk_widget_set_size_request (filew->textview_lrate, 120, 50);
    gtk_text_view_set_editable (GTK_TEXT_VIEW (filew->textview_lrate), FALSE);
    gtk_text_view_set_justification (GTK_TEXT_VIEW (filew->textview_lrate), GTK_JUSTIFY_CENTER);
    gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (filew->textview_lrate), FALSE);
    
    label52 = gtk_label_new (_("Rate/size"));
    gtk_widget_show (label52);
    gtk_fixed_put (GTK_FIXED (fixed5), label52, 30, 32);
    gtk_widget_set_size_request (label52, 63, 16);
    gtk_label_set_justify (GTK_LABEL (label52), GTK_JUSTIFY_LEFT);
    gtk_widget_modify_fg(label52, GTK_STATE_NORMAL, &palette->normal_fore);


    if (audio_channels>1) {
      frame6 = gtk_frame_new (NULL);
      gtk_widget_set_size_request (frame6, 460, 100);
      gtk_widget_show (frame6);
      gtk_box_pack_start (GTK_BOX (vbox5), frame6, TRUE, TRUE, 0);
      gtk_widget_modify_bg(frame6, GTK_STATE_NORMAL, &palette->normal_back);

      fixed6 = gtk_fixed_new ();
      gtk_widget_show (fixed6);
      gtk_container_add (GTK_CONTAINER (frame6), fixed6);
      
      filew->textview_rtime = gtk_text_view_new ();
      gtk_widget_show (filew->textview_rtime);
      gtk_fixed_put (GTK_FIXED (fixed6), filew->textview_rtime, 288, 16);
      gtk_widget_set_size_request (filew->textview_rtime, 120, 50);
      gtk_text_view_set_editable (GTK_TEXT_VIEW (filew->textview_rtime), FALSE);
      gtk_text_view_set_justification (GTK_TEXT_VIEW (filew->textview_rtime), GTK_JUSTIFY_CENTER);
      gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (filew->textview_rtime), FALSE);
      
      label51 = gtk_label_new (_("Total time"));
      gtk_widget_show (label51);
      gtk_fixed_put (GTK_FIXED (fixed6), label51, 220, 32);
      gtk_widget_set_size_request (label51, 63, 16);
      gtk_label_set_justify (GTK_LABEL (label51), GTK_JUSTIFY_LEFT);
      gtk_widget_modify_fg(label51, GTK_STATE_NORMAL, &palette->normal_fore);
	  
      filew->textview_rrate = gtk_text_view_new ();
      gtk_widget_show (filew->textview_rrate);
      gtk_fixed_put (GTK_FIXED (fixed6), filew->textview_rrate, 96, 16);
      gtk_widget_set_size_request (filew->textview_rrate, 120, 50);
      gtk_text_view_set_editable (GTK_TEXT_VIEW (filew->textview_rrate), FALSE);
      gtk_text_view_set_justification (GTK_TEXT_VIEW (filew->textview_rrate), GTK_JUSTIFY_CENTER);
      gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (filew->textview_rrate), FALSE);
      
      label53 = gtk_label_new (_("Rate/size"));
      gtk_widget_show (label53);
      gtk_fixed_put (GTK_FIXED (fixed6), label53, 30, 32);
      gtk_widget_set_size_request (label53, 63, 16);
      gtk_label_set_justify (GTK_LABEL (label53), GTK_JUSTIFY_LEFT);
      gtk_widget_modify_fg(label53, GTK_STATE_NORMAL, &palette->normal_fore);

      
      right = gtk_label_new (_("Right audio"));
      gtk_widget_show (right);
      gtk_frame_set_label_widget (GTK_FRAME (frame6), right);
      gtk_label_set_justify (GTK_LABEL (right), GTK_JUSTIFY_LEFT);
      gtk_widget_modify_fg(right, GTK_STATE_NORMAL, &palette->normal_fore);
    }
  }

  button8 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (button8);
  gtk_box_pack_end (GTK_BOX (vbox5), button8, FALSE, FALSE, 0);
  gtk_widget_set_size_request (button8, -1, 78);
  gtk_container_set_border_width (GTK_CONTAINER (button8), 12);
  gtk_button_set_relief (GTK_BUTTON (button8), GTK_RELIEF_NONE);
  GTK_WIDGET_SET_FLAGS (button8, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (button8);

  if (palette->style&STYLE_1) {
    gtk_widget_modify_bg(button8, GTK_STATE_SELECTED|GTK_STATE_ACTIVE, &palette->white);
  }

  g_signal_connect (GTK_OBJECT (button8), "clicked",
                      G_CALLBACK (on_cancel_button1_clicked),
                      filew);

  g_signal_connect (GTK_OBJECT (filew->info_window), "delete_event",
                      G_CALLBACK (return_true),
                      NULL);

  return filew;
}






//Warning dialog
GtkWidget*
create_dialog2 (gint warn_mask_number)
{
  GtkWidget *dialog2;
  GtkWidget *dialog_vbox2;
  GtkWidget *dialog_action_area2;
  GtkWidget *warning_cancelbutton;
  GtkWidget *warning_okbutton;
  GtkWidget *checkbutton8;

  mainw->warn_mask=FALSE;

  dialog2 = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (dialog2), _("LiVES: - Warning !"));
  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(dialog2),FALSE);
  }
  gtk_widget_modify_bg(dialog2, GTK_STATE_NORMAL, &palette->normal_back);

  gtk_window_set_transient_for(GTK_WINDOW(dialog2),GTK_WINDOW(mainw->LiVES));

  gtk_window_set_modal (GTK_WINDOW (dialog2), TRUE);
  gtk_window_set_position (GTK_WINDOW (dialog2), GTK_WIN_POS_CENTER);
  gtk_window_set_default_size (GTK_WINDOW (dialog2), 350, 200);

  dialog_vbox2 = GTK_DIALOG (dialog2)->vbox;
  gtk_widget_show (dialog_vbox2);

  mainw->warning_label = gtk_label_new (_("warning"));
  gtk_widget_show (mainw->warning_label);
  gtk_box_pack_start (GTK_BOX (dialog_vbox2), mainw->warning_label, TRUE, TRUE, 0);
  gtk_label_set_justify (GTK_LABEL (mainw->warning_label), GTK_JUSTIFY_CENTER);
  gtk_label_set_line_wrap (GTK_LABEL (mainw->warning_label), FALSE);
  gtk_widget_modify_fg(mainw->warning_label, GTK_STATE_NORMAL, &palette->normal_fore);

  if (warn_mask_number>0) {
    checkbutton8 = gtk_check_button_new_with_mnemonic (_("Do not show this warning again"));
    gtk_widget_show (checkbutton8);
    gtk_box_pack_start (GTK_BOX (dialog_vbox2), checkbutton8, FALSE, FALSE, 0);
    g_signal_connect (GTK_OBJECT (checkbutton8), "toggled",
                      G_CALLBACK (on_warn_mask_toggled),
                      NULL);
  }

  dialog_action_area2 = GTK_DIALOG (dialog2)->action_area;
  gtk_widget_show (dialog_action_area2);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area2), GTK_BUTTONBOX_END);

  warning_cancelbutton = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (warning_cancelbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (dialog2), warning_cancelbutton, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (warning_cancelbutton, GTK_CAN_DEFAULT);

  warning_okbutton = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (warning_okbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (dialog2), warning_okbutton, GTK_RESPONSE_OK);


  GTK_WIDGET_SET_FLAGS (warning_okbutton, GTK_CAN_DEFAULT);

  return dialog2;
}



// Information/error dialog
GtkWidget*
create_dialog3 (const gchar *text, gboolean is_blocking)
{
  GtkWidget *dialog3;
  GtkWidget *dialog_vbox3;
  GtkWidget *info_text;
  GtkWidget *dialog_action_area3;
  GtkWidget *info_ok_button;
  gchar *form_text;
  
  dialog3 = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (dialog3), "LiVES");
  if (is_blocking) gtk_window_set_modal (GTK_WINDOW (dialog3), TRUE);
  gtk_window_set_position (GTK_WINDOW (dialog3), GTK_WIN_POS_CENTER);
  gtk_window_set_default_size (GTK_WINDOW (dialog3), 350, 200);
  if (mainw!=NULL) {
    gtk_window_set_transient_for(GTK_WINDOW(dialog3),GTK_WINDOW(mainw->LiVES));
    gtk_widget_modify_bg(dialog3, GTK_STATE_NORMAL, &palette->normal_back);
    if (palette->style&STYLE_1) {
      gtk_dialog_set_has_separator(GTK_DIALOG(dialog3),FALSE);
    }
  }

  dialog_vbox3 = GTK_DIALOG (dialog3)->vbox;
  gtk_widget_show (dialog_vbox3);
  
  form_text=g_strdup_printf("\n\n%s",text);

  info_text = gtk_label_new (form_text);
  g_free(form_text);

  if (mainw!=NULL) {
    gtk_widget_modify_fg(info_text, GTK_STATE_NORMAL, &palette->normal_fore);
  }

  gtk_label_set_selectable (GTK_LABEL (info_text), TRUE);

  gtk_widget_show (info_text);
  gtk_box_pack_start (GTK_BOX (dialog_vbox3), info_text, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (info_text), GTK_JUSTIFY_CENTER);
  gtk_label_set_line_wrap (GTK_LABEL (info_text), FALSE);
  
  dialog_action_area3 = GTK_DIALOG (dialog3)->action_area;
  gtk_widget_show (dialog_action_area3);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area3), GTK_BUTTONBOX_END);
  
  info_ok_button = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (info_ok_button);
  gtk_dialog_add_action_widget (GTK_DIALOG (dialog3), info_ok_button, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (info_ok_button, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (info_ok_button);

  if (!is_blocking) {
    g_signal_connect (GTK_OBJECT (info_ok_button), "clicked",
		      G_CALLBACK (on_info_ok_button_clicked),
		      NULL);
    
  }

  gtk_widget_grab_focus (info_ok_button);
  return dialog3;
}



GtkWidget*
create_text_window (const gchar *title, const gchar *text)
{
  // general text window
  GtkWidget *text_window;
  GtkWidget *dialog_vbox20;
  GtkWidget *scrolledwindow3;
  GtkWidget *textview2;
  GtkWidget *dialog_action_area20;
  GtkWidget *okbutton17;

  text_window = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (text_window), g_strconcat ("LiVES: - ",title,NULL));
  gtk_window_set_position (GTK_WINDOW (text_window), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (text_window), TRUE);
  gtk_window_set_default_size (GTK_WINDOW (text_window), 600, 400);
  gtk_window_set_transient_for(GTK_WINDOW(text_window),GTK_WINDOW(mainw->LiVES));

  gtk_widget_modify_bg(text_window, GTK_STATE_NORMAL, &palette->normal_back);
  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(text_window),FALSE);
  }

  dialog_vbox20 = GTK_DIALOG (text_window)->vbox;
  gtk_widget_show (dialog_vbox20);

  scrolledwindow3 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow3);
  gtk_box_pack_start (GTK_BOX (dialog_vbox20), scrolledwindow3, TRUE, TRUE, 0);

  textview2 = gtk_text_view_new ();
  gtk_widget_show (textview2);
  gtk_container_add (GTK_CONTAINER (scrolledwindow3), textview2);

  gtk_text_view_set_editable (GTK_TEXT_VIEW (textview2), FALSE);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (textview2), GTK_WRAP_WORD);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (textview2), FALSE);

  if (palette->style&STYLE_1) {
    gtk_widget_modify_base(textview2, GTK_STATE_NORMAL, &palette->info_base);
    gtk_widget_modify_text(textview2, GTK_STATE_NORMAL, &palette->info_text);
  }

  gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview2)), text, -1);

  dialog_action_area20 = GTK_DIALOG (text_window)->action_area;
  gtk_widget_show (dialog_action_area20);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area20), GTK_BUTTONBOX_END);

  okbutton17 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton17);
  gtk_widget_set_size_request (okbutton17, 586, 78);
  gtk_container_set_border_width (GTK_CONTAINER (okbutton17), 12);
  gtk_button_set_relief (GTK_BUTTON (okbutton17), GTK_RELIEF_NONE);

  gtk_dialog_add_action_widget (GTK_DIALOG (text_window), okbutton17, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton17, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (okbutton17);

  g_signal_connect (GTK_OBJECT (okbutton17), "clicked",
		      G_CALLBACK (on_cancel_button1_clicked),
		      NULL);
  
  gtk_widget_grab_focus (okbutton17);

  return text_window;
}




void
add_to_winmenu(void) {
  // TODO - change to add_to_clipmenu, move to gui.c
  GtkWidget *active_image;
  cfile->menuentry = gtk_image_menu_item_new_with_label(cfile->name);
  gtk_widget_show (cfile->menuentry);
  gtk_container_add (GTK_CONTAINER (mainw->winmenu), cfile->menuentry);

  gtk_widget_set_sensitive (cfile->menuentry, TRUE);
  g_signal_connect (GTK_OBJECT (cfile->menuentry), "activate",
                      G_CALLBACK (switch_clip_activate),
                      NULL);

  if (cfile->opening==FALSE&&cfile->clip_type==CLIP_TYPE_DISK) {
    active_image = gtk_image_new_from_stock ("gtk-close", GTK_ICON_SIZE_MENU);
  }
  else {
    active_image = gtk_image_new_from_stock ("gtk-yes", GTK_ICON_SIZE_MENU);
  }
  gtk_widget_show (active_image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (cfile->menuentry), active_image);
  mainw->clips_available++;
  mainw->cliplist = g_list_append (mainw->cliplist, GINT_TO_POINTER (mainw->current_file));
  cfile->old_frames=cfile->frames;
}




void
remove_from_winmenu() {
  gtk_container_remove(GTK_CONTAINER(mainw->winmenu), cfile->menuentry);
  gtk_widget_destroy(cfile->menuentry);
  mainw->clips_available--;
  mainw->cliplist=g_list_remove (mainw->cliplist, GINT_TO_POINTER (mainw->current_file));
}



_insertw*
create_insert_dialog ()
{
  GtkWidget *dialog_vbox3;
  GtkWidget *hbox15;
  GtkWidget *label57;
  GtkObject *spinbutton_times_adj;
  GtkWidget *table2;
  GtkWidget *radiobutton1;
  GSList *radiobutton1_group = NULL;
  GSList *radiobutton2_group = NULL;
  GtkWidget *radiobutton4;
  GtkWidget *vseparator1;
  GtkWidget *vseparator2;
  GtkWidget *hseparator1;
  GtkWidget *dialog_action_area3;
  GtkWidget *cancelbutton1;
  GtkWidget *okbutton1;

  _insertw *insertw=(_insertw*)(g_malloc(sizeof(_insertw)));

  insertw->insert_dialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (insertw->insert_dialog), _("LiVES: - Insert"));
  gtk_window_set_position (GTK_WINDOW (insertw->insert_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (insertw->insert_dialog), TRUE);
  gtk_window_set_default_size (GTK_WINDOW (insertw->insert_dialog), 300, 200);
  if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) {
    gtk_widget_modify_bg (insertw->insert_dialog, GTK_STATE_NORMAL, &palette->normal_back);
  }
  else {
    gtk_widget_modify_bg (insertw->insert_dialog, GTK_STATE_NORMAL, &palette->menu_and_bars);
  }
  gtk_window_set_transient_for(GTK_WINDOW(insertw->insert_dialog),GTK_WINDOW(mainw->LiVES));

  dialog_vbox3 = GTK_DIALOG (insertw->insert_dialog)->vbox;
  gtk_widget_show (dialog_vbox3);

  hbox15 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox15);
  gtk_box_pack_start (GTK_BOX (dialog_vbox3), hbox15, TRUE, TRUE, 0);

  label57 = gtk_label_new (_("Number of times to insert"));
  gtk_widget_show (label57);
  gtk_box_pack_start (GTK_BOX (hbox15), label57, TRUE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label57), GTK_JUSTIFY_LEFT);
  if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) {
    gtk_widget_modify_fg (label57, GTK_STATE_NORMAL, &palette->normal_fore);
  }
  spinbutton_times_adj = gtk_adjustment_new (1, 1, 10000, 1, 10, 100);
  insertw->spinbutton_times = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_times_adj), 1, 0);
  gtk_widget_show (insertw->spinbutton_times);
  gtk_box_pack_start (GTK_BOX (hbox15), insertw->spinbutton_times, TRUE, FALSE, 0);
  gtk_entry_set_activates_default (GTK_ENTRY ((GtkEntry *)&(GTK_SPIN_BUTTON (insertw->spinbutton_times)->entry)), TRUE);

  insertw->fit_checkbutton = gtk_check_button_new_with_mnemonic (_ ("_Insert to fit audio"));
  gtk_widget_show (insertw->fit_checkbutton);
  gtk_box_pack_start (GTK_BOX (hbox15), insertw->fit_checkbutton, TRUE, FALSE, 0);
  gtk_widget_set_sensitive(GTK_WIDGET(insertw->fit_checkbutton),cfile->achans>0&&clipboard->achans==0);

  hseparator1 = gtk_hseparator_new ();
  gtk_widget_show (hseparator1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox3), hseparator1, TRUE, TRUE, 0);

  table2 = gtk_table_new (2, 3, FALSE);
  gtk_widget_show (table2);
  gtk_box_pack_start (GTK_BOX (dialog_vbox3), table2, TRUE, TRUE, 0);
  gtk_table_set_col_spacings (GTK_TABLE (table2), 43);

  radiobutton1 = gtk_radio_button_new_with_mnemonic (NULL, _("Insert _before selection"));
  gtk_widget_show (radiobutton1);
  gtk_table_attach (GTK_TABLE (table2), radiobutton1, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  gtk_container_set_border_width (GTK_CONTAINER (radiobutton1), 18);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton1), radiobutton1_group);
  radiobutton1_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton1));
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(radiobutton1),!(mainw->insert_after));
  if (cfile->frames==0) {
    gtk_widget_set_sensitive (radiobutton1, FALSE);
  }

  radiobutton4 = gtk_radio_button_new_with_mnemonic (NULL, _("Insert _after selection"));
  gtk_widget_show (radiobutton4);
  gtk_table_attach (GTK_TABLE (table2), radiobutton4, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_container_set_border_width (GTK_CONTAINER (radiobutton4), 18);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton4), radiobutton1_group);
  radiobutton1_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton4));
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(radiobutton4),mainw->insert_after);

  insertw->with_sound = gtk_radio_button_new_with_mnemonic (NULL, _("Insert _with sound"));
  gtk_widget_show (insertw->with_sound);
  gtk_table_attach (GTK_TABLE (table2), insertw->with_sound, 2, 3, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_container_set_border_width (GTK_CONTAINER (insertw->with_sound), 18);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (insertw->with_sound), radiobutton2_group);
  radiobutton2_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (insertw->with_sound));

  insertw->without_sound = gtk_radio_button_new_with_mnemonic (NULL, _("Insert with_out sound"));
  gtk_widget_show (insertw->without_sound);
  gtk_table_attach (GTK_TABLE (table2), insertw->without_sound, 2, 3, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_container_set_border_width (GTK_CONTAINER (insertw->without_sound), 18);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (insertw->without_sound), radiobutton2_group);
  radiobutton2_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (insertw->without_sound));

  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(insertw->with_sound),clipboard->achans>0&&mainw->ccpd_with_sound==TRUE);
  gtk_widget_set_sensitive (insertw->with_sound, clipboard->achans>0);
  gtk_widget_set_sensitive (insertw->without_sound, clipboard->achans>0);

  vseparator1 = gtk_vseparator_new ();
  gtk_widget_show (vseparator1);
  gtk_table_attach (GTK_TABLE (table2), vseparator1, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  vseparator2 = gtk_vseparator_new ();
  gtk_widget_show (vseparator2);
  gtk_table_attach (GTK_TABLE (table2), vseparator2, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  dialog_action_area3 = GTK_DIALOG (insertw->insert_dialog)->action_area;
  gtk_widget_show (dialog_action_area3);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area3), GTK_BUTTONBOX_END);

  cancelbutton1 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (insertw->insert_dialog), cancelbutton1, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton1, GTK_CAN_DEFAULT);

  okbutton1 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (insertw->insert_dialog), okbutton1, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton1, GTK_CAN_DEFAULT|GTK_CAN_FOCUS);
  gtk_widget_grab_default(okbutton1);
  gtk_widget_grab_focus(okbutton1);

  g_signal_connect (GTK_OBJECT (insertw->with_sound), "toggled",
                      G_CALLBACK (on_insertwsound_toggled),
                      NULL);
  g_signal_connect (GTK_OBJECT (radiobutton4), "toggled",
                      G_CALLBACK (on_insert_after_toggled),
                      NULL);
  g_signal_connect (GTK_OBJECT (cancelbutton1), "clicked",
                      G_CALLBACK (on_cancel_button1_clicked),
                      insertw);
  g_signal_connect (GTK_OBJECT (okbutton1), "clicked",
                      G_CALLBACK (on_insert_activate),
                      NULL);
  g_signal_connect (GTK_OBJECT (insertw->fit_checkbutton), "toggled",
		    G_CALLBACK (on_insfitaudio_toggled),
		    NULL);
  g_signal_connect_after (GTK_OBJECT (insertw->spinbutton_times), "value_changed",
                      G_CALLBACK (on_spin_value_changed),
                      GINT_TO_POINTER (1));
  g_signal_connect (GTK_OBJECT (insertw->insert_dialog), "delete_event",
                      G_CALLBACK (return_true),
                      NULL);

  return insertw;
}



GtkWidget*
create_new_pb_speed (gshort type)
{
  GtkWidget *new_pb_speed;
  GtkWidget *dialog_vbox6;
  GtkWidget *vbox6;
  GtkWidget *pb_speed_text;
  GtkObject *spinbutton_pb_speed_adj;
  GtkWidget *spinbutton_pb_speed;
  GtkWidget *dialog_action_area6;
  GtkWidget *cancelbutton4;
  GtkWidget *change_pb_ok;
  GtkWidget *change_audio_speed;
  gchar label_text[256];

  new_pb_speed = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER (new_pb_speed), 10);
  if (type==1) {
    gtk_window_set_title (GTK_WINDOW (new_pb_speed), _("LiVES: - Change playback speed"));
  }
  else if (type==2) {
    gtk_window_set_title (GTK_WINDOW (new_pb_speed), _("LiVES: - Resample Video"));
  }
  gtk_window_set_position (GTK_WINDOW (new_pb_speed), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (new_pb_speed), TRUE);
  gtk_window_set_default_size (GTK_WINDOW (new_pb_speed), 300, 200);
  gtk_window_set_transient_for(GTK_WINDOW(new_pb_speed),GTK_WINDOW(mainw->LiVES));

  gtk_widget_modify_bg(new_pb_speed, GTK_STATE_NORMAL, &palette->normal_back);
  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(new_pb_speed),FALSE);
  }

  dialog_vbox6 = GTK_DIALOG (new_pb_speed)->vbox;
  gtk_widget_show (dialog_vbox6);

  vbox6 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox6);
  gtk_box_pack_start (GTK_BOX (dialog_vbox6), vbox6, TRUE, TRUE, 0);

  if (type==1) {
    g_snprintf(label_text,256,_("\n\nCurrent playback speed is %.3f frames per second.\n\nPlease enter the desired playback speed\nin frames per second"),cfile->fps);
  }
  else if (type==2) {
    g_snprintf(label_text,256,_("\n\nCurrent playback speed is %.3f frames per second.\n\nPlease enter the resampled rate\nin frames per second"),cfile->fps);
  }
  pb_speed_text = gtk_label_new (label_text);
  gtk_widget_show (pb_speed_text);
  gtk_box_pack_start (GTK_BOX (vbox6), pb_speed_text, TRUE, TRUE, 0);
  gtk_widget_modify_fg(pb_speed_text, GTK_STATE_NORMAL, &palette->normal_fore);

  spinbutton_pb_speed_adj = gtk_adjustment_new (cfile->fps, 1, FPS_MAX, 0.01, .1, .5);
  spinbutton_pb_speed = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_pb_speed_adj), 1, 3);
  gtk_widget_show (spinbutton_pb_speed);
  gtk_box_pack_start (GTK_BOX (vbox6), spinbutton_pb_speed, FALSE, FALSE, 8);
  gtk_widget_set_size_request (spinbutton_pb_speed, 10, -1);
  gtk_entry_set_activates_default (GTK_ENTRY ((GtkEntry *)&(GTK_SPIN_BUTTON (spinbutton_pb_speed)->entry)), TRUE);

  change_audio_speed = gtk_check_button_new_with_mnemonic (_("Change the audio speed as well"));
  if (type==1&&cfile->achans>0) {
    gtk_widget_show (change_audio_speed);
  }
  gtk_box_pack_start (GTK_BOX (vbox6), change_audio_speed, FALSE, FALSE, 0);

  dialog_action_area6 = GTK_DIALOG (new_pb_speed)->action_area;
  gtk_widget_show (dialog_action_area6);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area6), GTK_BUTTONBOX_END);

  cancelbutton4 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton4);
  gtk_dialog_add_action_widget (GTK_DIALOG (new_pb_speed), cancelbutton4, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton4, GTK_CAN_DEFAULT);

  change_pb_ok = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (change_pb_ok);
  gtk_dialog_add_action_widget (GTK_DIALOG (new_pb_speed), change_pb_ok, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (change_pb_ok, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (change_pb_ok);

  g_signal_connect (GTK_OBJECT (change_audio_speed), "toggled",
		    G_CALLBACK (on_boolean_toggled),
		    GINT_TO_POINTER (1));
  g_signal_connect (GTK_OBJECT (cancelbutton4), "clicked",
		    G_CALLBACK (on_cancel_button1_clicked),
		    NULL);
  if (type==1) {
    g_signal_connect (GTK_OBJECT (change_pb_ok), "clicked",
		      G_CALLBACK (on_change_speed_ok_clicked),
		      NULL);
  }
  else if (type==2) {
    g_signal_connect (GTK_OBJECT (change_pb_ok), "clicked",
		      G_CALLBACK (on_resample_vid_ok),
		      NULL);
  }
  g_signal_connect_after (GTK_OBJECT (spinbutton_pb_speed), "value_changed",
			  G_CALLBACK (on_spin_value_changed),
			  GINT_TO_POINTER (1));


  return new_pb_speed;
}




GtkWidget*
create_opensel_dialog ()
{
  GtkWidget *opensel_dialog;
  GtkWidget *dialog_vbox9;
  GtkWidget *vbox15;
  GtkWidget *table5;
  GtkWidget *label46;
  GtkWidget *label47;
  GtkObject *spinbutton23_adj;
  GtkWidget *spinbutton23;
  GtkObject *spinbutton24_adj;
  GtkWidget *spinbutton24;
  GtkWidget *dialog_action_area9;
  GtkWidget *cancelbutton7;
  GtkWidget *okbutton6;

  opensel_dialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (opensel_dialog), _("LiVES: - Open Selection"));
  gtk_window_set_position (GTK_WINDOW (opensel_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (opensel_dialog), TRUE);
  gtk_window_set_transient_for(GTK_WINDOW(opensel_dialog),GTK_WINDOW(mainw->LiVES));
  if (palette->style&STYLE_1) {
    gtk_widget_modify_bg(opensel_dialog, GTK_STATE_NORMAL, &palette->normal_back);
    gtk_dialog_set_has_separator(GTK_DIALOG(opensel_dialog),FALSE);
  }

  gtk_container_set_border_width (GTK_CONTAINER (opensel_dialog), 10);
  gtk_window_set_default_size (GTK_WINDOW (opensel_dialog), 300, 200);

  dialog_vbox9 = GTK_DIALOG (opensel_dialog)->vbox;
  gtk_widget_show (dialog_vbox9);

  vbox15 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox15);
  gtk_box_pack_start (GTK_BOX (dialog_vbox9), vbox15, TRUE, TRUE, 0);

  table5 = gtk_table_new (2, 2, FALSE);
  gtk_widget_show (table5);
  gtk_box_pack_start (GTK_BOX (vbox15), table5, TRUE, TRUE, 12);

  gtk_table_set_row_spacings (GTK_TABLE (table5), 20);
  label46 = gtk_label_new (_("    Selection start time (sec)"));
  gtk_widget_show (label46);
  gtk_table_attach (GTK_TABLE (table5), label46, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label46), 0, 0.5);
  gtk_widget_modify_fg(label46, GTK_STATE_NORMAL, &palette->normal_fore);

  label47 = gtk_label_new (_("    Number of frames to open"));
  gtk_widget_show (label47);
  gtk_table_attach (GTK_TABLE (table5), label47, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label47), 0, 0.5);
  gtk_widget_modify_fg(label47, GTK_STATE_NORMAL, &palette->normal_fore);

  spinbutton23_adj = gtk_adjustment_new (0, 0, 65535, 1, 10, 100);
  spinbutton23 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton23_adj), 1, 2);
  gtk_widget_show (spinbutton23);
  gtk_table_attach (GTK_TABLE (table5), spinbutton23, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 42, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton23), TRUE);

  spinbutton24_adj = gtk_adjustment_new (1000, 1, 100000, 1, 10, 100);
  spinbutton24 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton24_adj), 1, 0);
  gtk_widget_show (spinbutton24);
  gtk_table_attach (GTK_TABLE (table5), spinbutton24, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 42, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton24), TRUE);


  dialog_action_area9 = GTK_DIALOG (opensel_dialog)->action_area;
  gtk_widget_show (dialog_action_area9);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area9), GTK_BUTTONBOX_END);

  cancelbutton7 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton7);
  gtk_dialog_add_action_widget (GTK_DIALOG (opensel_dialog), cancelbutton7, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton7, GTK_CAN_DEFAULT);

  okbutton6 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton6);
  gtk_dialog_add_action_widget (GTK_DIALOG (opensel_dialog), okbutton6, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton6, GTK_CAN_DEFAULT);
  gtk_widget_grab_default(okbutton6);

  widget_add_preview (GTK_BOX (dialog_vbox9), GTK_BOX (dialog_vbox9), 3);

  g_signal_connect_after (GTK_OBJECT (spinbutton23), "value_changed",
                            G_CALLBACK (on_spin_value_changed),
                            GINT_TO_POINTER (1));
  g_signal_connect_after (GTK_OBJECT (spinbutton24), "value_changed",
                            G_CALLBACK (on_spin_value_changed),
                            GINT_TO_POINTER (2));
  g_signal_connect (GTK_OBJECT (cancelbutton7), "clicked",
                      G_CALLBACK (on_cancel_opensel_clicked),
                      NULL);
  g_signal_connect (GTK_OBJECT (okbutton6), "clicked",
                      G_CALLBACK (on_opensel_range_ok_clicked),
                      NULL);

   return opensel_dialog;
}



_resaudw *
create_resaudw (gshort type)
{
  // type 1 == resample
  // type 2 == insert silence

  GtkWidget *dialog_vbox15;
  GtkWidget *vbox21;
  GtkWidget *frame6;
  GtkWidget *hbox24;
  GtkWidget *label94;
  GtkWidget *combo_entry2;
  GtkWidget *label95;
  GtkWidget *combo_entry3;
  GtkWidget *label96;
  GtkWidget *combo_entry1;
  GtkWidget *vseparator3;
  GtkWidget *vbox22;
  GtkWidget *radiobutton26;
  GSList *radiobutton26_group = NULL;
  GtkWidget *radiobutton27;
  GtkWidget *vseparator4;
  GtkWidget *vbox23;
  GtkWidget *radiobutton28;
  GSList *radiobutton28_group = NULL;
  GtkWidget *radiobutton29;
  GtkWidget *label92;
  GtkWidget *frame7;
  GtkWidget *hbox25;
  GtkWidget *label98;
  GtkWidget *combo4;
  GtkWidget *label99;
  GtkWidget *combo5;
  GtkWidget *label100;
  GtkWidget *combo6;
  GtkWidget *vseparator5;
  GtkWidget *vbox24;
  GSList *radiobutton30_group = NULL;
  GtkWidget *vseparator6;
  GtkWidget *vbox25;
  GSList *radiobutton32_group = NULL;
  GtkWidget *label93;
  GtkWidget *dialog_action_area15;
  GtkWidget *cancelbutton13;
  GtkWidget *okbutton12;
  GList *channels = NULL;
  GList *sampsize = NULL;
  GList *rate = NULL;
  gchar *tmp;

  _resaudw *resaudw=(_resaudw*)(g_malloc(sizeof(_resaudw)));

  channels = g_list_append (channels, "1");
  channels = g_list_append (channels, "2");

  sampsize = g_list_append (sampsize, "8");
  sampsize = g_list_append (sampsize, "16");

  rate = g_list_append (rate, "5512");
  rate = g_list_append (rate, "8000");
  rate = g_list_append (rate, "11025");
  rate = g_list_append (rate, "22050");
  rate = g_list_append (rate, "32000");
  rate = g_list_append (rate, "44100");
  rate = g_list_append (rate, "48000");
  rate = g_list_append (rate, "88200");
  rate = g_list_append (rate, "96000");
  rate = g_list_append (rate, "128000");


  resaudw->dialog = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER (resaudw->dialog), 10);
  if (type==1) {
    gtk_window_set_title (GTK_WINDOW (resaudw->dialog), _("LiVES: - Resample Audio"));
  }
  if (type==2) {
    gtk_window_set_title (GTK_WINDOW (resaudw->dialog), _("LiVES: - Insert Silence"));
  }
  gtk_window_set_position (GTK_WINDOW (resaudw->dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (resaudw->dialog), TRUE);
  gtk_window_set_transient_for(GTK_WINDOW(resaudw->dialog),GTK_WINDOW(mainw->LiVES));

  dialog_vbox15 = GTK_DIALOG (resaudw->dialog)->vbox;
  gtk_widget_show (dialog_vbox15);
  if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) {
    gtk_widget_modify_bg (resaudw->dialog, GTK_STATE_NORMAL, &palette->normal_back);
  }
  else {
    gtk_widget_modify_bg (resaudw->dialog, GTK_STATE_NORMAL, &palette->menu_and_bars);
  }


  vbox21 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox21);
  gtk_box_pack_start (GTK_BOX (dialog_vbox15), vbox21, TRUE, TRUE, 0);

  frame6 = gtk_frame_new (NULL);
 
  if (type==1) {

    gtk_widget_show (frame6);
    gtk_box_pack_start (GTK_BOX (vbox21), frame6, TRUE, TRUE, 0);
    
    if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) {
      gtk_widget_modify_bg (frame6, GTK_STATE_NORMAL, &palette->normal_back);
    }
    else {
      gtk_widget_modify_bg (frame6, GTK_STATE_NORMAL, &palette->menu_and_bars);
    }
    
    hbox24 = gtk_hbox_new (FALSE, 0);
    gtk_widget_show (hbox24);
    gtk_container_add (GTK_CONTAINER (frame6), hbox24);
    gtk_container_set_border_width (GTK_CONTAINER (hbox24), 10);
    
    label94 = gtk_label_new (_("Rate (Hz) "));
    gtk_widget_show (label94);
    gtk_box_pack_start (GTK_BOX (hbox24), label94, FALSE, FALSE, 0);
    gtk_label_set_justify (GTK_LABEL (label94), GTK_JUSTIFY_LEFT);
    
    combo_entry2 = gtk_entry_new();
    gtk_widget_show (combo_entry2);
    gtk_box_pack_start (GTK_BOX (hbox24), combo_entry2, TRUE, TRUE, 1);

    gtk_editable_set_editable (GTK_EDITABLE (combo_entry2), FALSE);
    gtk_entry_set_width_chars (GTK_ENTRY (combo_entry2), 8);
    
    tmp=g_strdup_printf ("%d",(gint)mainw->fx1_val);
    gtk_entry_set_text (GTK_ENTRY (combo_entry2),tmp);
    g_free (tmp);
    GTK_WIDGET_UNSET_FLAGS (combo_entry2, GTK_CAN_FOCUS);
    
    label95 = gtk_label_new (_("         Channels "));
    gtk_widget_show (label95);
    gtk_box_pack_start (GTK_BOX (hbox24), label95, FALSE, FALSE, 0);
    gtk_label_set_justify (GTK_LABEL (label95), GTK_JUSTIFY_LEFT);
    
    combo_entry3 = gtk_entry_new();
    gtk_widget_show (combo_entry3);
    gtk_box_pack_start (GTK_BOX (hbox24), combo_entry3, TRUE, TRUE, 1);
    
    gtk_editable_set_editable (GTK_EDITABLE (combo_entry3), FALSE);
    gtk_entry_set_width_chars (GTK_ENTRY (combo_entry3), 3);
    
    tmp=g_strdup_printf ("%d",(gint)mainw->fx2_val);
    gtk_entry_set_text (GTK_ENTRY (combo_entry3),tmp);
    g_free (tmp);
    GTK_WIDGET_UNSET_FLAGS (combo_entry3, GTK_CAN_FOCUS);
    
    label96 = gtk_label_new (_("        Sample Size "));
    gtk_widget_show (label96);
    gtk_box_pack_start (GTK_BOX (hbox24), label96, FALSE, FALSE, 0);
    gtk_label_set_justify (GTK_LABEL (label96), GTK_JUSTIFY_LEFT);

    combo_entry1 = gtk_entry_new();
    gtk_widget_show (combo_entry1);
    gtk_box_pack_start (GTK_BOX (hbox24), combo_entry1, TRUE, TRUE, 1);

    gtk_editable_set_editable (GTK_EDITABLE (combo_entry1), FALSE);
    gtk_entry_set_width_chars (GTK_ENTRY (combo_entry1), 3);

    tmp=g_strdup_printf ("%d",(gint)mainw->fx3_val);
    gtk_entry_set_text (GTK_ENTRY (combo_entry1),tmp);
    g_free (tmp);
    GTK_WIDGET_UNSET_FLAGS (combo_entry1, GTK_CAN_FOCUS);

    vseparator3 = gtk_vseparator_new ();
    gtk_widget_show (vseparator3);
    gtk_box_pack_start (GTK_BOX (hbox24), vseparator3, FALSE, FALSE, 12);

    vbox22 = gtk_vbox_new (FALSE, 0);
    gtk_widget_show (vbox22);
    gtk_box_pack_start (GTK_BOX (hbox24), vbox22, TRUE, TRUE, 0);

    radiobutton26 = gtk_radio_button_new_with_mnemonic (NULL, _("Signed"));
    gtk_widget_show (radiobutton26);
    gtk_box_pack_start (GTK_BOX (vbox22), radiobutton26, TRUE, FALSE, 0);
    gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton26), radiobutton26_group);
    radiobutton26_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton26));
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton26), TRUE);

    radiobutton27 = gtk_radio_button_new_with_mnemonic (NULL, _("Unsigned"));
    gtk_widget_show (radiobutton27);
    gtk_box_pack_start (GTK_BOX (vbox22), radiobutton27, TRUE, FALSE, 0);
    gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton27), radiobutton26_group);
    radiobutton26_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton27));

    if ((gint)mainw->fx4_val&AFORM_UNSIGNED) {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton27), TRUE);
    }
    else {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton26), TRUE);
    }

    gtk_widget_set_sensitive (radiobutton26, FALSE);
    gtk_widget_set_sensitive (radiobutton27, FALSE);

    vseparator4 = gtk_vseparator_new ();
    gtk_widget_show (vseparator4);
    gtk_box_pack_start (GTK_BOX (hbox24), vseparator4, FALSE, FALSE, 12);
    
    vbox23 = gtk_vbox_new (FALSE, 0);
    gtk_widget_show (vbox23);
    gtk_box_pack_start (GTK_BOX (hbox24), vbox23, TRUE, TRUE, 0);

    radiobutton28 = gtk_radio_button_new_with_mnemonic (NULL, _("Little Endian"));
    gtk_widget_show (radiobutton28);
    gtk_box_pack_start (GTK_BOX (vbox23), radiobutton28, TRUE, FALSE, 0);
    gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton28), radiobutton28_group);
    radiobutton28_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton28));
    
    radiobutton29 = gtk_radio_button_new_with_mnemonic (NULL, _("Big Endian"));
    gtk_widget_show (radiobutton29);
    gtk_box_pack_start (GTK_BOX (vbox23), radiobutton29, TRUE, FALSE, 0);
    gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton29), radiobutton28_group);
    radiobutton28_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton29));
    
    if ((gint)mainw->fx4_val&AFORM_BIG_ENDIAN) {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton29), TRUE);
    }
    else {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton28), TRUE);
    }

    gtk_widget_set_sensitive (radiobutton29, FALSE);
    gtk_widget_set_sensitive (radiobutton28, FALSE);

    label92 = gtk_label_new (_("Current"));
    
    gtk_widget_show (label92);
    gtk_frame_set_label_widget (GTK_FRAME (frame6), label92);
    gtk_label_set_justify (GTK_LABEL (label92), GTK_JUSTIFY_LEFT);
    
  }

  frame7 = gtk_frame_new (NULL);
  gtk_widget_show (frame7);
  gtk_box_pack_start (GTK_BOX (vbox21), frame7, TRUE, TRUE, 0);

  if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) {
    gtk_widget_modify_bg (frame7, GTK_STATE_NORMAL, &palette->normal_back);
  }
  else {
    gtk_widget_modify_bg (frame7, GTK_STATE_NORMAL, &palette->menu_and_bars);
  }

  hbox25 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox25);
  gtk_container_add (GTK_CONTAINER (frame7), hbox25);
  gtk_container_set_border_width (GTK_CONTAINER (hbox25), 10);

  label98 = gtk_label_new (_("Rate (Hz) "));
  gtk_widget_show (label98);
  gtk_box_pack_start (GTK_BOX (hbox25), label98, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label98), GTK_JUSTIFY_LEFT);

  combo4 = gtk_combo_new ();
  gtk_combo_set_popdown_strings (GTK_COMBO (combo4), rate);
  gtk_widget_show (combo4);
  gtk_box_pack_start (GTK_BOX (hbox25), combo4, TRUE, TRUE, 1);

  resaudw->entry_arate = GTK_COMBO (combo4)->entry;
  gtk_widget_show (resaudw->entry_arate);
  gtk_entry_set_width_chars (GTK_ENTRY (resaudw->entry_arate), 8);

  tmp=g_strdup_printf ("%d",(gint)mainw->fx1_val);
  gtk_entry_set_text (GTK_ENTRY (resaudw->entry_arate),tmp);
  g_free (tmp);

  label99 = gtk_label_new (_("    Channels "));
  gtk_widget_show (label99);
  gtk_box_pack_start (GTK_BOX (hbox25), label99, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label99), GTK_JUSTIFY_LEFT);

  combo5 = gtk_combo_new ();
  gtk_combo_set_popdown_strings (GTK_COMBO (combo5), channels);
  gtk_widget_show (combo5);
  gtk_box_pack_start (GTK_BOX (hbox25), combo5, FALSE, FALSE, 0);

  resaudw->entry_achans = GTK_COMBO (combo5)->entry;
  gtk_widget_show (resaudw->entry_achans);
  gtk_editable_set_editable (GTK_EDITABLE (resaudw->entry_achans), FALSE);
  gtk_entry_set_width_chars (GTK_ENTRY (resaudw->entry_achans), 3);

  tmp=g_strdup_printf ("%d",(gint)mainw->fx2_val);
  gtk_entry_set_text (GTK_ENTRY (resaudw->entry_achans),tmp);
  g_free (tmp);

  label100 = gtk_label_new (_("    Sample Size "));
  gtk_widget_show (label100);
  gtk_box_pack_start (GTK_BOX (hbox25), label100, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label100), GTK_JUSTIFY_LEFT);

  combo6 = gtk_combo_new ();
  gtk_combo_set_popdown_strings (GTK_COMBO (combo6), sampsize);
  gtk_widget_show (combo6);
  gtk_box_pack_start (GTK_BOX (hbox25), combo6, TRUE, TRUE, 0);

  resaudw->entry_asamps = GTK_COMBO (combo6)->entry;
  gtk_widget_show (resaudw->entry_asamps);
  gtk_entry_set_max_length (GTK_ENTRY (resaudw->entry_asamps), 2);
  gtk_editable_set_editable (GTK_EDITABLE (resaudw->entry_asamps), FALSE);
  gtk_entry_set_width_chars (GTK_ENTRY (resaudw->entry_asamps), 3);

  tmp=g_strdup_printf ("%d",(gint)mainw->fx3_val);
  gtk_entry_set_text (GTK_ENTRY (resaudw->entry_asamps),tmp);
  g_free (tmp);

  vseparator5 = gtk_vseparator_new ();
  gtk_widget_show (vseparator5);
  gtk_box_pack_start (GTK_BOX (hbox25), vseparator5, FALSE, FALSE, 12);

  vbox24 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox24);
  gtk_box_pack_start (GTK_BOX (hbox25), vbox24, TRUE, TRUE, 0);

  resaudw->rb_signed = gtk_radio_button_new_with_mnemonic (NULL, _("Signed"));
  gtk_widget_show (resaudw->rb_signed);
  gtk_box_pack_start (GTK_BOX (vbox24), resaudw->rb_signed, TRUE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (resaudw->rb_signed), radiobutton30_group);
  radiobutton30_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (resaudw->rb_signed));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (resaudw->rb_signed), TRUE);

  resaudw->rb_unsigned = gtk_radio_button_new_with_mnemonic (NULL, _("Unsigned"));
  gtk_widget_show (resaudw->rb_unsigned);
  gtk_box_pack_start (GTK_BOX (vbox24), resaudw->rb_unsigned, TRUE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (resaudw->rb_unsigned), radiobutton30_group);
  radiobutton30_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (resaudw->rb_unsigned));

  if ((gint)mainw->fx4_val&AFORM_UNSIGNED) {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (resaudw->rb_unsigned), TRUE);
  }
  else {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (resaudw->rb_signed), TRUE);
  }

  if (cfile->asampsize==8) {
    gtk_widget_set_sensitive (resaudw->rb_signed, FALSE);
    gtk_widget_set_sensitive (resaudw->rb_unsigned, FALSE);
  }

  vseparator6 = gtk_vseparator_new ();
  gtk_widget_show (vseparator6);
  gtk_box_pack_start (GTK_BOX (hbox25), vseparator6, FALSE, FALSE, 12);

  vbox25 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox25);
  gtk_box_pack_start (GTK_BOX (hbox25), vbox25, TRUE, TRUE, 0);

  resaudw->rb_littleend = gtk_radio_button_new_with_mnemonic (NULL, _("Little Endian"));
  gtk_widget_show (resaudw->rb_littleend);
  gtk_box_pack_start (GTK_BOX (vbox25), resaudw->rb_littleend, TRUE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (resaudw->rb_littleend), radiobutton32_group);
  radiobutton32_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (resaudw->rb_littleend));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (resaudw->rb_littleend), TRUE);

  resaudw->rb_bigend = gtk_radio_button_new_with_mnemonic (NULL, _("Big Endian"));
  gtk_widget_show (resaudw->rb_bigend);
  gtk_box_pack_start (GTK_BOX (vbox25), resaudw->rb_bigend, TRUE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (resaudw->rb_bigend), radiobutton32_group);
  radiobutton32_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (resaudw->rb_bigend));

  if ((gint)mainw->fx4_val&AFORM_BIG_ENDIAN) {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (resaudw->rb_bigend), TRUE);
  }
  else {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (resaudw->rb_littleend), TRUE);
  }

  if (cfile->asampsize==8) {
    gtk_widget_set_sensitive (resaudw->rb_littleend, FALSE);
    gtk_widget_set_sensitive (resaudw->rb_bigend, FALSE);
    }

  label93 = gtk_label_new (_("New"));
  gtk_widget_show (label93);
  gtk_frame_set_label_widget (GTK_FRAME (frame7), label93);
  gtk_label_set_justify (GTK_LABEL (label93), GTK_JUSTIFY_LEFT);

  dialog_action_area15 = GTK_DIALOG (resaudw->dialog)->action_area;
  gtk_widget_show (dialog_action_area15);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area15), GTK_BUTTONBOX_END);

  cancelbutton13 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton13);
  gtk_dialog_add_action_widget (GTK_DIALOG (resaudw->dialog), cancelbutton13, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton13, GTK_CAN_DEFAULT);

  okbutton12 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton12);
  gtk_dialog_add_action_widget (GTK_DIALOG (resaudw->dialog), okbutton12, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton12, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (okbutton12);

  g_signal_connect (GTK_OBJECT(resaudw->entry_asamps), "changed",
		    G_CALLBACK (on_resaudw_asamps_changed),
		    NULL);

  g_signal_connect (GTK_OBJECT (cancelbutton13), "clicked",
		    G_CALLBACK (on_cancel_button1_clicked),
		    resaudw);

  if (type==1) {
    g_signal_connect (GTK_OBJECT (okbutton12), "clicked",
		      G_CALLBACK (on_resaudio_ok_clicked),
		      NULL);
  }
  if (type==2) {
    g_signal_connect (GTK_OBJECT (okbutton12), "clicked",
		      G_CALLBACK (on_ins_silence_details_clicked),
		      NULL);
  }

  g_signal_connect (GTK_OBJECT (resaudw->dialog), "delete_event",
                      G_CALLBACK (return_true),
                      NULL);

  g_list_free (channels);
  g_list_free (sampsize);
  g_list_free (rate);

  return resaudw;
}




// TODO - move all prefs stuff into prefs.c
_prefsw *
create_prefs_dialog (void)
{
  GtkWidget *dialog_vbox8;
  GtkWidget *vbox7;
  GtkWidget *ins_resample;
  GtkWidget *hbox6;
  GtkWidget *hbox100;
  GtkObject *spinbutton_warn_fsize_adj;
  GtkObject *spinbutton_bwidth_adj;
  GtkWidget *label28;
  GtkWidget *label157;
  GtkWidget *hseparator2;
  GtkWidget *hbox8;
  GtkWidget *hseparator3;
  GtkWidget *hbox7;
  GtkWidget *pref_gui;
  GtkWidget *vbox9;
  GtkWidget *vbox109;
  GtkWidget *vbox99;
  GtkWidget *vbox69;
  GtkWidget *vbox98;
  GtkWidget *frame4;
  GtkWidget *hbox9;
  GtkWidget *hbox109;
  GtkWidget *hbox99;
  GtkWidget *hbox19;
  GtkWidget *label33;
  GtkWidget *label133;
  GtkWidget *label34;
  GtkWidget *label134;
  GtkWidget *label31;
  GtkWidget *frame5;
  GtkWidget *hbox10;
  GtkWidget *label35;
  GtkWidget *label36;
  GtkWidget *label32;
  GtkWidget *label25;
  GtkWidget *label84;
  GtkWidget *label86;
  GtkWidget *label87;
  GtkWidget *label97;
  GtkWidget *label88;
  GtkWidget *vbox18;
  GtkWidget *vbox12;
  GtkWidget *hbox11;
  GtkWidget *hbox94;
  GtkWidget *hbox101;
  GtkWidget *label37;
  GtkWidget *hseparator8;
  GtkWidget *hbox15;
  GtkWidget *hbox115;
  GtkWidget *label56;
  GtkWidget *label94;
  GtkWidget *hbox93;
  GtkWidget *table4;
  GtkWidget *label39;
  GtkWidget *label99;
  GtkWidget *label98;
  GtkWidget *label100;
  GtkWidget *label40;
  GtkWidget *label41;
  GtkWidget *label42;
  GtkWidget *label52;
  GtkWidget *label43;
  GtkWidget *label27;
  GtkWidget *vbox14;
  GtkWidget *hbox13;
  GtkWidget *label44;
  GtkObject *spinbutton_def_fps_adj;
  GtkWidget *label29;
  GtkWidget *dialog_action_area8;
  GtkWidget *okbutton5;
  GtkWidget *applybutton;
  GtkWidget *dirbutton1;
  GtkWidget *dirbutton2;
  GtkWidget *dirbutton3;
  GtkWidget *dirbutton4;
  GtkWidget *dirbutton5;
  GtkWidget *dirbutton6;
  GtkWidget *dirimage1;
  GtkWidget *dirimage2;
  GtkWidget *dirimage3;
  GtkWidget *dirimage4;
  GtkWidget *dirimage5;
  GtkWidget *dirimage6;
  GtkWidget *hseparator13;
  GtkWidget *hbox31;
  GtkWidget *label126;
  GtkWidget *pp_combo;
  GtkWidget *combo_entry4;
  GtkWidget *label128;
  GtkWidget *entry20;
  GtkWidget *label129;
  GtkWidget *entry21;
  GtkWidget *ntsc;
  GtkWidget *png;
  GtkWidget *label158;
  GtkWidget *label159;
  GtkWidget *hbox116;
  GtkObject *spinbutton_ocp_adj;

  GtkWidget *label;
  GtkWidget *hbox;

  // tooltips
  GtkTooltips *tooltips;

  // radio button groups
  GSList *rb_group = NULL;
  GSList *pal_ntsc = NULL;
  GSList *jpeg_png = NULL;
  GSList *rb_group2 = NULL;

  // drop down lists
  GList *themes = NULL;
  GList *ofmt = NULL;
  GList *ofmt_all = NULL;
  GList *vidp = NULL;
  GList *audp = NULL;
  GList *encoders = NULL;
  GList *vid_playback_plugins = NULL;
  
  gchar **array;

  int i;

  _prefsw *prefsw=(_prefsw*)(g_malloc(sizeof(_prefsw)));

  tooltips = gtk_tooltips_new ();

  prefsw->prefs_dialog = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->prefs_dialog), 10);
  gtk_window_set_title (GTK_WINDOW (prefsw->prefs_dialog), _("LiVES: - Preferences"));
  gtk_window_set_position (GTK_WINDOW (prefsw->prefs_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (prefsw->prefs_dialog), TRUE);
  gtk_window_set_default_size (GTK_WINDOW (prefsw->prefs_dialog), 660, 440);
  gtk_window_set_transient_for(GTK_WINDOW(prefsw->prefs_dialog),GTK_WINDOW(mainw->LiVES));

  gtk_widget_modify_bg(prefsw->prefs_dialog, GTK_STATE_NORMAL, &palette->normal_back);
  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(prefsw->prefs_dialog),FALSE);
  }

  dialog_vbox8 = GTK_DIALOG (prefsw->prefs_dialog)->vbox;
  gtk_widget_show (dialog_vbox8);

  prefsw->prefs_notebook = gtk_notebook_new ();
  gtk_widget_show (prefsw->prefs_notebook);
  gtk_box_pack_start (GTK_BOX (dialog_vbox8), prefsw->prefs_notebook, TRUE, TRUE, 0);
  gtk_notebook_set_tab_hborder (GTK_NOTEBOOK (prefsw->prefs_notebook), 10);
  gtk_notebook_popup_enable(GTK_NOTEBOOK(prefsw->prefs_notebook));

  vbox7 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox7);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), vbox7);
  gtk_container_set_border_width (GTK_CONTAINER (vbox7), 20);

  hbox6 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox6);
  gtk_box_pack_start (GTK_BOX (vbox7), hbox6, TRUE, TRUE, 0);

  label28 = gtk_label_new (_("Background colour (for Blank Background and effects)"));
  gtk_widget_show (label28);
  gtk_box_pack_start (GTK_BOX (hbox6), label28, FALSE, FALSE, 16);
  gtk_label_set_justify (GTK_LABEL (label28), GTK_JUSTIFY_LEFT);

  prefsw->bgcolour_entry = gtk_entry_new ();
  gtk_entry_set_max_length(GTK_ENTRY(prefsw->bgcolour_entry),255);

  gtk_entry_set_text(GTK_ENTRY(prefsw->bgcolour_entry),prefs->bgcolour);

  gtk_widget_show (prefsw->bgcolour_entry);
  gtk_box_pack_start (GTK_BOX (hbox6), prefsw->bgcolour_entry, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, prefsw->bgcolour_entry, _("What is your favourite colour ?"), NULL);

  prefsw->show_tool = gtk_check_button_new_with_mnemonic (_("Show toolbar when background is blanked"));
  gtk_widget_show (prefsw->show_tool);
  gtk_box_pack_start (GTK_BOX (vbox7), prefsw->show_tool, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->show_tool), 22);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->show_tool), future_prefs->show_tool);

  hseparator2 = gtk_hseparator_new ();
  gtk_widget_show (hseparator2);
  gtk_box_pack_start (GTK_BOX (vbox7), hseparator2, FALSE, TRUE, 0);

  hbox8 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox8);
  gtk_box_pack_start (GTK_BOX (vbox7), hbox8, TRUE, TRUE, 0);

  prefsw->fs_max_check = gtk_check_button_new_with_mnemonic (_("Open file selection maximised"));
  gtk_widget_show (prefsw->fs_max_check);
  gtk_box_pack_start (GTK_BOX (hbox8), prefsw->fs_max_check, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->fs_max_check), 22);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->fs_max_check), prefs->fileselmax);

  prefsw->recent_check = gtk_check_button_new_with_mnemonic (_("Show recent files in the File menu"));
  gtk_widget_show (prefsw->recent_check);
  gtk_box_pack_start (GTK_BOX (hbox8), prefsw->recent_check, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->recent_check), 22);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->recent_check), prefs->show_recent);

  hseparator3 = gtk_hseparator_new ();
  gtk_widget_show (hseparator3);
  gtk_box_pack_start (GTK_BOX (vbox7), hseparator3, FALSE, TRUE, 0);
  hbox7 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox7);
  gtk_box_pack_start (GTK_BOX (vbox7), hbox7, FALSE, FALSE, 0);

  prefsw->stop_screensaver_check = gtk_check_button_new_with_mnemonic (_("Stop xscreensaver on playback    "));
  gtk_widget_show (prefsw->stop_screensaver_check);
  gtk_box_pack_start (GTK_BOX (hbox7), prefsw->stop_screensaver_check, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->stop_screensaver_check), 22);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->stop_screensaver_check), prefs->stop_screensaver);

  prefsw->open_maximised_check = gtk_check_button_new_with_mnemonic (_("Open main window maximised"));
  gtk_widget_show (prefsw->open_maximised_check);
  gtk_box_pack_start (GTK_BOX (hbox7), prefsw->open_maximised_check, FALSE, FALSE, 0);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->open_maximised_check), prefs->open_maximised);

  pref_gui = gtk_label_new (_("GUI"));
  gtk_widget_show (pref_gui);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 0), pref_gui);
  gtk_label_set_justify (GTK_LABEL (pref_gui), GTK_JUSTIFY_LEFT);


  // decoding

  vbox109 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox109);
  gtk_container_set_border_width (GTK_CONTAINER (vbox109), 10);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), vbox109);

  hbox109 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox109);
  gtk_box_pack_start (GTK_BOX (vbox109), hbox109, TRUE, FALSE, 0);

  label133 = gtk_label_new (_("Video open command             "));
  gtk_widget_show (label133);
  gtk_box_pack_start (GTK_BOX (hbox109), label133, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label133), GTK_JUSTIFY_LEFT);

  prefsw->video_open_entry = gtk_entry_new ();
  gtk_entry_set_max_length(GTK_ENTRY(prefsw->video_open_entry),255);
  gtk_widget_show (prefsw->video_open_entry);
  gtk_box_pack_start (GTK_BOX (hbox109), prefsw->video_open_entry, FALSE, TRUE, 0);
  gtk_entry_set_text(GTK_ENTRY(prefsw->video_open_entry),prefs->video_open_command);

  // TODO - more lazy loading...
  if (prefs->ocp==-1) prefs->ocp=get_int_pref ("open_compression_percent");

  hbox116 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox116);
  gtk_box_pack_start (GTK_BOX (vbox109), hbox116, TRUE, FALSE, 0);

  label158 = gtk_label_new (_("Open compression                  "));
  gtk_widget_show (label158);
  gtk_label_set_justify (GTK_LABEL (label158), GTK_JUSTIFY_LEFT);
  gtk_box_pack_start (GTK_BOX (hbox116), label158, FALSE, TRUE, 0);

  spinbutton_ocp_adj = gtk_adjustment_new (prefs->ocp, 0, 100, 1, 5, 10);
  prefsw->spinbutton_ocp = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_ocp_adj), 1, 0);
  gtk_widget_show (prefsw->spinbutton_ocp);
  gtk_box_pack_start (GTK_BOX (hbox116), prefsw->spinbutton_ocp, FALSE, TRUE, 0);

  label159 = gtk_label_new (_ (" %     ( lower = slower, larger files; for jpeg, higher quality )"));
  gtk_widget_show (label159);
  gtk_box_pack_start (GTK_BOX (hbox116), label159, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label159), GTK_JUSTIFY_LEFT);

  hbox115 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox115);
  gtk_box_pack_start (GTK_BOX (vbox109), hbox115, TRUE, FALSE, 0);

  label157 = gtk_label_new (_("Default image format          "));
  gtk_widget_show (label157);
  gtk_label_set_justify (GTK_LABEL (label157), GTK_JUSTIFY_LEFT);
  gtk_box_pack_start (GTK_BOX (hbox115), label157, FALSE, TRUE, 0);

  prefsw->jpeg = gtk_radio_button_new_with_mnemonic (NULL, _("_jpeg"));
  gtk_widget_show (prefsw->jpeg);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->jpeg), 18);
  gtk_box_pack_start (GTK_BOX (hbox115), prefsw->jpeg, FALSE, FALSE, 0);

  gtk_radio_button_set_group (GTK_RADIO_BUTTON (prefsw->jpeg), jpeg_png);
  jpeg_png = gtk_radio_button_get_group (GTK_RADIO_BUTTON (prefsw->jpeg));

  png = gtk_radio_button_new_with_mnemonic (jpeg_png, _("_png (experimental, requires editing of smogrify)"));
  gtk_widget_show (png);
  gtk_container_set_border_width (GTK_CONTAINER (png), 18);
  gtk_box_pack_start (GTK_BOX (hbox115), png, FALSE, FALSE, 0);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (png),!strcmp (prefs->image_ext,"png"));


  label126 = gtk_label_new (_("Decoding"));
  gtk_widget_show (label126);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 1), label126);
  gtk_label_set_justify (GTK_LABEL (label126), GTK_JUSTIFY_LEFT);


  // playback

  vbox9 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox9);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), vbox9);

  frame4 = gtk_frame_new (NULL);
  gtk_widget_show (frame4);
  gtk_box_pack_start (GTK_BOX (vbox9), frame4, TRUE, TRUE, 0);

  vbox69=gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox69);
  gtk_container_add (GTK_CONTAINER (frame4), vbox69);
  gtk_container_set_border_width (GTK_CONTAINER (vbox69), 10);

  hbox9 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox9);
  gtk_container_add (GTK_CONTAINER (vbox69), hbox9);

  label33 = gtk_label_new (_("Player      "));
  gtk_widget_show (label33);
  gtk_box_pack_start (GTK_BOX (hbox9), label33, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label33), GTK_JUSTIFY_LEFT);

  vidp = g_list_append (vidp, _ ("internal (recommended)"));
  vidp = g_list_append (vidp, "mplayer");

  prefsw->vidp_combo = gtk_combo_new ();
  gtk_combo_set_popdown_strings (GTK_COMBO (prefsw->vidp_combo), vidp);
  gtk_box_pack_start (GTK_BOX (hbox9), prefsw->vidp_combo, FALSE, FALSE, 0);
  gtk_widget_show(prefsw->vidp_combo);

  gtk_editable_set_editable (GTK_EDITABLE((GTK_COMBO(prefsw->vidp_combo))->entry),FALSE);
  if (prefs->video_player==VID_PLAYER_INT) {
    gtk_entry_set_text (GTK_ENTRY((GTK_COMBO(prefsw->vidp_combo))->entry),_ ("internal (recommended)"));
  }
  else {
    gtk_entry_set_text (GTK_ENTRY((GTK_COMBO(prefsw->vidp_combo))->entry),"mplayer");
  }

  label34 = gtk_label_new (_("     Video play command      "));
  gtk_widget_show (label34);
  gtk_box_pack_start (GTK_BOX (hbox9), label34, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label34), GTK_JUSTIFY_LEFT);

  prefsw->video_command_entry = gtk_entry_new ();
  gtk_entry_set_max_length(GTK_ENTRY(prefsw->video_command_entry),255);
  gtk_widget_show (prefsw->video_command_entry);
  gtk_box_pack_start (GTK_BOX (hbox9), prefsw->video_command_entry, TRUE, TRUE, 0);

  gtk_entry_set_text(GTK_ENTRY(prefsw->video_command_entry),prefs->video_play_command);

  if (prefs->video_player==VID_PLAYER_INT) {
    gtk_widget_hide(prefsw->video_command_entry);
    gtk_widget_hide(label34);
  }
  
  hbox101 = gtk_hbox_new (TRUE, 0);
  gtk_widget_show (hbox101);
  gtk_box_pack_start (GTK_BOX (vbox69), hbox101, FALSE, FALSE, 0);

  prefsw->checkbutton_show_stats = gtk_check_button_new_with_mnemonic (_ ("_Show FPS statistics"));
  gtk_widget_show (prefsw->checkbutton_show_stats);
  gtk_box_pack_start (GTK_BOX (hbox101), prefsw->checkbutton_show_stats, FALSE, FALSE, 0);

  gtk_widget_set_sensitive (prefsw->checkbutton_show_stats,prefs->video_player==VID_PLAYER_INT);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->checkbutton_show_stats),prefs->show_player_stats);

  hseparator13 = gtk_hseparator_new ();
  gtk_widget_show (hseparator13);
  gtk_box_pack_start (GTK_BOX (vbox69), hseparator13, TRUE, TRUE, 0);

  hbox31 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox31);
  gtk_box_pack_start (GTK_BOX (vbox69), hbox31, FALSE, FALSE, 0);

  label126 = gtk_label_new (_("Plugin      "));
  gtk_widget_show (label126);
  gtk_box_pack_start (GTK_BOX (hbox31), label126, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label126), GTK_JUSTIFY_LEFT);

  pp_combo = gtk_combo_new ();
  gtk_widget_show (pp_combo);
  gtk_box_pack_start (GTK_BOX (hbox31), pp_combo, FALSE, FALSE, 0);

  combo_entry4 = GTK_COMBO (pp_combo)->entry;
  gtk_editable_set_editable (GTK_EDITABLE(combo_entry4),FALSE);
  gtk_widget_show (combo_entry4);

  vid_playback_plugins=get_plugin_list (PLUGIN_VID_PLAYBACK,FALSE);
  vid_playback_plugins=g_list_prepend (vid_playback_plugins, "None");
  gtk_combo_set_popdown_strings (GTK_COMBO (pp_combo), vid_playback_plugins);
  g_list_free (vid_playback_plugins);

  if (vid_playback_plugin!=NULL) {
    gtk_entry_set_text (GTK_ENTRY(combo_entry4),vid_playback_plugin->name);
  }
  else {
    gtk_entry_set_text (GTK_ENTRY(combo_entry4),"None");
  }

  label128 = gtk_label_new (_("    Host "));
  gtk_widget_show (label128);
  gtk_box_pack_start (GTK_BOX (hbox31), label128, TRUE, TRUE, 6);
  gtk_label_set_justify (GTK_LABEL (label128), GTK_JUSTIFY_LEFT);

  entry20 = gtk_entry_new ();
  gtk_widget_show (entry20);
  gtk_entry_set_max_length(GTK_ENTRY(entry20),15);

  // fixed for now
  gtk_entry_set_text(GTK_ENTRY(entry20),"127.0.0.1");
  gtk_box_pack_start (GTK_BOX (hbox31), entry20, FALSE, FALSE, 0);
  gtk_entry_set_editable (GTK_ENTRY (entry20), FALSE);

  label129 = gtk_label_new (_("    Port"));
  gtk_widget_show (label129);
  gtk_box_pack_start (GTK_BOX (hbox31), label129, TRUE, TRUE, 6);
  gtk_label_set_justify (GTK_LABEL (label129), GTK_JUSTIFY_LEFT);

  entry21 = gtk_entry_new ();
  gtk_entry_set_max_length(GTK_ENTRY(entry21),5);

  // fixed for now
  gtk_entry_set_text(GTK_ENTRY(entry21),"N/A");
  gtk_widget_show (entry21);
  gtk_box_pack_start (GTK_BOX (hbox31), entry21, FALSE, FALSE, 0);
  gtk_entry_set_editable (GTK_ENTRY (entry21), FALSE);

  label31 = gtk_label_new (_("Video"));
  gtk_widget_show (label31);
  gtk_frame_set_label_widget (GTK_FRAME (frame4), label31);
  gtk_label_set_justify (GTK_LABEL (label31), GTK_JUSTIFY_LEFT);

  frame5 = gtk_frame_new (NULL);
  gtk_widget_show (frame5);
  gtk_box_pack_start (GTK_BOX (vbox9), frame5, TRUE, TRUE, 0);

  hbox10 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox10);
  gtk_container_add (GTK_CONTAINER (frame5), hbox10);
  gtk_container_set_border_width (GTK_CONTAINER (hbox10), 10);

  label35 = gtk_label_new (_("Player     "));
  gtk_widget_show (label35);
  gtk_box_pack_start (GTK_BOX (hbox10), label35, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label35), GTK_JUSTIFY_LEFT);

  audp = g_list_append (audp, _ ("sox (recommended)"));
  audp = g_list_append (audp, "mplayer");

  prefsw->audp_combo = gtk_combo_new ();
  gtk_combo_set_popdown_strings (GTK_COMBO (prefsw->audp_combo), audp);
  gtk_box_pack_start (GTK_BOX (hbox10), prefsw->audp_combo, FALSE, FALSE, 0);
  gtk_widget_show(prefsw->audp_combo);

  gtk_editable_set_editable (GTK_EDITABLE((GTK_COMBO(prefsw->audp_combo))->entry),FALSE);
  if (prefs->audio_player==AUD_PLAYER_SOX) {
    gtk_entry_set_text (GTK_ENTRY((GTK_COMBO(prefsw->audp_combo))->entry),_ ("sox (recommended)"));
  }
  else {
    gtk_entry_set_text (GTK_ENTRY((GTK_COMBO(prefsw->audp_combo))->entry),"mplayer");
  }

  label36 = gtk_label_new (_("     Audio play command      "));
  gtk_widget_show (label36);
  gtk_box_pack_start (GTK_BOX (hbox10), label36, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label36), GTK_JUSTIFY_LEFT);

  prefsw->audio_command_entry = gtk_entry_new ();
  gtk_entry_set_max_length(GTK_ENTRY(prefsw->audio_command_entry),255);
  gtk_widget_show (prefsw->audio_command_entry);
  gtk_box_pack_start (GTK_BOX (hbox10), prefsw->audio_command_entry, TRUE, TRUE, 0);

  // get from prefs
  gtk_entry_set_text(GTK_ENTRY(prefsw->audio_command_entry),prefs->audio_play_command);

  label32 = gtk_label_new (_("Audio"));
  gtk_widget_show (label32);
  gtk_frame_set_label_widget (GTK_FRAME (frame5), label32);
  gtk_label_set_justify (GTK_LABEL (label32), GTK_JUSTIFY_LEFT);

  label25 = gtk_label_new (_("Playback"));
  gtk_widget_show (label25);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 2), label25);
  gtk_label_set_justify (GTK_LABEL (label25), GTK_JUSTIFY_LEFT);

  vbox12 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox12);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), vbox12);
  gtk_container_set_border_width (GTK_CONTAINER (vbox12), 20);

  hbox11 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox11);
  gtk_box_pack_start (GTK_BOX (vbox12), hbox11, TRUE, TRUE, 0);

  label37 = gtk_label_new (_("      Encoder                  "));
  gtk_widget_show (label37);
  gtk_box_pack_start (GTK_BOX (hbox11), label37, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label37), GTK_JUSTIFY_LEFT);

  prefsw->encoder_combo = gtk_combo_new ();
  gtk_box_pack_start (GTK_BOX (hbox11), prefsw->encoder_combo, FALSE, FALSE, 0);

  prefsw->debug_encoders = gtk_check_button_new_with_mnemonic (_("_Debug Mode"));
  gtk_widget_show (prefsw->debug_encoders);

  hbox15 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox15);

  gtk_box_pack_start (GTK_BOX (vbox12), hbox15, TRUE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (hbox15), prefsw->debug_encoders, TRUE, FALSE, 0);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->debug_encoders),prefs->debug_encoders);

  prefsw->pal = gtk_radio_button_new_with_mnemonic (NULL, _("_PAL"));
  gtk_widget_show (prefsw->pal);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->pal), 18);
  gtk_box_pack_start (GTK_BOX (hbox15), prefsw->pal, FALSE, FALSE, 0);

  gtk_radio_button_set_group (GTK_RADIO_BUTTON (prefsw->pal), pal_ntsc);
  pal_ntsc = gtk_radio_button_get_group (GTK_RADIO_BUTTON (prefsw->pal));

  ntsc = gtk_radio_button_new_with_mnemonic (pal_ntsc, _("_NTSC"));
  gtk_widget_show (ntsc);
  gtk_container_set_border_width (GTK_CONTAINER (ntsc), 18);
  gtk_box_pack_start (GTK_BOX (hbox15), ntsc, FALSE, FALSE, 0);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ntsc),!prefs->is_pal);

  if (capable->has_encoder_plugins==TRUE) {
    // scan for encoder plugins
    if ((encoders=get_plugin_list (PLUGIN_ENCODERS,FALSE))==NULL) {
      do_plugin_error(PLUGIN_ENCODERS,NULL);
    }
    else {
      // TODO - check if prefs->img_ext == ".png", if so remove any encoders
      // which cannot handle png
      gtk_combo_set_popdown_strings (GTK_COMBO (prefsw->encoder_combo), encoders);
      g_list_free (encoders);
    }
  }
  else {
    gtk_widget_set_sensitive (prefsw->debug_encoders,FALSE);
  }

  gtk_widget_show(prefsw->encoder_combo);

  gtk_editable_set_editable (GTK_EDITABLE((GTK_COMBO(prefsw->encoder_combo))->entry),FALSE);
  gtk_entry_set_text (GTK_ENTRY((GTK_COMBO(prefsw->encoder_combo))->entry),prefs->encoder.name);

  hseparator8 = gtk_hseparator_new ();
  gtk_widget_show (hseparator8);
  gtk_box_pack_start (GTK_BOX (vbox12), hseparator8, TRUE, TRUE, 0);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox12), hbox, TRUE, TRUE, 0);

  label56 = gtk_label_new (_("Output format"));
  gtk_box_pack_start (GTK_BOX (hbox), label56, TRUE, FALSE, 0);
  
  prefsw->ofmt_combo = gtk_combo_new ();

  if (capable->has_encoder_plugins) {
    // reqest formats from the encoder plugin
    if ((ofmt_all=plugin_request_by_line(PLUGIN_ENCODERS,prefs->encoder.name,"get_formats"))!=NULL) {
      for (i=0;i<g_list_length(ofmt_all);i++) {
	if (get_token_count (g_list_nth_data (ofmt_all,i),'|')>2) {
	  array=g_strsplit (g_list_nth_data (ofmt_all,i),"|",-1);
	  if (!strcmp(array[0],prefs->encoder.of_name)) {
	    prefs->encoder.of_allowed_acodecs=atoi(array[2]);
	  }
	  ofmt=g_list_append(ofmt,g_strdup (array[1]));
	  g_strfreev (array);
	}
      }
      memcpy (&future_prefs->encoder,&prefs->encoder,sizeof(_encoder));
      gtk_combo_set_popdown_strings (GTK_COMBO (prefsw->ofmt_combo), ofmt);
      g_list_free(ofmt);
      g_list_free(ofmt_all);
    }
    else {
      do_plugin_error(PLUGIN_ENCODERS,prefs->encoder.name);
      future_prefs->encoder.of_allowed_acodecs=0;
    }
    
    gtk_box_pack_start (GTK_BOX (hbox), prefsw->ofmt_combo, TRUE, TRUE, 10);
    add_fill_to_box (GTK_BOX (hbox));
    gtk_widget_show_all(hbox);

    gtk_editable_set_editable (GTK_EDITABLE((GTK_COMBO(prefsw->ofmt_combo))->entry),FALSE);
    gtk_entry_set_text (GTK_ENTRY((GTK_COMBO(prefsw->ofmt_combo))->entry),prefs->encoder.of_name);
  
    prefsw->acodec_combo = gtk_combo_new ();
    prefsw->acodec_entry=(GtkWidget*)(GTK_ENTRY((GTK_COMBO(prefsw->acodec_combo))->entry));
    prefsw->acodec_list=NULL;

    set_acodec_list_from_allowed(prefsw);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox12), hbox, TRUE, TRUE, 0);

    label = gtk_label_new (_("Audio codec"));
    gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, FALSE, 10);
    
    gtk_box_pack_start (GTK_BOX (hbox), prefsw->acodec_combo, TRUE, TRUE, 10);
    add_fill_to_box (GTK_BOX (hbox));
    gtk_widget_show_all (hbox);

    gtk_editable_set_editable (GTK_EDITABLE(prefsw->acodec_entry),FALSE);
    gtk_label_set_mnemonic_widget (GTK_LABEL (label),prefsw->acodec_entry);
  }

  label = gtk_label_new (_("Encoding"));
  gtk_widget_show (label);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 3), label);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
  

  // effects

  vbox12 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox12);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), vbox12);
  gtk_container_set_border_width (GTK_CONTAINER (vbox12), 20);
  
  prefsw->checkbutton_antialias = gtk_check_button_new_with_mnemonic (_("Use _antialiasing when resizing"));
  gtk_widget_show (prefsw->checkbutton_antialias);
  gtk_box_pack_start (GTK_BOX (vbox12), prefsw->checkbutton_antialias, FALSE, FALSE, 10);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefsw->checkbutton_antialias),prefs->antialias);

  label = gtk_label_new (_("Effects"));
  gtk_widget_show (label);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 4), label);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);

  table4 = gtk_table_new (7, 3, FALSE);
  gtk_widget_show (table4);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), table4);
  gtk_container_set_border_width (GTK_CONTAINER (table4), 20);
  
  label39 = gtk_label_new (_("      Video load directory (default)     "));
  gtk_widget_show (label39);
  gtk_table_attach (GTK_TABLE (table4), label39, 0, 1, 0, 1,
		    (GtkAttachOptions) (GTK_FILL),
		    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label39), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label39), 0, 0.5);
  
  label40 = gtk_label_new (_("      Video save directory (default)"));
  gtk_widget_show (label40);
  gtk_table_attach (GTK_TABLE (table4), label40, 0, 1, 1, 2,
		    (GtkAttachOptions) (GTK_FILL),
		    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label40), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label40), 0, 0.5);
  
  label41 = gtk_label_new (_("      Audio load directory (default)"));
  gtk_widget_show (label41);
  gtk_table_attach (GTK_TABLE (table4), label41, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label41), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label41), 0, 0.5);
  
  label42 = gtk_label_new (_("      Image directory (default)"));
  gtk_widget_show (label42);
  gtk_table_attach (GTK_TABLE (table4), label42, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label42), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label42), 0, 0.5);
  
  label52 = gtk_label_new (_("      Backup/Restore directory (default)"));
  gtk_widget_show (label52);
  gtk_table_attach (GTK_TABLE (table4), label52, 0, 1, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label52), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label52), 0, 0.5);
  
  label43 = gtk_label_new (_("      Temp directory"));
  gtk_widget_show (label43);
  gtk_table_attach (GTK_TABLE (table4), label43, 0, 1, 5, 6,
		    (GtkAttachOptions) (GTK_FILL),
		    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label43), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label43), 0, 0.5);
  
  prefsw->vid_load_dir_entry = gtk_entry_new ();
  gtk_entry_set_max_length(GTK_ENTRY(prefsw->vid_load_dir_entry),255);
  gtk_widget_show (prefsw->vid_load_dir_entry);
  gtk_table_attach (GTK_TABLE (table4), prefsw->vid_load_dir_entry, 1, 2, 0, 1,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);
  
  // get from prefs
  gtk_entry_set_text(GTK_ENTRY(prefsw->vid_load_dir_entry),prefs->def_vid_load_dir);
  
  
  prefsw->vid_save_dir_entry = gtk_entry_new ();
  gtk_entry_set_max_length(GTK_ENTRY(prefsw->vid_save_dir_entry),255);
  gtk_widget_show (prefsw->vid_save_dir_entry);
  gtk_table_attach (GTK_TABLE (table4), prefsw->vid_save_dir_entry, 1, 2, 2, 3,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);
  
  // get from prefs
  gtk_entry_set_text(GTK_ENTRY(prefsw->vid_save_dir_entry),prefs->def_vid_save_dir);
  
  prefsw->audio_dir_entry = gtk_entry_new ();
  gtk_entry_set_max_length(GTK_ENTRY(prefsw->audio_dir_entry),255);
  gtk_widget_show (prefsw->audio_dir_entry);
  gtk_table_attach (GTK_TABLE (table4), prefsw->audio_dir_entry, 1, 2, 1, 2,
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		    (GtkAttachOptions) (0), 0, 0);
  
   
   // get from prefs
   gtk_entry_set_text(GTK_ENTRY(prefsw->audio_dir_entry),prefs->def_audio_dir);
   
   prefsw->image_dir_entry = gtk_entry_new ();
   gtk_entry_set_max_length(GTK_ENTRY(prefsw->image_dir_entry),255);
   gtk_widget_show (prefsw->image_dir_entry);
   gtk_table_attach (GTK_TABLE (table4), prefsw->image_dir_entry, 1, 2, 3, 4,
		     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		     (GtkAttachOptions) (0), 0, 0);
   
   // get from prefs
   gtk_entry_set_text(GTK_ENTRY(prefsw->image_dir_entry),prefs->def_image_dir);
   
   prefsw->proj_dir_entry = gtk_entry_new ();
   gtk_entry_set_max_length(GTK_ENTRY(prefsw->proj_dir_entry),255);
   gtk_widget_show (prefsw->proj_dir_entry);
   gtk_table_attach (GTK_TABLE (table4), prefsw->proj_dir_entry, 1, 2, 4, 5,
		     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		     (GtkAttachOptions) (0), 0, 0);
   
   // get from prefs
   gtk_entry_set_text(GTK_ENTRY(prefsw->proj_dir_entry),prefs->def_proj_dir);
   
   prefsw->tmpdir_entry = gtk_entry_new ();
   gtk_entry_set_max_length(GTK_ENTRY(prefsw->tmpdir_entry),255);
   gtk_widget_show (prefsw->tmpdir_entry);
   gtk_table_attach (GTK_TABLE (table4), prefsw->tmpdir_entry, 1, 2, 5, 6,
		     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		     (GtkAttachOptions) (0), 0, 0);
   
   // get from prefs
   gtk_entry_set_text(GTK_ENTRY(prefsw->tmpdir_entry),future_prefs->tmpdir);
   
   dirbutton1 = gtk_button_new ();
   gtk_widget_show (dirbutton1);
   
   
   dirimage1 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
   gtk_widget_show (dirimage1);
   gtk_container_add (GTK_CONTAINER (dirbutton1), dirimage1);
   
   gtk_table_attach (GTK_TABLE (table4), dirbutton1, 2, 3, 0, 1,
		     (GtkAttachOptions) (0),
		     (GtkAttachOptions) (0), 0, 0);
   
   
   dirbutton2 = gtk_button_new ();
   gtk_widget_show (dirbutton2);
   
   
   dirimage2 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
   gtk_widget_show (dirimage2);
   gtk_container_add (GTK_CONTAINER (dirbutton2), dirimage2);
   
   gtk_table_attach (GTK_TABLE (table4), dirbutton2, 2, 3, 1, 2,
		     (GtkAttachOptions) (0),
		     (GtkAttachOptions) (0), 0, 0);
   
   
   dirbutton3 = gtk_button_new ();
   gtk_widget_show (dirbutton3);

   
   dirimage3 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
   gtk_widget_show (dirimage3);
   gtk_container_add (GTK_CONTAINER (dirbutton3), dirimage3);
   
   gtk_table_attach (GTK_TABLE (table4), dirbutton3, 2, 3, 2, 3,
		     (GtkAttachOptions) (0),
		     (GtkAttachOptions) (0), 0, 0);
   
   
   dirbutton4 = gtk_button_new ();
   gtk_widget_show (dirbutton4);
   
   
   dirimage4 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
   gtk_widget_show (dirimage4);
   gtk_container_add (GTK_CONTAINER (dirbutton4), dirimage4);
   
   gtk_table_attach (GTK_TABLE (table4), dirbutton4, 2, 3, 3, 4,
		     (GtkAttachOptions) (0),
		     (GtkAttachOptions) (0), 0, 0);
   

   dirbutton5 = gtk_button_new ();
   gtk_widget_show (dirbutton5);

   
   dirimage5 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
   gtk_widget_show (dirimage5);
  gtk_container_add (GTK_CONTAINER (dirbutton5), dirimage5);
  
  gtk_table_attach (GTK_TABLE (table4), dirbutton5, 2, 3, 4, 5,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  
  
  dirbutton6 = gtk_button_new ();
  gtk_widget_show (dirbutton6);


  dirimage6 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (dirimage6);
  gtk_container_add (GTK_CONTAINER (dirbutton6), dirimage6);

  gtk_table_attach (GTK_TABLE (table4), dirbutton6, 2, 3, 5, 6,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);


  label27 = gtk_label_new (_("Directories"));
  gtk_widget_show (label27);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 5), label27);
  gtk_label_set_justify (GTK_LABEL (label27), GTK_JUSTIFY_LEFT);

  vbox18 = gtk_vbox_new (FALSE, 10);
  gtk_widget_show (vbox18);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), vbox18);
  gtk_container_set_border_width (GTK_CONTAINER (vbox18), 20);

  prefsw->checkbutton_warn_fps = gtk_check_button_new_with_mnemonic (_("Warn on Insert / Merge if _frame rate of clipboard does not match frame rate of selection"));
  gtk_widget_show (prefsw->checkbutton_warn_fps);
  gtk_box_pack_start (GTK_BOX (vbox18), prefsw->checkbutton_warn_fps, FALSE, FALSE, 0);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->checkbutton_warn_fps), !(prefs->warning_mask&WARN_MASK_FPS));

  hbox100 = gtk_hbox_new(FALSE,10);
  gtk_box_pack_start (GTK_BOX (vbox18), hbox100, FALSE, FALSE, 0);
  gtk_widget_show(hbox100);
  prefsw->checkbutton_warn_fsize = gtk_check_button_new_with_mnemonic (_("Warn on Open if file _size exceeds "));
  gtk_widget_show (prefsw->checkbutton_warn_fsize);
  gtk_box_pack_start (GTK_BOX (hbox100), prefsw->checkbutton_warn_fsize, FALSE, TRUE, 0);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->checkbutton_warn_fsize), !(prefs->warning_mask&WARN_MASK_FSIZE));
  spinbutton_warn_fsize_adj = gtk_adjustment_new (prefs->warn_file_size, 1, 2048, 1, 10, 100);

  prefsw->spinbutton_warn_fsize = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_warn_fsize_adj), 1, 0);
  gtk_widget_show (prefsw->spinbutton_warn_fsize);
  gtk_box_pack_start (GTK_BOX (hbox100), prefsw->spinbutton_warn_fsize, FALSE, TRUE, 0);

  label100 = gtk_label_new (_ (" MB"));
  gtk_widget_show (label100);
  gtk_box_pack_start (GTK_BOX (hbox100), label100, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label100), GTK_JUSTIFY_LEFT);

  prefsw->checkbutton_warn_save_quality = gtk_check_button_new_with_mnemonic (_("Warn about loss of _quality when saving to an existing file"));
  gtk_widget_show (prefsw->checkbutton_warn_save_quality);
  gtk_box_pack_start (GTK_BOX (vbox18), prefsw->checkbutton_warn_save_quality, FALSE, TRUE, 0);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->checkbutton_warn_save_quality), !(prefs->warning_mask&WARN_MASK_SAVE_QUALITY));

  prefsw->checkbutton_warn_set = gtk_check_button_new_with_mnemonic (_("Show a warning before saving a se_t"));
  gtk_widget_show (prefsw->checkbutton_warn_set);
  gtk_box_pack_start (GTK_BOX (vbox18), prefsw->checkbutton_warn_set, FALSE, TRUE, 0);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->checkbutton_warn_set), !(prefs->warning_mask&WARN_MASK_SAVE_SET));

  label86 = gtk_label_new ("");
  gtk_widget_show (label86);
  gtk_box_pack_start (GTK_BOX (vbox18), label86, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label86), GTK_JUSTIFY_LEFT);

  label87 = gtk_label_new ("");
  gtk_widget_show (label87);
  gtk_box_pack_start (GTK_BOX (vbox18), label87, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label87), GTK_JUSTIFY_LEFT);

  label84 = gtk_label_new (_("Warnings"));
  gtk_widget_show (label84);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 6), label84);
  gtk_label_set_justify (GTK_LABEL (label84), GTK_JUSTIFY_LEFT);

  vbox14 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox14);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), vbox14);
  gtk_container_set_border_width (GTK_CONTAINER (vbox14), 20);

  prefsw->check_midi = gtk_check_button_new_with_mnemonic (_("Midi synch (requires the files midistart and midistop)"));
  gtk_widget_show (prefsw->check_midi);
  gtk_box_pack_start (GTK_BOX (vbox14), prefsw->check_midi, FALSE, FALSE, 16);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->check_midi), 10);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->check_midi), prefs->midisynch);
  gtk_widget_set_sensitive(prefsw->check_midi,capable->has_midistartstop);

  hbox99 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox99);
  gtk_box_pack_start (GTK_BOX (vbox14), hbox99, TRUE, TRUE, 0);

  label97 = gtk_label_new (_("When inserting/merging frames:  "));
  gtk_widget_show (label97);
  gtk_box_pack_start (GTK_BOX (hbox99), label97, FALSE, FALSE, 16);
  gtk_label_set_justify (GTK_LABEL (label97), GTK_JUSTIFY_LEFT);

  prefsw->ins_speed = gtk_radio_button_new_with_mnemonic (NULL, _("_Speed Up/Slow Down Insertion"));
  gtk_widget_show (prefsw->ins_speed);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->ins_speed), 18);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (prefsw->ins_speed), rb_group2);
  rb_group2 = gtk_radio_button_get_group (GTK_RADIO_BUTTON (prefsw->ins_speed));
  gtk_box_pack_start (GTK_BOX (hbox99), prefsw->ins_speed, FALSE, FALSE, 0);

  ins_resample = gtk_radio_button_new_with_mnemonic (NULL, _("_Resample Insertion"));
  gtk_widget_show (ins_resample);
  gtk_container_set_border_width (GTK_CONTAINER (ins_resample), 18);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (ins_resample), rb_group2);
  rb_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (ins_resample));
  gtk_box_pack_start (GTK_BOX (hbox99), ins_resample, FALSE, FALSE, 0);
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(ins_resample),prefs->ins_resample);

  prefsw->check_xmms_pause = gtk_check_button_new_with_mnemonic (_("Pause xmms during audio playback"));
  gtk_box_pack_start (GTK_BOX (vbox14), prefsw->check_xmms_pause, FALSE, FALSE, 16);
  gtk_container_set_border_width (GTK_CONTAINER (prefsw->check_xmms_pause), 10);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefsw->check_xmms_pause), prefs->pause_xmms);

  if (capable->has_xmms==TRUE) {
    gtk_widget_show (prefsw->check_xmms_pause);
  }

  hbox19 = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox14), hbox19, TRUE, TRUE, 0);
  
  label134 = gtk_label_new (_("CD device           "));
  gtk_widget_show (label134);
  gtk_box_pack_start (GTK_BOX (hbox19), label134, FALSE, FALSE, 18);
  gtk_label_set_justify (GTK_LABEL (label134), GTK_JUSTIFY_LEFT);
  
  prefsw->cdplay_entry = gtk_entry_new ();
  gtk_entry_set_max_length(GTK_ENTRY(prefsw->cdplay_entry),255);
  gtk_box_pack_start (GTK_BOX (hbox19), prefsw->cdplay_entry, FALSE, TRUE, 0);

  if (capable->has_cdda2wav==TRUE) {
    gtk_widget_show (prefsw->cdplay_entry);
    gtk_widget_show (hbox19);
  }
  else {
    gtk_widget_hide (prefsw->cdplay_entry);
    gtk_widget_hide (hbox19);
  }

  // get from prefs
  gtk_entry_set_text(GTK_ENTRY(prefsw->cdplay_entry),prefs->cdplay_device);

  gtk_tooltips_set_tip (tooltips, prefsw->cdplay_entry, _("LiVES can load audio tracks from this CD"), NULL);

  hbox13 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox13);
  gtk_box_pack_start (GTK_BOX (vbox14), hbox13, TRUE, TRUE, 0);

  label44 = gtk_label_new (_("Default FPS        "));
  gtk_widget_show (label44);
  gtk_box_pack_start (GTK_BOX (hbox13), label44, FALSE, FALSE, 18);
  gtk_label_set_justify (GTK_LABEL (label44), GTK_JUSTIFY_LEFT);

  spinbutton_def_fps_adj = gtk_adjustment_new (prefs->default_fps, 1, 2048, 1, 10, 10);

  prefsw->spinbutton_def_fps = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_def_fps_adj), 1, 2);
  gtk_widget_show (prefsw->spinbutton_def_fps);
  gtk_box_pack_start (GTK_BOX (hbox13), prefsw->spinbutton_def_fps, FALSE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, prefsw->spinbutton_def_fps, _("Frames per second to use when none is specified"), NULL);

  label29 = gtk_label_new (_("Misc"));
  gtk_widget_show (label29);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 7), label29);
  gtk_label_set_justify (GTK_LABEL (label29), GTK_JUSTIFY_LEFT);

  vbox99 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox99);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), vbox99);
  gtk_container_set_border_width (GTK_CONTAINER (vbox99), 20);

  hbox93 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox93);
  gtk_box_pack_start (GTK_BOX (vbox99), hbox93, TRUE, TRUE, 0);

  label94 = gtk_label_new (_("New theme:           "));
  gtk_widget_show (label94);
  gtk_box_pack_start (GTK_BOX (hbox93), label94, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label94), GTK_JUSTIFY_LEFT);

  prefsw->theme_combo = gtk_combo_new ();


  // scan for themes
  themes=get_plugin_list (PLUGIN_THEMES,FALSE);
  themes=g_list_prepend (themes, "none");

  gtk_combo_set_popdown_strings (GTK_COMBO (prefsw->theme_combo), themes);
  g_list_free (themes);

  gtk_box_pack_start (GTK_BOX (hbox93), prefsw->theme_combo, FALSE, FALSE, 0);
  gtk_widget_show(prefsw->theme_combo);

  gtk_editable_set_editable (GTK_EDITABLE((GTK_COMBO(prefsw->theme_combo))->entry),FALSE);
  gtk_entry_set_text (GTK_ENTRY((GTK_COMBO(prefsw->theme_combo))->entry),future_prefs->theme);

  label99 = gtk_label_new (_("Themes"));
  gtk_widget_show (label99);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 8), label99);
  gtk_label_set_justify (GTK_LABEL (label99), GTK_JUSTIFY_LEFT);

  vbox98 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox98);
  gtk_container_add (GTK_CONTAINER (prefsw->prefs_notebook), vbox98);
  gtk_container_set_border_width (GTK_CONTAINER (vbox98), 20);

  hbox94 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox94);
  gtk_box_pack_start (GTK_BOX (vbox98), hbox94, TRUE, TRUE, 0);

  label88 = gtk_label_new (_("Download bandwidth (Kb/s)       "));
  gtk_widget_show (label88);
  gtk_box_pack_start (GTK_BOX (hbox94), label88, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label88), GTK_JUSTIFY_LEFT);

  spinbutton_bwidth_adj = gtk_adjustment_new (prefs->dl_bandwidth, 0, 100000, 1, 10, 100);

  prefsw->spinbutton_bwidth = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_bwidth_adj), 1, 0);
  gtk_widget_show (prefsw->spinbutton_bwidth);
  gtk_box_pack_start (GTK_BOX (hbox94), prefsw->spinbutton_bwidth, FALSE, TRUE, 0);

  label98 = gtk_label_new (_("Streaming"));
  gtk_widget_show (label98);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (prefsw->prefs_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (prefsw->prefs_notebook), 9), label98);
  gtk_label_set_justify (GTK_LABEL (label98), GTK_JUSTIFY_LEFT);

  dialog_action_area8 = GTK_DIALOG (prefsw->prefs_dialog)->action_area;
  gtk_widget_show (dialog_action_area8);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area8), GTK_BUTTONBOX_END);

  prefsw->cancelbutton = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (prefsw->cancelbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (prefsw->prefs_dialog), prefsw->cancelbutton, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (prefsw->cancelbutton, GTK_CAN_DEFAULT);

  applybutton = gtk_button_new_from_stock ("gtk-apply");
  gtk_widget_show (applybutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (prefsw->prefs_dialog), applybutton, 0);
  GTK_WIDGET_SET_FLAGS (applybutton, GTK_CAN_DEFAULT);

  okbutton5 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton5);
  gtk_dialog_add_action_widget (GTK_DIALOG (prefsw->prefs_dialog), okbutton5, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton5, GTK_CAN_DEFAULT);


  g_signal_connect(dirbutton1, "clicked", G_CALLBACK (on_filesel_simple_clicked),prefsw->vid_load_dir_entry);
  g_signal_connect(dirbutton2, "clicked", G_CALLBACK (on_filesel_simple_clicked),prefsw->vid_save_dir_entry);
  g_signal_connect(dirbutton3, "clicked", G_CALLBACK (on_filesel_simple_clicked),prefsw->audio_dir_entry);
  g_signal_connect(dirbutton4, "clicked", G_CALLBACK (on_filesel_simple_clicked),prefsw->image_dir_entry);
  g_signal_connect(dirbutton5, "clicked", G_CALLBACK (on_filesel_simple_clicked),prefsw->proj_dir_entry);
  g_signal_connect(dirbutton6, "clicked", G_CALLBACK (on_filesel_complex_clicked),prefsw->tmpdir_entry);


  g_signal_connect (GTK_OBJECT((GTK_COMBO(prefsw->vidp_combo))->entry),"changed",
		    G_CALLBACK (on_vidp_entry_changed),
		    NULL);

  if (capable->has_encoder_plugins==TRUE) {
    prefsw->encoder_name_fn=g_signal_connect (GTK_OBJECT((GTK_COMBO(prefsw->encoder_combo))->entry),"changed",G_CALLBACK (on_encoder_entry_changed),NULL);
    prefsw->encoder_ofmt_fn=g_signal_connect (GTK_OBJECT((GTK_COMBO(prefsw->ofmt_combo))->entry),"changed",G_CALLBACK (on_encoder_ofmt_changed),NULL);
  }
  g_signal_connect (GTK_OBJECT(combo_entry4),"changed",G_CALLBACK (on_vid_playback_plugin_changed),NULL);

  g_signal_connect (GTK_OBJECT (prefsw->cancelbutton), "clicked",
		    G_CALLBACK (on_cancel_button1_clicked),
		    prefsw);


  g_signal_connect (GTK_OBJECT (okbutton5), "clicked",
		    G_CALLBACK (on_prefs_ok_clicked),
		    NULL);

  g_signal_connect (GTK_OBJECT (applybutton), "clicked",
		    G_CALLBACK (on_prefs_apply_clicked),
		    NULL);

  g_signal_connect (GTK_OBJECT (prefsw->prefs_dialog), "delete_event",
		    G_CALLBACK (on_prefs_delete_event),
		    prefsw);

  g_list_free (vidp);
  g_list_free (audp);

  return prefsw;
}



_locw*
create_location_dialog (void)
{
  GtkWidget *dialog_vbox13;
  GtkWidget *hbox19;
  GtkWidget *label83;
  GtkWidget *dialog_action_area13;
  GtkWidget *cancelbutton11;
  GtkWidget *okbutton10;
  GtkWidget *label;
  GtkWidget *nobw;

  _locw *locw=(_locw*)(g_malloc(sizeof(_locw)));

  locw->loc_dialog = gtk_dialog_new ();
  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(locw->loc_dialog),FALSE);
  }
  gtk_widget_modify_bg (locw->loc_dialog, GTK_STATE_NORMAL, &palette->normal_back);
  gtk_window_set_transient_for(GTK_WINDOW(locw->loc_dialog),GTK_WINDOW(mainw->LiVES));

  gtk_window_set_position (GTK_WINDOW (locw->loc_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (locw->loc_dialog), TRUE);

  gtk_window_set_default_size (GTK_WINDOW (locw->loc_dialog), 300, 200);
  
  gtk_container_set_border_width (GTK_CONTAINER (locw->loc_dialog), 10);
  gtk_window_set_title (GTK_WINDOW (locw->loc_dialog), _("LiVES: - Open Location"));

  dialog_vbox13 = GTK_DIALOG (locw->loc_dialog)->vbox;
  gtk_widget_show (dialog_vbox13);

  label = gtk_label_new (_("\n\nTo open a stream, you must make sure that you have the correct libraries compiled in mplayer.\nAlso make sure you have set your bandwidth in Preferences|Streaming"));
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (dialog_vbox13), label, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_CENTER);
  gtk_widget_modify_fg (label, GTK_STATE_NORMAL, &palette->normal_fore);

  hbox19 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox19);
  gtk_box_pack_start (GTK_BOX (dialog_vbox13), hbox19, TRUE, TRUE, 0);

  label83 = gtk_label_new (_ ("URL : "));
  gtk_widget_show (label83);
  gtk_box_pack_start (GTK_BOX (hbox19), label83, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label83), GTK_JUSTIFY_LEFT);
  gtk_widget_modify_fg (label83, GTK_STATE_NORMAL, &palette->normal_fore);

  locw->entry = gtk_entry_new_with_max_length (255);
  gtk_widget_show (locw->entry);
  gtk_box_pack_start (GTK_BOX (hbox19), locw->entry, TRUE, TRUE, 0);
  gtk_entry_set_activates_default (GTK_ENTRY (locw->entry), TRUE);

  nobw = gtk_check_button_new_with_mnemonic (_("Do not send bandwidth information"));
  gtk_widget_show (nobw);
  gtk_box_pack_start (GTK_BOX (dialog_vbox13), nobw, FALSE, FALSE, 0);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(nobw),prefs->no_bandwidth);
  gtk_widget_modify_fg (nobw, GTK_STATE_NORMAL, &palette->normal_fore);
  gtk_tooltips_set_tip (mainw->tooltips, nobw,_("Try this setting if you are having problems getting a stream"), NULL);

  dialog_action_area13 = GTK_DIALOG (locw->loc_dialog)->action_area;
  gtk_widget_show (dialog_action_area13);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area13), GTK_BUTTONBOX_END);

  cancelbutton11 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton11);
  gtk_dialog_add_action_widget (GTK_DIALOG (locw->loc_dialog), cancelbutton11, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton11, GTK_CAN_DEFAULT);

  okbutton10 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton10);
  gtk_dialog_add_action_widget (GTK_DIALOG (locw->loc_dialog), okbutton10, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton10, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (okbutton10);


  g_signal_connect (GTK_OBJECT (cancelbutton11), "clicked",
		    G_CALLBACK (on_cancel_button1_clicked),
		    locw);

  g_signal_connect (GTK_OBJECT (okbutton10), "clicked",
		    G_CALLBACK (on_location_select),
		    NULL);

  g_signal_connect (GTK_OBJECT (locw->loc_dialog), "delete_event",
                      G_CALLBACK (return_true),
                      NULL);

  g_signal_connect (GTK_OBJECT (nobw), "toggled",
		    G_CALLBACK (on_locbw_toggled),
		    NULL);

  return locw;
}



_renamew*
create_rename_dialog (gint type)
{
  GtkWidget *dialog_vbox13;
  GtkWidget *hbox19;
  GtkWidget *label83;
  GtkWidget *dialog_action_area13;
  GtkWidget *cancelbutton11;
  GtkWidget *okbutton10;

  _renamew *renamew=(_renamew*)(g_malloc(sizeof(_renamew)));

  renamew->rename_dialog = gtk_dialog_new ();

  gtk_window_set_position (GTK_WINDOW (renamew->rename_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_transient_for(GTK_WINDOW(renamew->rename_dialog),GTK_WINDOW(mainw->LiVES));
  gtk_window_set_modal (GTK_WINDOW (renamew->rename_dialog), TRUE);

  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(renamew->rename_dialog),FALSE);
  }
  gtk_widget_modify_bg (renamew->rename_dialog, GTK_STATE_NORMAL, &palette->normal_back);

  gtk_window_set_default_size (GTK_WINDOW (renamew->rename_dialog), 300, 200);

  gtk_container_set_border_width (GTK_CONTAINER (renamew->rename_dialog), 10);

  if (type==1) {
    gtk_window_set_title (GTK_WINDOW (renamew->rename_dialog), _("LiVES: - Rename Clip"));
  }
  else if (type==2||type==3) {
    gtk_window_set_title (GTK_WINDOW (renamew->rename_dialog), _("LiVES: - Enter Set Name"));
  }

  dialog_vbox13 = GTK_DIALOG (renamew->rename_dialog)->vbox;
  gtk_widget_show (dialog_vbox13);

  hbox19 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox19);
  gtk_box_pack_start (GTK_BOX (dialog_vbox13), hbox19, TRUE, TRUE, 0);

  if (type==1) {
    label83 = gtk_label_new (_("New name "));
  }
  else if (type==2||type==3) {
    label83 = gtk_label_new (_("Set name "));
  }
  else {
    label83 = gtk_label_new ("");
  }
  gtk_widget_show (label83);
  gtk_box_pack_start (GTK_BOX (hbox19), label83, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label83), GTK_JUSTIFY_LEFT);
  gtk_widget_modify_fg (label83, GTK_STATE_NORMAL, &palette->normal_fore);

  renamew->entry = gtk_entry_new_with_max_length (255);
  if (type==2&&strlen (mainw->set_name)) {
    gtk_entry_set_text (GTK_ENTRY (renamew->entry),mainw->set_name);
  }

  gtk_widget_show (renamew->entry);
  gtk_box_pack_start (GTK_BOX (hbox19), renamew->entry, TRUE, TRUE, 0);
  gtk_entry_set_activates_default (GTK_ENTRY (renamew->entry), TRUE);

  dialog_action_area13 = GTK_DIALOG (renamew->rename_dialog)->action_area;
  gtk_widget_show (dialog_action_area13);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area13), GTK_BUTTONBOX_END);

  cancelbutton11 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton11);
  gtk_dialog_add_action_widget (GTK_DIALOG (renamew->rename_dialog), cancelbutton11, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton11, GTK_CAN_DEFAULT);

  okbutton10 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton10);
  gtk_dialog_add_action_widget (GTK_DIALOG (renamew->rename_dialog), okbutton10, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton10, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (okbutton10);


  g_signal_connect (GTK_OBJECT (cancelbutton11), "clicked",
		    G_CALLBACK (on_cancel_button1_clicked),
		    renamew);
  if (type==1) {
    g_signal_connect (GTK_OBJECT (okbutton10), "clicked",
		      G_CALLBACK (on_rename_set_name),
		      NULL);
  }
  else if (type==2) {
    g_signal_connect (GTK_OBJECT (okbutton10), "clicked",
		      G_CALLBACK (on_save_set_ok),
		      NULL);
  }
  else if (type==3) {
    g_signal_connect (GTK_OBJECT (okbutton10), "clicked",
		      G_CALLBACK (on_load_set_ok),
		      NULL);
  }


  g_signal_connect (GTK_OBJECT (renamew->rename_dialog), "delete_event",
                      G_CALLBACK (return_true),
                      NULL);

  return renamew;
}





GtkWidget*
create_cdtrack_dialog (gint type)
{
  // type 0 = cd track
  // type 1 = dvd title/chapter
  // type 2 = vcd title -- do we need chapter as well ?

  // TODO - add pref for dvd/vcd device
  // TODO - prompt for audio channel

  // TODO - for CD make this nicer - get track names

  GtkWidget *cd_dialog;
  GtkWidget *dialog_vbox11;
  GtkWidget *hbox16;
  GtkWidget *hbox17;
  GtkWidget *label61;
  GtkWidget *label62;
  GtkObject *spinbutton35_adj;
  GtkWidget *spinbutton35;
  GtkObject *spinbutton36_adj;
  GtkWidget *spinbutton36;
  GtkWidget *dialog_action_area11;
  GtkWidget *cancelbutton9;
  GtkWidget *okbutton8;
  gchar *label;
  
  cd_dialog = gtk_dialog_new ();
  if (type==0) {
    gtk_window_set_title (GTK_WINDOW (cd_dialog), _("LiVES:- Load CD Track"));
  }
  else if (type==1) {
    gtk_window_set_title (GTK_WINDOW (cd_dialog), _("LiVES:- Select DVD Title/Chapter"));
  }
  else {
    gtk_window_set_title (GTK_WINDOW (cd_dialog), _("LiVES:- Select VCD Title"));
  }

  gtk_window_set_position (GTK_WINDOW (cd_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (cd_dialog), TRUE);

  gtk_container_set_border_width (GTK_CONTAINER (cd_dialog), 10);
  gtk_window_set_default_size (GTK_WINDOW (cd_dialog), 300, 200);
  gtk_window_set_transient_for(GTK_WINDOW(cd_dialog),GTK_WINDOW(mainw->LiVES));

  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(cd_dialog),FALSE);
  }
  gtk_widget_modify_bg (cd_dialog, GTK_STATE_NORMAL, &palette->normal_back);

  dialog_vbox11 = GTK_DIALOG (cd_dialog)->vbox;
  gtk_widget_show (dialog_vbox11);

  hbox16 = gtk_hbox_new (FALSE, 50);
  gtk_widget_show (hbox16);
  gtk_box_pack_start (GTK_BOX (dialog_vbox11), hbox16, TRUE, TRUE, 0);

  if (type==0) {
    label=g_strdup_printf(_("Track to load (from %s)"),prefs->cdplay_device);
  }
  else if (type==1) {
    label=g_strdup_printf(_("DVD Title"));
  }
  else {
    label=g_strdup_printf(_("VCD Title"));
  }
  label61 = gtk_label_new (label);
  g_free(label);
  gtk_widget_modify_fg (label61, GTK_STATE_NORMAL, &palette->normal_fore);

  gtk_widget_show (label61);
  gtk_box_pack_start (GTK_BOX (hbox16), label61, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label61), GTK_JUSTIFY_LEFT);

  spinbutton35_adj = gtk_adjustment_new (mainw->fx1_val, 1, 256, 1, 10, 10);
  spinbutton35 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton35_adj), 1, 0);
  gtk_widget_show (spinbutton35);
  gtk_box_pack_start (GTK_BOX (hbox16), spinbutton35, FALSE, TRUE, 0);

  hbox17 = gtk_hbox_new (FALSE, 50);
  spinbutton36_adj = gtk_adjustment_new (mainw->fx2_val, 1, 1024, 1, 10, 10);
  spinbutton36 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton36_adj), 1, 0);

  if (type==1) {
    gtk_widget_show (hbox17);
    gtk_box_pack_start (GTK_BOX (dialog_vbox11), hbox17, TRUE, TRUE, 0);

    label62=gtk_label_new(_("Chapter  "));
    gtk_widget_modify_fg (label62, GTK_STATE_NORMAL, &palette->normal_fore);

    gtk_widget_show (label62);
    gtk_box_pack_start (GTK_BOX (hbox17), label62, FALSE, FALSE, 0);
    gtk_label_set_justify (GTK_LABEL (label62), GTK_JUSTIFY_LEFT);

    gtk_widget_show (spinbutton36);
    gtk_box_pack_start (GTK_BOX (hbox17), spinbutton36, FALSE, TRUE, 0);
  }

  dialog_action_area11 = GTK_DIALOG (cd_dialog)->action_area;
  gtk_widget_show (dialog_action_area11);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area11), GTK_BUTTONBOX_END);

  cancelbutton9 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton9);
  gtk_dialog_add_action_widget (GTK_DIALOG (cd_dialog), cancelbutton9, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton9, GTK_CAN_DEFAULT);

  okbutton8 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton8);
  gtk_dialog_add_action_widget (GTK_DIALOG (cd_dialog), okbutton8, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton8, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (okbutton8);
  
  g_signal_connect_after (GTK_OBJECT (spinbutton35), "value_changed",
			  G_CALLBACK (on_spin_value_changed),
			  GINT_TO_POINTER (1));

  g_signal_connect_after (GTK_OBJECT (spinbutton36), "value_changed",
			  G_CALLBACK (on_spin_value_changed),
			  GINT_TO_POINTER (2));

  g_signal_connect (GTK_OBJECT (cancelbutton9), "clicked",
		    G_CALLBACK (on_cancel_button1_clicked),
		    NULL);

  if (type==0) {
    g_signal_connect (GTK_OBJECT (okbutton8), "clicked",
		      G_CALLBACK (on_load_cdtrack_ok_clicked),
		      NULL);
  }
  else {
    g_signal_connect (GTK_OBJECT (okbutton8), "clicked",
		      G_CALLBACK (on_load_vcd_ok_clicked),
		      GINT_TO_POINTER (type));
  }
  return cd_dialog;
}




_xranw *
create_rp_dialog (void)
{
  GtkWidget *dialog_vbox14;
  GtkWidget *vbox19;
  GtkWidget *hbox21;
  GtkWidget *label89;
  GtkObject *spinbutton50_adj;
  GtkWidget *hbox20;
  GtkWidget *label88;
  GtkWidget *button13;
  GtkWidget *image413;
  GtkWidget *hbox22;
  GtkWidget *label90;
  GtkObject *spinbutton51_adj;
  GtkWidget *hbox23;
  GtkWidget *label91;
  GtkObject *spinbutton52_adj;
  GtkWidget *dialog_action_area14;
  GtkWidget *cancelbutton12;
  GtkWidget *okbutton11;

  _xranw *xranw=(_xranw*)(g_malloc(sizeof(_xranw)));

  xranw->rp_dialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (xranw->rp_dialog), _("LiVES: - Play Random Music"));

  dialog_vbox14 = GTK_DIALOG (xranw->rp_dialog)->vbox;
  gtk_widget_show (dialog_vbox14);
  gtk_window_set_transient_for(GTK_WINDOW(xranw->rp_dialog),GTK_WINDOW(mainw->LiVES));

  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(xranw->rp_dialog),FALSE);
  }
  gtk_widget_modify_bg (xranw->rp_dialog, GTK_STATE_NORMAL, &palette->normal_back);

  gtk_window_set_position (GTK_WINDOW (xranw->rp_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (xranw->rp_dialog), TRUE);

  gtk_container_set_border_width (GTK_CONTAINER (xranw->rp_dialog), 10);
  gtk_window_set_default_size (GTK_WINDOW (xranw->rp_dialog), 600, 300);

  vbox19 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox19);
  gtk_box_pack_start (GTK_BOX (dialog_vbox14), vbox19, TRUE, TRUE, 0);

  hbox21 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox21);
  gtk_box_pack_start (GTK_BOX (vbox19), hbox21, TRUE, TRUE, 0);

  label89 = gtk_label_new (_("Number of Tracks Requested:    "));
  gtk_widget_show (label89);
  gtk_box_pack_start (GTK_BOX (hbox21), label89, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label89), GTK_JUSTIFY_LEFT);
  gtk_widget_modify_fg (label89, GTK_STATE_NORMAL, &palette->normal_fore);

  spinbutton50_adj = gtk_adjustment_new (10, 1, 100, 1, 10, 10);
  xranw->numtracks = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton50_adj), 1, 0);
  gtk_widget_show (xranw->numtracks);
  gtk_box_pack_start (GTK_BOX (hbox21), xranw->numtracks, FALSE, FALSE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (xranw->numtracks), TRUE);
  gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (xranw->numtracks), GTK_UPDATE_IF_VALID);

  hbox20 = gtk_hbox_new (FALSE, 16);
  gtk_widget_show (hbox20);
  gtk_box_pack_start (GTK_BOX (vbox19), hbox20, TRUE, TRUE, 0);

  label88 = gtk_label_new (_("Directory:"));
  gtk_widget_show (label88);
  gtk_box_pack_start (GTK_BOX (hbox20), label88, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label88), GTK_JUSTIFY_LEFT);
  gtk_widget_modify_fg (label88, GTK_STATE_NORMAL, &palette->normal_fore);

  xranw->dir = gtk_entry_new ();
  gtk_widget_show (xranw->dir);
  gtk_box_pack_start (GTK_BOX (hbox20), xranw->dir, TRUE, TRUE, 0);
  gtk_entry_set_text(GTK_ENTRY(xranw->dir),mainw->audio_dir);

  button13 = gtk_button_new ();
  gtk_widget_show (button13);
  gtk_box_pack_start (GTK_BOX (hbox20), button13, FALSE, FALSE, 0);

  image413 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image413);
  gtk_container_add (GTK_CONTAINER (button13), image413);

  xranw->subdir_check = gtk_check_button_new_with_mnemonic (_("Include Subdirectories"));
  gtk_widget_show (xranw->subdir_check);
  gtk_box_pack_start (GTK_BOX (hbox20), xranw->subdir_check, FALSE, FALSE, 0);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(xranw->subdir_check),TRUE);
  gtk_widget_modify_fg (xranw->subdir_check, GTK_STATE_NORMAL, &palette->normal_fore);

  hbox22 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox22);
  gtk_box_pack_start (GTK_BOX (vbox19), hbox22, TRUE, TRUE, 0);

  label90 = gtk_label_new (_("Minimum File Size (MB)         "));
  gtk_widget_show (label90);
  gtk_box_pack_start (GTK_BOX (hbox22), label90, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label90), GTK_JUSTIFY_LEFT);
  gtk_widget_modify_fg (label90, GTK_STATE_NORMAL, &palette->normal_fore);

  spinbutton51_adj = gtk_adjustment_new (2, 0, 100, 1, 10, 10);
  xranw->minsize = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton51_adj), 1, 0);
  gtk_widget_show (xranw->minsize);
  gtk_box_pack_start (GTK_BOX (hbox22),xranw->minsize, TRUE, FALSE, 0);

  hbox23 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox23);
  gtk_box_pack_start (GTK_BOX (vbox19), hbox23, TRUE, TRUE, 0);

  label91 = gtk_label_new (_("Maximum File Size (MB)        "));
  gtk_widget_show (label91);
  gtk_box_pack_start (GTK_BOX (hbox23), label91, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label91), GTK_JUSTIFY_LEFT);
  gtk_widget_modify_fg (label91, GTK_STATE_NORMAL, &palette->normal_fore);

  spinbutton52_adj = gtk_adjustment_new (10, 0, 100, 1, 10, 10);
  xranw->maxsize = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton52_adj), 1, 0);
  gtk_widget_show (xranw->maxsize);
  gtk_box_pack_start (GTK_BOX (hbox23), xranw->maxsize, TRUE, FALSE, 0);

  dialog_action_area14 = GTK_DIALOG (xranw->rp_dialog)->action_area;
  gtk_widget_show (dialog_action_area14);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area14), GTK_BUTTONBOX_END);

  cancelbutton12 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton12);
  gtk_dialog_add_action_widget (GTK_DIALOG (xranw->rp_dialog), cancelbutton12, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton12, GTK_CAN_DEFAULT);

  okbutton11 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton11);
  gtk_dialog_add_action_widget (GTK_DIALOG (xranw->rp_dialog), okbutton11, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton11, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (okbutton11);

  g_signal_connect(okbutton11, "clicked", G_CALLBACK (on_xmms_ran_ok_clicked),NULL);
  g_signal_connect(cancelbutton12, "clicked", G_CALLBACK (on_cancel_button1_clicked),xranw);
  g_signal_connect(button13, "clicked", G_CALLBACK (on_filesel_simple_clicked),xranw->dir);
  g_signal_connect (GTK_OBJECT (xranw->rp_dialog), "delete_event",
                      G_CALLBACK (return_true),
                      NULL);

  return xranw;
}


_commentsw*
create_comments_dialog (void)
{
  GtkWidget *dialog_vbox17;
  GtkWidget *table10;
  GtkWidget *label102;
  GtkWidget *label103;
  GtkWidget *label105;
  GtkWidget *dialog_action_area17;
  GtkWidget *okbutton14;
  GtkWidget *cancelbutton14;

  _commentsw *commentsw=(_commentsw*)(g_malloc(sizeof(_commentsw)));

  commentsw->comments_dialog = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER (commentsw->comments_dialog), 10);
  gtk_window_set_title (GTK_WINDOW (commentsw->comments_dialog), _("LiVES: - File Comments (optional)"));
  gtk_window_set_position (GTK_WINDOW (commentsw->comments_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (commentsw->comments_dialog), TRUE);
  gtk_window_set_transient_for(GTK_WINDOW(commentsw->comments_dialog),GTK_WINDOW(mainw->LiVES));
  if (palette->style&STYLE_1) {
    gtk_dialog_set_has_separator(GTK_DIALOG(commentsw->comments_dialog),FALSE);
  }
  gtk_widget_modify_bg (commentsw->comments_dialog, GTK_STATE_NORMAL, &palette->normal_back);

  gtk_window_set_default_size (GTK_WINDOW (commentsw->comments_dialog), 600, 200);

  dialog_vbox17 = GTK_DIALOG (commentsw->comments_dialog)->vbox;
  gtk_widget_show (dialog_vbox17);

  table10 = gtk_table_new (4, 2, FALSE);
  gtk_widget_show (table10);
  gtk_box_pack_start (GTK_BOX (dialog_vbox17), table10, TRUE, TRUE, 0);

  label102 = gtk_label_new (_("Title/Name : "));
  gtk_widget_show (label102);
  gtk_table_attach (GTK_TABLE (table10), label102, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label102), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label102), 0, 0.5);

  label103 = gtk_label_new (_("Author/Artist : "));
  gtk_widget_show (label103);
  gtk_table_attach (GTK_TABLE (table10), label103, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label103), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label103), 0, 0.5);

  label105 = gtk_label_new (_("Comments : "));
  gtk_widget_show (label105);
  gtk_table_attach (GTK_TABLE (table10), label105, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label105), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label105), 0, 0.5);

  commentsw->title_entry = gtk_entry_new ();
  gtk_widget_show (commentsw->title_entry);
  gtk_table_attach (GTK_TABLE (table10), commentsw->title_entry, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_entry_set_text (GTK_ENTRY (commentsw->title_entry),cfile->title);

  commentsw->author_entry = gtk_entry_new ();
  gtk_widget_show (commentsw->author_entry);
  gtk_table_attach (GTK_TABLE (table10), commentsw->author_entry, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_entry_set_text (GTK_ENTRY (commentsw->author_entry),cfile->author);

  commentsw->comment_entry = gtk_entry_new ();
  gtk_widget_show (commentsw->comment_entry);
  gtk_table_attach (GTK_TABLE (table10), commentsw->comment_entry, 1, 2, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_entry_set_max_length (GTK_ENTRY (commentsw->comment_entry), 250);

  gtk_entry_set_text (GTK_ENTRY (commentsw->comment_entry),cfile->comment);

  dialog_action_area17 = GTK_DIALOG (commentsw->comments_dialog)->action_area;
  gtk_widget_show (dialog_action_area17);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area17), GTK_BUTTONBOX_END);

  cancelbutton14 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton14);
  gtk_dialog_add_action_widget (GTK_DIALOG (commentsw->comments_dialog), cancelbutton14, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton14, GTK_CAN_DEFAULT);

  okbutton14 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton14);
  gtk_dialog_add_action_widget (GTK_DIALOG (commentsw->comments_dialog), okbutton14, GTK_RESPONSE_OK);

  GTK_WIDGET_SET_FLAGS (okbutton14, GTK_CAN_DEFAULT|GTK_CAN_FOCUS);
  gtk_widget_grab_focus (okbutton14);
  gtk_widget_grab_default (okbutton14);

  return commentsw;
}

