// utils.c
// LiVES
// (c) G. Finch 2003 <salsaman@xs4all.nl>
// Released under the GPL 2.0
// see file ../COPYING for licensing details

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "main.h"
#include "support.h"

static gboolean  omute,  osepwin,  ofs,  ofaded,  odouble;

inline gchar *
g_strappend (gchar *string, gint len, const gchar *new) {
  g_snprintf(string,len,"%s",g_strconcat (string,new,NULL));
  return string;
}

void d_print(const gchar *text) {
  gchar *switchtext;
  GtkTextIter end_iter;
  gtk_text_buffer_get_end_iter(gtk_text_view_get_buffer(GTK_TEXT_VIEW(mainw->textview1)),&end_iter);

  if (GTK_IS_TEXT_VIEW (mainw->textview1)) {
    if (mainw->current_file!=mainw->last_dprint_file&&mainw->current_file!=0) {
      if (mainw->current_file>0) {
	switchtext=g_strdup_printf (_ ("\n==============================\nSwitched to clip %s\n"),cfile->name);
      }
      else {
	switchtext=g_strdup_printf (_ ("\n==============================\nSwitched to empty clip\n"));
      }
      gtk_text_buffer_insert(gtk_text_view_get_buffer (GTK_TEXT_VIEW (mainw->textview1)),&end_iter,switchtext,-1);
      g_free (switchtext);
      mainw->last_dprint_file=mainw->current_file;
    }
    gtk_text_buffer_insert(gtk_text_view_get_buffer (GTK_TEXT_VIEW (mainw->textview1)),&end_iter,text,-1);
     if (mainw->mark==NULL) {
      mainw->mark=gtk_text_buffer_create_mark(gtk_text_view_get_buffer (GTK_TEXT_VIEW (mainw->textview1)),NULL,&end_iter,FALSE);
    }
     gtk_text_view_scroll_mark_onscreen(GTK_TEXT_VIEW (mainw->textview1),mainw->mark);
  }
}


void
get_next_free_file(void) {
  // get next free file slot, or -1 if we are full
    while ((mainw->first_free_file!=-1)&&!(mainw->files[mainw->first_free_file]==NULL)) {
      mainw->first_free_file++;
      if (mainw->first_free_file>=MAX_FILES) mainw->first_free_file=-1;
    }
}


void
get_dirname(gchar *filename) {
  // get directory name from a file
  //filename should point to gchar[256]
  g_snprintf (filename,256,"%s%s",g_path_get_dirname (filename),G_DIR_SEPARATOR_S);
}

void
get_basename(gchar *filename) {
  // get basename from a file
  //filename should point to gchar[256]
  g_snprintf (filename,256,"%s",g_path_get_basename (filename));
}


void
get_location(const gchar *exe, gchar *val, gint maxlen) {
  gchar *loc;
  if ((loc=g_find_program_in_path (exe))!=NULL) {
    g_snprintf (val,maxlen,"%s",loc);
    g_free (loc);
  }
  else {
    memset (val,0,1);
  }
}





// TODO - move prefs stuff to preferences.c
#define PREFS_TIMEOUT 10000000

void
get_pref(const gchar *key, gchar *val, gint maxlen) {
  FILE *valfile;
  gchar *vfile;
  gchar *com=g_strdup_printf("smogrify get_pref %s",key);
  gint timeout=PREFS_TIMEOUT/prefs->sleep_time;

  memset(val,0,1);

  if (system(com)) {
    tempdir_warning();
    g_free(com);
    return;
  }

  vfile=g_strdup_printf("%s/.smogval.%d.%d",prefs->tmpdir,getuid(),getgid());
  do {
    if (!(valfile=fopen(vfile,"r"))) {
      if (!(mainw==NULL)) {
	while (g_main_context_iteration(NULL,FALSE));
      }
      g_usleep(prefs->sleep_time);
      timeout--;
    }
  } while (!valfile&&timeout>0);
  if (timeout<=0) {
    tempdir_warning();
  }
  else {
    fgets(val,maxlen,valfile);
    fclose(valfile);
    unlink(vfile);
  }
  g_free(vfile);
  g_free(com);
}



void
get_pref_default(const gchar *key, gchar *val, gint maxlen) {
  FILE *valfile;
  gchar *vfile;
  gint timeout=PREFS_TIMEOUT/prefs->sleep_time;
  gchar *com=g_strdup_printf("smogrify get_pref_default %s",key);

  memset(val,0,1);

  if (system(com)) {
      tempdir_warning();
      g_free(com);
      return;
  }
  vfile=g_strdup_printf("%s/.smogval.%d.%d",prefs->tmpdir,getuid(),getgid());
  do {
    if (!(valfile=fopen(vfile,"r"))) {
      if (!(mainw==NULL)) {
	while (g_main_context_iteration(NULL,FALSE));
      }
      g_usleep(prefs->sleep_time);
      timeout--;
    }
  } while (!valfile&&timeout>0);
  if (timeout<=0) {
    tempdir_warning();
  }
  else {
    fgets(val,maxlen,valfile);
    fclose(valfile);
    unlink(vfile);
  }
  g_free(vfile);
  g_free(com);
}


gboolean
get_boolean_pref(const gchar *key) {
  gchar buffer[512];
  get_pref(key,buffer,512);
  if (!strcmp(buffer,"true")) return TRUE;
  return FALSE;
}

gint
get_int_pref(const gchar *key) {
  gchar buffer[512];
  get_pref(key,buffer,512);
  return atoi(buffer);
}

gdouble
get_double_pref(const gchar *key) {
  gchar buffer[512];
  get_pref(key,buffer,512);
  return strtod(buffer,NULL);
}


void
set_pref(const gchar *key, const gchar *value) {
  gchar *com=g_strdup_printf("smogrify set_pref %s \"%s\"",key,value);
  if (system(com)) {
    tempdir_warning();
  }
  g_free(com);
}


void
set_int_pref(const gchar *key, gint value) {
  gchar *com=g_strdup_printf("smogrify set_pref %s %d",key,value);
  if (system(com)) {
    tempdir_warning();
  }
  g_free(com);
}

void
set_double_pref(const gchar *key, gdouble value) {
  gchar *com=g_strdup_printf("smogrify set_pref %s %.3f",key,value);
  if (system(com)) {
    tempdir_warning();
  }
  g_free(com);
}


void
set_boolean_pref(const gchar *key, gboolean value) {
  gchar *com;

  if (value==TRUE) {
    com=g_strdup_printf("smogrify set_pref %s %s",key,"true");
  }
  else {
    com=g_strdup_printf("smogrify set_pref %s %s",key,"false");
  }
  if (system(com)) {
    tempdir_warning();
  }
  g_free(com);
}


gboolean 
apply_prefs(gboolean skip_warn) {
  // set current prefs from prefs dialog
  // TODO - move into prefs.c
  int idx;

  gboolean needs_restart=FALSE;
  const gchar *bgcolour=gtk_entry_get_text(GTK_ENTRY(prefsw->bgcolour_entry));
  const gchar *video_play_command=gtk_entry_get_text(GTK_ENTRY(prefsw->video_command_entry));
  const gchar *video_open_command=gtk_entry_get_text(GTK_ENTRY(prefsw->video_open_entry));
  const gchar *audio_play_command=gtk_entry_get_text(GTK_ENTRY(prefsw->audio_command_entry));
  const gchar *def_vid_load_dir=gtk_entry_get_text(GTK_ENTRY(prefsw->vid_load_dir_entry));
  const gchar *def_vid_save_dir=gtk_entry_get_text(GTK_ENTRY(prefsw->vid_save_dir_entry));
  const gchar *def_audio_dir=gtk_entry_get_text(GTK_ENTRY(prefsw->audio_dir_entry));
  const gchar *def_image_dir=gtk_entry_get_text(GTK_ENTRY(prefsw->image_dir_entry));
  const gchar *def_proj_dir=gtk_entry_get_text(GTK_ENTRY(prefsw->proj_dir_entry));
  const gchar *cdplay_device=gtk_entry_get_text(GTK_ENTRY(prefsw->cdplay_entry));
  gchar tmpdir[256];
  const gchar *theme=gtk_entry_get_text(GTK_ENTRY((GTK_COMBO(prefsw->theme_combo))->entry));
  const gchar *vidp=gtk_entry_get_text(GTK_ENTRY((GTK_COMBO(prefsw->vidp_combo))->entry));
  const gchar *audp=gtk_entry_get_text(GTK_ENTRY((GTK_COMBO(prefsw->audp_combo))->entry));
  const gchar *audio_codec=gtk_entry_get_text(GTK_ENTRY(prefsw->acodec_entry));

  gdouble default_fps=gtk_spin_button_get_value(GTK_SPIN_BUTTON(prefsw->spinbutton_def_fps));
  gboolean pause_xmms=FALSE;
  gboolean antialias=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->stop_screensaver_check));
  gboolean stop_screensaver=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->stop_screensaver_check));
  gboolean open_maximised=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->open_maximised_check));
  gboolean fs_maximised=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->fs_max_check));
  gboolean show_recent=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->recent_check));
  gboolean warn_fps=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->checkbutton_warn_fps));
  gboolean warn_set=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->checkbutton_warn_set));
  gboolean warn_fsize=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->checkbutton_warn_fsize));
  gboolean warn_save_quality=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->checkbutton_warn_save_quality));
  gboolean midisynch=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->check_midi));
  gboolean ins_speed=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->ins_speed));
  gboolean show_player_stats=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->checkbutton_show_stats));
  gboolean debug_encoders=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->debug_encoders));
  gboolean is_pal=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->pal));
  gboolean ext_jpeg=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->jpeg));
  gboolean show_tool=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->show_tool));
  gint fsize_to_warn=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(prefsw->spinbutton_warn_fsize));
  gint dl_bwidth=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(prefsw->spinbutton_bwidth));
  gint ocp=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(prefsw->spinbutton_ocp));
  gboolean save_warn_mask=FALSE;

  gchar video_player[256];
  gchar audio_player[256];


  // get encder acodec
  for (idx=0;strcmp(g_list_nth_data (prefsw->acodec_list,idx),audio_codec);idx++);
  future_prefs->encoder.audio_codec=prefsw->acodec_list_to_format[idx];
  
  g_snprintf (tmpdir,256,"%s",gtk_entry_get_text(GTK_ENTRY(prefsw->tmpdir_entry)));

  g_snprintf(video_player,256,"mplayer");
  if (strcmp (vidp,"mplayer")) {
    g_snprintf(video_player,256,"internal");
  }

  g_snprintf(audio_player,256,"mplayer");
  if (strcmp (audp,"mplayer")) {
    g_snprintf(audio_player,256,"sox");
  }

  // fps warning
  if (warn_fps*WARN_MASK_FPS==(prefs->warning_mask&WARN_MASK_FPS)) {
      prefs->warning_mask^=WARN_MASK_FPS;
      save_warn_mask=TRUE;
  }

  // set warning
  if (warn_set*WARN_MASK_SAVE_SET==(prefs->warning_mask&WARN_MASK_SAVE_SET)) {
      prefs->warning_mask^=WARN_MASK_SAVE_SET;
      save_warn_mask=TRUE;
  }

  // save quality warning
  if (warn_save_quality*WARN_MASK_SAVE_QUALITY==(prefs->warning_mask&WARN_MASK_SAVE_QUALITY)) {
      prefs->warning_mask^=WARN_MASK_SAVE_QUALITY;
      save_warn_mask=TRUE;
  }

  // file size warning
  if (warn_fsize*WARN_MASK_FSIZE==(prefs->warning_mask&WARN_MASK_FSIZE)) {
      prefs->warning_mask^=WARN_MASK_FSIZE;
      save_warn_mask=TRUE;
  }

  if (save_warn_mask==TRUE) {
    set_int_pref("lives_warning_mask",prefs->warning_mask);
  }

  if (fsize_to_warn!=(prefs->warn_file_size)) {
    prefs->warn_file_size=fsize_to_warn;
    set_int_pref("warn_file_size",fsize_to_warn);
  }

  if (dl_bwidth!=(prefs->dl_bandwidth)) {
    prefs->dl_bandwidth=dl_bwidth;
    set_int_pref("dl_bandwidth_K",dl_bwidth);
  }

  if (ocp!=(prefs->ocp)) {
    prefs->ocp=ocp;
    set_int_pref("open_compression_percent",ocp);
  }

  if (show_tool!=(prefs->show_tool)) {
    future_prefs->show_tool=prefs->show_tool=show_tool;
    set_boolean_pref("show_toolbar",show_tool);
  }

  // temp dir
  g_strappend (tmpdir,256,"/");
  get_dirname(tmpdir);

  if (strcmp(prefs->tmpdir,tmpdir)||strcmp (future_prefs->tmpdir,tmpdir)) {
    if (g_file_test (tmpdir, G_FILE_TEST_EXISTS)&&(strlen (tmpdir)<10||strncmp (tmpdir+strlen (tmpdir)-10,"/livestmp/",10))) g_strappend (tmpdir,256,"livestmp/");
    if (strcmp(prefs->tmpdir,tmpdir)||strcmp (future_prefs->tmpdir,tmpdir)) {
      gchar *msg;
      if (!check_dir_access (tmpdir)) {
	msg=g_strdup_printf (_ ("Unable to create or write to the new temporary directory.\nYou may need to create it as the root user first, e.g:\n\nmkdir %s; chmod 777 %s\n\nThe directory will not be changed now.\n"),tmpdir,tmpdir);
	do_blocking_error_dialog (msg);
      }
      else {
	g_snprintf(future_prefs->tmpdir,256,"%s",tmpdir);
	msg=g_strdup (_ ("You have chosen to change the temporary directory.\nPlease make sure you have no other copies of LiVES open.\n\nIf you do have other copies of LiVES open, please close them now, *before* pressing OK.\n\nAlternatively, press Cancel to restore the temporary directory to its original setting."));	
	if (!skip_warn) {
	  if (do_warning_dialog(msg)) {
	    mainw->prefs_changed=needs_restart=TRUE;
	  }
	  else {
	    g_snprintf(future_prefs->tmpdir,256,"%s",prefs->tmpdir);
	  }}}
      g_free (msg);
    }
  }

  // background colour
  if (strcmp(prefs->bgcolour,bgcolour)) {
    GdkColor newcolour;
    
    if (gdk_color_parse (bgcolour, &newcolour)==TRUE) {
      gdk_color_parse (bgcolour, &palette->fade_colour);
      sprintf(prefs->bgcolour,bgcolour);
      set_pref("bgcolour",prefs->bgcolour);
      
      // toolbar buttons
      gtk_widget_modify_bg (mainw->toolbar, GTK_STATE_NORMAL, &palette->fade_colour);
      gtk_widget_modify_bg (mainw->t_stopbutton, GTK_STATE_PRELIGHT, &palette->fade_colour);
      gtk_widget_modify_bg (mainw->t_bckground, GTK_STATE_PRELIGHT, &palette->fade_colour);
      gtk_widget_modify_bg (mainw->t_sepwin, GTK_STATE_PRELIGHT, &palette->fade_colour);
      gtk_widget_modify_bg (mainw->t_double, GTK_STATE_PRELIGHT, &palette->fade_colour);
      gtk_widget_modify_bg (mainw->t_single, GTK_STATE_PRELIGHT, &palette->fade_colour);
      gtk_widget_modify_bg (mainw->t_fullscreen, GTK_STATE_PRELIGHT, &palette->fade_colour);
      gtk_widget_modify_bg (mainw->t_infobutton, GTK_STATE_PRELIGHT, &palette->fade_colour);
      gtk_widget_modify_bg (mainw->t_forward, GTK_STATE_PRELIGHT, &palette->fade_colour);
      gtk_widget_modify_bg (mainw->t_back, GTK_STATE_PRELIGHT, &palette->fade_colour);
      gtk_widget_modify_bg (mainw->t_faster, GTK_STATE_PRELIGHT, &palette->fade_colour);
      gtk_widget_modify_bg (mainw->t_slower, GTK_STATE_PRELIGHT, &palette->fade_colour);
    }
  }
  
  // stop xscreensaver
  if (prefs->stop_screensaver!=stop_screensaver) {
    prefs->stop_screensaver=stop_screensaver;
    set_boolean_pref("stop_screensaver",prefs->stop_screensaver);
  }

  // antialias
  if (prefs->antialias!=antialias) {
    prefs->antialias=antialias;
    set_boolean_pref("antialias",antialias);
  }

  // open maximised
  if (prefs->open_maximised!=open_maximised) {
    prefs->open_maximised=open_maximised;
    set_boolean_pref("open_maximised",open_maximised);
  }

  // filesel maximised
  if (prefs->fileselmax!=fs_maximised) {
    prefs->fileselmax=fs_maximised;
    set_boolean_pref("filesel_maximised",fs_maximised);
  }

  // fps stats
  if (prefs->show_player_stats!=show_player_stats) {
    prefs->show_player_stats=show_player_stats;
    set_boolean_pref("show_player_stats",show_player_stats);
  }

  // show recent
  if (prefs->show_recent!=show_recent) {
    prefs->show_recent=show_recent;
    set_boolean_pref("show_recent_files",show_recent);
    if (prefs->show_recent==TRUE) {
      gtk_widget_show (mainw->recent_menu);
    }
    else {
      gtk_widget_hide (mainw->recent_menu);
    }
  }

  if (capable->has_xmms==TRUE) {
    pause_xmms=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefsw->check_xmms_pause));
    // pause xmms during playback
    if (prefs->pause_xmms!=pause_xmms) {
      prefs->pause_xmms=pause_xmms;
      set_boolean_pref("pause_xmms_on_playback",pause_xmms);
    }
  }
  
  // midi synch
  if (prefs->midisynch!=midisynch) {
    prefs->midisynch=midisynch;
    set_boolean_pref("midisynch",midisynch);
  }

  // jpeg/png
  if (strcmp (prefs->image_ext,"jpg")&&ext_jpeg) {
    set_pref("default_image_format","jpeg");
    g_snprintf (prefs->image_ext,16,"jpg");
  }
  else if (!strcmp(prefs->image_ext,"jpg")&&!ext_jpeg) {
    set_pref("default_image_format","png");
    g_snprintf (prefs->image_ext,16,"png");
  }

  // encoder
  if (strcmp(prefs->encoder.name,future_prefs->encoder.name)) {
    sprintf(prefs->encoder.name,future_prefs->encoder.name);
    set_pref("encoder",prefs->encoder.name);
    sprintf(prefs->encoder.of_restrict,future_prefs->encoder.of_restrict);
    prefs->encoder.of_allowed_acodecs=future_prefs->encoder.of_allowed_acodecs;
  }

  // debug encoder
  if (prefs->debug_encoders!=debug_encoders) {
    set_boolean_pref("debug_encoders",(prefs->debug_encoders=debug_encoders));
  }

  // pal/ntsc
  if (!prefs->is_pal&&is_pal) {
    set_pref("PAL/NTSC","PAL");
    prefs->is_pal=TRUE;
  }
  else if (prefs->is_pal&&!is_pal) {
    set_pref("PAL/NTSC","NTSC");
    prefs->is_pal=FALSE;
  }

  // output format
  if (strcmp(prefs->encoder.of_name,future_prefs->encoder.of_name)) {
    sprintf(prefs->encoder.of_name,future_prefs->encoder.of_name);
    sprintf(prefs->encoder.of_restrict,future_prefs->encoder.of_restrict);
    prefs->encoder.of_allowed_acodecs=future_prefs->encoder.of_allowed_acodecs;
    set_pref("output_type",prefs->encoder.of_name);
  }

  if (prefs->encoder.audio_codec!=future_prefs->encoder.audio_codec) {
    prefs->encoder.audio_codec=future_prefs->encoder.audio_codec;
    if (prefs->encoder.audio_codec<AUDIO_CODEC_NONE) {
      set_int_pref ("encoder_acodec",prefs->encoder.audio_codec);
    }
  }


  // video play command
  if (strcmp(prefs->video_play_command,video_play_command)) {
    sprintf(prefs->video_play_command,video_play_command);
    set_pref("video_play_command",prefs->video_play_command);
  }

  // video open command
  if (strcmp(prefs->video_open_command,video_open_command)) {
    sprintf(prefs->video_open_command,video_open_command);
    set_pref("video_open_command",prefs->video_open_command);
  }

  //playback plugin
  if (strlen (future_prefs->vpp_name)) {
    if (!g_strcasecmp (future_prefs->vpp_name,"none")) {
      if (vid_playback_plugin!=NULL) {
	g_module_close (vid_playback_plugin->gmodule);
	g_free (vid_playback_plugin);
	vid_playback_plugin=NULL;
	set_pref ("vid_playback_plugin","None");
      }
    }
    else {
      if (open_vid_playback_plugin (future_prefs->vpp_name)) {
	set_pref ("vid_playback_plugin",vid_playback_plugin->name);
	do_error_dialog (_ ("\n\nLocal video playback plugins are only activated in\nfull screen, separate window (fs) mode\n"));
      }
    }
    memset (future_prefs->vpp_name,0,64);
  }


  // audio play command
  if (strcmp(prefs->audio_play_command,audio_play_command)) {
    sprintf(prefs->audio_play_command,audio_play_command);
    set_pref("audio_play_command",prefs->audio_play_command);
  }

  // cd play device
  if (strcmp(prefs->cdplay_device,cdplay_device)) {
    sprintf(prefs->cdplay_device,cdplay_device);
    set_pref("cdplay_device",prefs->cdplay_device);
  }

  // default video load directory
  if (strcmp(prefs->def_vid_load_dir,def_vid_load_dir)) {
      g_snprintf(prefs->def_vid_load_dir,256,"%s/",def_vid_load_dir);
      get_dirname(prefs->def_vid_load_dir);
      set_pref("vid_load_dir",prefs->def_vid_load_dir);
      g_snprintf(mainw->vid_load_dir,256,"%s",prefs->def_vid_load_dir);
  }

  // default video save directory
  if (strcmp(prefs->def_vid_save_dir,def_vid_save_dir)) {
      g_snprintf(prefs->def_vid_save_dir,256,"%s/",def_vid_save_dir);
      get_dirname(prefs->def_vid_save_dir);
      set_pref("vid_save_dir",prefs->def_vid_save_dir);
      g_snprintf(mainw->vid_save_dir,256,"%s",prefs->def_vid_save_dir);
  }

  // default audio directory
  if (strcmp(prefs->def_audio_dir,def_audio_dir)) {
      g_snprintf(prefs->def_audio_dir,256,"%s/",def_audio_dir);
      get_dirname(prefs->def_audio_dir);
      set_pref("audio_dir",prefs->def_audio_dir);
      g_snprintf(mainw->audio_dir,256,"%s",prefs->def_audio_dir);
  }

  // default image directory
  if (strcmp(prefs->def_image_dir,def_image_dir)) {
      g_snprintf(prefs->def_image_dir,256,"%s/",def_image_dir);
      get_dirname(prefs->def_image_dir);
      set_pref("image_dir",prefs->def_image_dir);
      g_snprintf(mainw->image_dir,256,"%s",prefs->def_image_dir);
  }

  // default project directory - for backup and restore
  if (strcmp(prefs->def_proj_dir,def_proj_dir)) {
      g_snprintf(prefs->def_proj_dir,256,"%s/",def_proj_dir);
      get_dirname(prefs->def_proj_dir);
      set_pref("proj_dir",prefs->def_proj_dir);
      g_snprintf(mainw->proj_load_dir,256,"%s",prefs->def_proj_dir);
      g_snprintf(mainw->proj_save_dir,256,"%s",prefs->def_proj_dir);
  }

  // the theme
  if (strcmp(future_prefs->theme,theme)) {
      g_snprintf(future_prefs->theme,64,"%s",theme);
      set_pref("gui_theme",future_prefs->theme);
      mainw->prefs_changed=TRUE;
  }

  // default fps
  if (prefs->default_fps!=default_fps) {
    prefs->default_fps=default_fps;
    set_double_pref("default_fps",prefs->default_fps);
  }

  if (ins_speed==prefs->ins_resample) {
    prefs->ins_resample=!ins_speed;
    set_boolean_pref ("insert_resample",prefs->ins_resample);
  }

    // switch to sox
  if (!(strcmp(audio_player,"sox"))&&prefs->audio_player==AUD_PLAYER_MPLAYER) {
    switch_aud_to_sox();
  }

  // switch to mplayer audio
  else if (!(strcmp (audio_player,"mplayer"))&&prefs->audio_player==AUD_PLAYER_SOX) {
    switch_aud_to_mplayer();
 }

  // vid player
  
  if (strcmp(video_player,"mplayer")&&prefs->video_player==VID_PLAYER_MPLAYER) { 
      if (mainw->playing_file>-1) {
	do_error_dialog (_ ("Unable to change video player during playback."));
      }
      else {
	if (prefs->sepwin_type==1&&mainw->sep_win==TRUE) {
	  make_play_window();
	}
	// switch to internal
	switch_to_int_player();

	if (mainw->current_file>-1&&mainw->loop==TRUE) {
	  gtk_widget_set_sensitive(mainw->playsel,TRUE);
	}
	set_pref("video_player",video_player);
	get_play_times();
	gtk_widget_show(mainw->record_perf);
	gtk_widget_set_sensitive (mainw->sticky,TRUE);
	gtk_widget_set_sensitive (mainw->showfct,TRUE);
	gtk_widget_set_sensitive (mainw->loop_ping_pong,TRUE);
	gtk_widget_add_accelerator (mainw->record_perf, "activate", mainw->accel_group,
				    GDK_r, 0,
				    GTK_ACCEL_VISIBLE);
	if (mainw->current_file>-1) {
	  gtk_widget_set_sensitive(mainw->sepwin,TRUE);
	}
	if (mainw->current_file>-1) {
	  if (cfile->perf_counter>0) {
	    d_print(_ ("Ready to record.\n"));
	  }
	}
      }
    }
    else if (strcmp(video_player,"internal")&&prefs->video_player==VID_PLAYER_INT) {
      if (mainw->playing_file>-1) {
	do_error_dialog (_ ("Unable to change video player during playback."));
      }
      else {
	if (prefs->sepwin_type==1&&mainw->sep_win==TRUE) {
	  kill_play_window();
	}
	// switch to mplayer
	switch_to_mplayer();
	prefs->video_player=VID_PLAYER_MPLAYER;
	if (mainw->fs==TRUE) {
	  gtk_widget_set_sensitive(mainw->sepwin,FALSE);
	}
	gtk_widget_set_sensitive (mainw->sticky,FALSE);
	gtk_widget_set_sensitive (mainw->showfct,FALSE);
	gtk_widget_set_sensitive (mainw->loop_ping_pong,FALSE);
	gtk_widget_hide(mainw->record_perf);
	gtk_widget_remove_accelerator(mainw->record_perf, mainw->accel_group, GDK_r, 0);
	// get default command for mplayer
	get_pref_default("mplayer_play_video_command",prefs->video_play_command,256);
	set_pref("video_play_command",prefs->video_play_command);
	set_pref("video_player",video_player);
	get_play_times();
      }
    }
  return needs_restart;
}




void
save_future_prefs(void) {
  // save future prefs on exit, if they have changed

  // show_recent is a special case, future prefs has our original value
  if (prefs->show_recent==FALSE&&future_prefs->show_recent==TRUE) {
    set_pref("recent1","");
    set_pref("recent2","");
    set_pref("recent3","");
    set_pref("recent4","");
  }

  if (strncmp(future_prefs->tmpdir,"NULL",4)) {
    set_pref("tempdir",future_prefs->tmpdir);
  }
  if (prefs->show_tool!=future_prefs->show_tool) {
    set_boolean_pref ("show_toolbar",future_prefs->show_tool);
  }
}




void
get_play_times(void) {
  // update the on-screen timer bars,
  // and if we are not playing,
  // get play times for video, audio channels, and total (longest) time
  gchar *tmpstr;
  gdouble offset=0;
  gdouble offset_left=0;
  gdouble offset_right=0;
  gdouble allocwidth;
  gdouble allocheight;

  if (mainw->current_file==-1||mainw->foreign==TRUE) return;

  if (mainw->playing_file==-1) {
    get_total_time (cfile);
    set_main_title (cfile->name,0);
  }
  // draw timer bars
  allocwidth=mainw->video_draw->allocation.width;
  allocheight=mainw->video_draw->allocation.height;

  gdk_draw_rectangle (mainw->laudio_drawable,
		      mainw->laudio_draw->style->bg_gc[GTK_WIDGET_STATE (mainw->laudio_draw)],
		      TRUE,
		      0, 0,
		      allocwidth,
		      allocheight
		      );
  
  gdk_draw_rectangle (mainw->raudio_drawable,
		      mainw->raudio_draw->style->bg_gc[GTK_WIDGET_STATE (mainw->raudio_draw)],
		      TRUE,
		      0, 0,
		      allocwidth,
		      allocheight
		      );
      
  gdk_draw_rectangle (mainw->video_drawable,
		      mainw->video_draw->style->bg_gc[GTK_WIDGET_STATE (mainw->video_draw)],
		      TRUE,
		      0, 0,
		      allocwidth,
		      allocheight
		      );


  if (cfile->frames>0) {
    if (cfile->events[EVENT_FPS]==NULL) {
      offset_left=(cfile->start-1)/cfile->fps/cfile->total_time*allocwidth;
      offset_right=(cfile->end)/cfile->fps/cfile->total_time*allocwidth;
    }
    else {
      offset_left=calc_variable_time(cfile->start)/cfile->total_time*allocwidth;
      offset_right=calc_variable_time(cfile->end+1)/cfile->total_time*allocwidth;
    }
  
    gdk_draw_rectangle (mainw->video_drawable,
			mainw->video_draw->style->black_gc,
			TRUE,
			0, 0,
			cfile->video_time/cfile->total_time*allocwidth-1,
			prefs->bar_height);
    
    gdk_draw_rectangle (mainw->video_drawable,
			mainw->video_draw->style->white_gc,
			TRUE,
			offset_left, 0,
			offset_right-offset_left,
			prefs->bar_height);
  }

  if (cfile->achans>0) {
    if (mainw->playing_file>-1) {
      offset_left=(mainw->audio_start-1.)/cfile->fps/cfile->total_time*allocwidth;
      if (mainw->audio_end) {
	offset_right=(mainw->audio_end)/cfile->fps/cfile->total_time*allocwidth;
      }
      else {
	offset_right=allocwidth*cfile->laudio_time/cfile->total_time;
      }
    }
    
    
    gdk_draw_rectangle (mainw->laudio_drawable,
			mainw->laudio_draw->style->black_gc,
			TRUE,
			0, 0,
			cfile->laudio_time/cfile->total_time*allocwidth-1,
			prefs->bar_height);
    
    
    gdk_draw_rectangle (mainw->laudio_drawable,
			mainw->laudio_draw->style->white_gc,
			TRUE,
			offset_left, 0,
			offset_right-offset_left,
			prefs->bar_height);
  
    
    gdk_draw_rectangle (mainw->laudio_drawable,
			mainw->laudio_draw->style->bg_gc[GTK_WIDGET_STATE (mainw->laudio_draw)],
			TRUE,
			cfile->laudio_time/cfile->total_time*allocwidth, 0,
			allocwidth-(cfile->laudio_time/cfile->total_time*allocwidth),
			prefs->bar_height);
    
    if (cfile->achans>1) {
      gdk_draw_rectangle (mainw->raudio_drawable,
			  mainw->raudio_draw->style->black_gc,
			  TRUE,
			  0, 0,
			  cfile->raudio_time/cfile->total_time*allocwidth-1,
			  prefs->bar_height);
      
      gdk_draw_rectangle (mainw->raudio_drawable,
			  mainw->raudio_draw->style->white_gc,
			  TRUE,
			  offset_left, 0,
			  offset_right-offset_left,
			  prefs->bar_height);
      
      
      gdk_draw_rectangle (mainw->raudio_drawable,
			  mainw->raudio_draw->style->bg_gc[GTK_WIDGET_STATE (mainw->raudio_draw)],
			  TRUE,
			  cfile->raudio_time/cfile->total_time*allocwidth, 0,
			  allocwidth-(cfile->raudio_time/cfile->total_time*allocwidth),
			  prefs->bar_height);
    }
  }

  // playback cursors
  if (mainw->playing_file>-1) {
    if (prefs->video_player!=VID_PLAYER_MPLAYER) {
      if (cfile->frames>0) {
	if (cfile->events[EVENT_FPS]==NULL) {
	  offset=(mainw->actual_frame-.5)/cfile->fps;
	}
	else {
	  offset=calc_variable_time(mainw->actual_frame);
	}
	offset/=cfile->total_time/allocwidth;
	if (offset>=offset_left&&offset<=offset_right) {
	  gdk_draw_line (mainw->video_drawable,
			 mainw->video_draw->style->black_gc,
			 offset, 0,
			 offset,
			 prefs->bar_height);
	}
	else {
	  gdk_draw_line (mainw->video_drawable,
			 mainw->video_draw->style->white_gc,
			 offset, 0,
			 offset,
			 prefs->bar_height);
	}
	
	if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) { // light style
	  gdk_draw_line (mainw->video_drawable,
			 mainw->video_draw->style->black_gc,
			 offset, prefs->bar_height,
			 offset,
			 allocheight-prefs->bar_height);
	}
	else {
	  gdk_draw_line (mainw->video_drawable,
			 mainw->video_draw->style->white_gc,
			 offset, prefs->bar_height,
			 offset,
			 allocheight-prefs->bar_height);
	}
      }
      
      if (cfile->achans>0&&cfile->is_loaded) {
	offset=allocwidth*(mainw->aframeno-.5)/cfile->fps/cfile->total_time;
	
	if ((offset>=offset_left&&offset<=offset_right)||mainw->loop==TRUE) {
	  gdk_draw_line (mainw->laudio_drawable,
			 mainw->laudio_draw->style->black_gc,
			 offset, 0,
			 offset,
			 prefs->bar_height);
	}
	else {
	  gdk_draw_line (mainw->laudio_drawable,
			 mainw->laudio_draw->style->white_gc,
			 offset, 0,
			 offset,
			 prefs->bar_height);
	}
	
	if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) { // light style
	  gdk_draw_line (mainw->laudio_drawable,
			 mainw->laudio_draw->style->black_gc,
			 offset, prefs->bar_height,
			 offset,
			 allocheight-prefs->bar_height);
	}
	else {
	  gdk_draw_line (mainw->laudio_drawable,
			 mainw->laudio_draw->style->white_gc,
			 offset, prefs->bar_height,
			 offset,
			 allocheight-prefs->bar_height);
	}
	
	if (cfile->achans>1) {
	  if ((offset>=offset_left&&offset<=offset_right)||mainw->loop==TRUE) {
	    gdk_draw_line (mainw->raudio_drawable,
			   mainw->raudio_draw->style->black_gc,
			   offset, 0,
			   offset,
			   prefs->bar_height);
	  }
	  else {
	    gdk_draw_line (mainw->raudio_drawable,
			   mainw->raudio_draw->style->white_gc,
			   offset, 0,
			   offset,
			   prefs->bar_height);
	  }
	  
	  
	  if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) { // light style
	    gdk_draw_line (mainw->raudio_drawable,
			   mainw->raudio_draw->style->black_gc,
			   offset, prefs->bar_height,
			   offset,
			   allocheight-prefs->bar_height);
	  }
	  else {
	    gdk_draw_line (mainw->raudio_drawable,
			   mainw->raudio_draw->style->white_gc,
			   offset, prefs->bar_height,
			   offset,
			   allocheight-prefs->bar_height);
	  }
	}
      }
      GTK_RULER (mainw->hruler)->position=offset*cfile->total_time/allocwidth;
      gtk_widget_queue_draw (mainw->hruler);
    }}
  
  if (mainw->playing_file==-1||(mainw->switch_during_pb&&!mainw->faded)) {
    if (cfile->total_time>0.) {
      // set the range of the timeline
      if (cfile->opening_loc==FALSE) {
	gtk_widget_show (mainw->hruler);
      }
      gtk_widget_show (mainw->eventbox5);
      gtk_widget_show (mainw->video_draw);
      gtk_widget_show (mainw->laudio_draw);
      gtk_widget_show (mainw->raudio_draw);
      GTK_RULER (mainw->hruler)->upper=cfile->total_time;
      draw_little_bars();
      if (mainw->playing_file==-1&&prefs->sepwin_type==1&&mainw->sep_win==TRUE&&cfile->is_loaded&&prefs->video_player!=VID_PLAYER_MPLAYER) {
	if (mainw->preview_box==NULL) {
	  // create the preview box that shows frames
	  make_preview_box();
	}
	// and add it the play window
	if (mainw->preview_box->parent==NULL) {
	  gtk_widget_queue_draw(mainw->play_window);
	  gtk_container_add (GTK_CONTAINER (mainw->play_window), mainw->preview_box);
	}
	load_preview_image(FALSE);
      }
    }
    else {
      gtk_widget_hide (mainw->hruler);
      gtk_widget_hide (mainw->eventbox5);
    }

    if (cfile->opening_loc==TRUE||(cfile->frames==123456789&&cfile->opening==TRUE)) {
      tmpstr=g_strdup_printf(_ ("Video [opening...]"));
    }
    else {
      if (cfile->video_time>0.) {
	tmpstr=g_strdup_printf(_ ("Video [%.2f sec]"),cfile->video_time);
      }
      else {
	if (cfile->video_time<=0.&&cfile->frames>0) {
	  tmpstr=g_strdup (_ ("(Undefined)"));
	}
	else {
	  tmpstr=g_strdup (_ ("(No video)"));
	}
      }
    }
    gtk_label_set_text(GTK_LABEL(mainw->vidbar),tmpstr);
    g_free(tmpstr);
    if (cfile->achans==0) {
      tmpstr=g_strdup (_ ("(No audio)"));
    }
    else {
      if (cfile->opening_audio==TRUE) {
	if (cfile->achans==1) {
	  tmpstr=g_strdup_printf (_ ("Mono  [opening...]"));
	}
	else {
	  tmpstr=g_strdup_printf(_ ("Left Audio [opening...]"));
	}
      }
      else {
	if (cfile->achans==1) {
	  tmpstr=g_strdup_printf(_ ("Mono [%.2f sec]"),cfile->laudio_time);
	}
	else {
	  tmpstr=g_strdup_printf(_ ("Left Audio [%.2f sec]"),cfile->laudio_time);
	}
      }
    }
    gtk_label_set_text(GTK_LABEL(mainw->laudbar),tmpstr);
    g_free(tmpstr);
    if (cfile->achans>1) {
      if (cfile->opening_audio==TRUE) {
	tmpstr=g_strdup_printf(_ ("Right Audio [opening...]"));
      }
      else {
	tmpstr=g_strdup_printf(_ ("Right Audio [%.2f sec]"),cfile->raudio_time);
      }
      gtk_label_set_text(GTK_LABEL(mainw->raudbar),tmpstr);
      gtk_widget_show (mainw->raudbar);
      g_free(tmpstr);
    }
    else {
      gtk_widget_hide (mainw->raudbar);
    }
  }
  gtk_widget_queue_draw(mainw->video_draw);
  gtk_widget_queue_draw(mainw->laudio_draw);
  gtk_widget_queue_draw(mainw->raudio_draw);
  gtk_widget_queue_draw(mainw->vidbar);
  gtk_widget_queue_draw(mainw->hruler);
}
    

void 
draw_little_bars (void) {
//draw the vertical bars when we are not playing
  gdouble allocheight=mainw->video_draw->allocation.height-prefs->bar_height;
  gdouble offset=cfile->pointer_time/cfile->total_time*mainw->vidbar->allocation.width;
  gint frame;

  if (!(frame=calc_variable_frame(cfile->pointer_time,CALC_NEAREST))) frame=cfile->frames;

    if (cfile->frames>0) {
      if (frame>=cfile->start&&frame<=cfile->end) {
	gdk_draw_line (mainw->video_drawable,
		       mainw->video_draw->style->black_gc,
		       offset, 0,
		       offset,
		       prefs->bar_height);
      }
      else {
	gdk_draw_line (mainw->video_drawable,
		       mainw->video_draw->style->white_gc,
		       offset, 0,
		       offset,
		       prefs->bar_height);
      }
      
      if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) { // light style
	gdk_draw_line (mainw->video_drawable,
		       mainw->video_draw->style->black_gc,
		       offset, prefs->bar_height,
		       offset,
		       allocheight);
      }
      else {
	gdk_draw_line (mainw->video_drawable,
		       mainw->video_draw->style->white_gc,
		       offset, prefs->bar_height,
		       offset,
		       allocheight);
      }
    }

    if (cfile->achans>0) {
      if ((frame>=cfile->start&&frame<=cfile->end)||mainw->loop==TRUE) {
	gdk_draw_line (mainw->laudio_drawable,
		       mainw->laudio_draw->style->black_gc,
		       offset, 0,
		       offset,
		       prefs->bar_height);
      }
      else {
	gdk_draw_line (mainw->laudio_drawable,
		       mainw->laudio_draw->style->white_gc,
		       offset, 0,
		       offset,
		       prefs->bar_height);
      }
      
      if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) { // light style
	gdk_draw_line (mainw->laudio_drawable,
		       mainw->laudio_draw->style->black_gc,
		       offset, prefs->bar_height,
		       offset,
		       allocheight);
      }
      else {
	gdk_draw_line (mainw->laudio_drawable,
		       mainw->laudio_draw->style->white_gc,
		       offset, prefs->bar_height,
		       offset,
		       allocheight);
      }

      if (cfile->achans>1) {
	
	if ((frame>=cfile->start&&frame<=cfile->end)||mainw->loop==TRUE) {
	  gdk_draw_line (mainw->raudio_drawable,
			 mainw->raudio_draw->style->black_gc,
			 offset, 0,
			 offset,
			 prefs->bar_height);
	}
	else {
	  gdk_draw_line (mainw->raudio_drawable,
			 mainw->raudio_draw->style->white_gc,
			 offset, 0,
			 offset,
			 prefs->bar_height);
	}
	

	if (palette->style&STYLE_3||palette->style==STYLE_PLAIN) { // light style
	  gdk_draw_line (mainw->raudio_drawable,
			 mainw->raudio_draw->style->black_gc,
			 offset, prefs->bar_height,
			 offset,
			 allocheight);
	}
	else {
	  gdk_draw_line (mainw->raudio_drawable,
			 mainw->raudio_draw->style->white_gc,
			 offset, prefs->bar_height,
			 offset,
			 allocheight);
	}
      }
    }
}



void 
get_total_time (file *file) {
  // get times (video, left and right audio)

  file->laudio_time=file->raudio_time=file->video_time=file->total_time=0.;

  if (file->opening==TRUE&&file->frames!=123456789) {
    if (file->frames*file->fps>0) {
      file->video_time=file->total_time=file->frames/file->fps;
    }
    return;
  }

  if (file->events[EVENT_FPS]!=NULL&&file->frames>0) {
    // calculate variable time if we have fps changes
    file->total_time=file->video_time=calc_variable_time(file->frames+1);
  }
  else {
    if (file->fps>0.) {
      file->total_time=file->video_time=file->frames/file->fps;
    }
  }

  if (file->asampsize*file->arate*file->achans) {
    file->laudio_time=file->afilesize/file->asampsize*8./file->arate/file->achans;
    if (file->achans>1) {
      file->raudio_time=file->laudio_time;
    }
  }

  if (file->laudio_time>file->total_time) file->total_time=file->laudio_time;
  if (file->raudio_time>file->total_time) file->total_time=file->raudio_time;

  if (file->laudio_time+file->raudio_time==0&&file->opening==FALSE) {
    file->achans=file->afilesize=file->asampsize=file->arate=file->arps=0;
  }
}


gdouble 
calc_variable_time (gint frame) {
  // return the absolute time of the start of a frame
  // even  when there are FPS changes recorded
  // (assumes that frames are not reordered)
  int i;
  gdouble time;
  gdouble fps=cfile->fps;

  if (cfile->perf_counter>0||prefs->video_player==VID_PLAYER_MPLAYER||cfile->events[EVENT_FPS]==NULL||frame<cfile->perf_start) {
    return (frame-1)/cfile->fps;
  }

  time=(cfile->perf_start-1)/fps;

  // walk fps events and sum the total intervals
  for (i=0;i<=(cfile->perf_end-cfile->perf_start);i++) {
    if (cfile->perf_start+i>=frame) return time;
    if ((cfile->events[EVENT_FPS]+i)->value>0) fps=(cfile->events[EVENT_FPS]+i)->value/1000.;
    time+=1./fps;
  }

  // performance is shorter
  time+=(frame-cfile->perf_end-1)/cfile->fps;
  return time;
}

gint
calc_variable_frame (gdouble time, gshort calc_method) {
  // return the nearest frame for a given time even when there are FPS changes 
  // recorded
  // should work even with reordered frames
  // WARNING: assumes lengthof cfile->events[EVENT_FPS] == lengthof cfile->events[EVENT_FRAMES]
  // == cfile->perf_end-cfile->perf_start+1

  // TODO - optimise this for times > cfile->perf_end (low priority)

  // calc method can now be one of CALC_LAST (for resampling) 
  // or CALC_NEAREST (for selections)

  int i;
  gdouble xtime;
  gdouble fps=cfile->fps;
  gint last_perf_val;
  gint perf_length=cfile->perf_end-cfile->perf_start;
  gint value;

  if (time<0.) return cfile->frames?1:0;

  if (prefs->video_player==VID_PLAYER_MPLAYER||(cfile->perf_start-1)/fps>=time||cfile->events[EVENT_FPS]==NULL) {
    if (!(last_perf_val=(gint)(time*fps+(calc_method==CALC_NEAREST?1.49999:.99999)))) last_perf_val=1;
    return last_perf_val>cfile->frames?0:last_perf_val;
  }

  xtime=(cfile->perf_start-1)/fps;

  // walk fps events and sum the intervals
  for (i=0;i<=perf_length;i++) {
    if ((value=(cfile->events[EVENT_FPS]+i)->value)>0) {
      fps=value/1000.;
    }
    if ((xtime+=1./fps)>time) {
      if (calc_method==CALC_LAST||(xtime-time)>(time+1./fps-xtime)) {
	if (cfile->events[EVENT_FRAMES]!=NULL) {
	  return (cfile->events[EVENT_FRAMES]+i)->value;
	}
      }
      else if (++i<=(cfile->perf_end-cfile->perf_start)&&cfile->events[EVENT_FRAMES]!=NULL) {
	return (cfile->events[EVENT_FRAMES]+i)->value;
      }
      return cfile->perf_start+i<=cfile->frames?cfile->perf_start+i:1;
    }
  }

  if (cfile->events[EVENT_FRAMES]!=NULL) {
    last_perf_val=(cfile->events[EVENT_FRAMES]+i-1)->value;
    return (last_perf_val=(gint)(last_perf_val+(time-xtime)*cfile->fps+calc_method==CALC_NEAREST?0.:-.49999))>cfile->frames?0:last_perf_val;
  }
  return (last_perf_val=(gint)(cfile->perf_end+(time-xtime)*cfile->fps+calc_method==CALC_NEAREST?0.:-.49999))>cfile->frames?0:last_perf_val;
}



void 
find_when_to_stop (void) {
  // work out when to stop playing
  //
  // ---------------
  //        no loop              loop to fit                 loop cont
  //        -------              -----------                 ---------
  // a>v    stop on video end    stop on audio end           no stop
  // v>a    stop on video end    stop on video end           no stop
  if (cfile->clip_type!=CLIP_TYPE_DISK) mainw->whentostop=NEVER_STOP;
  else if (cfile->opening_only_audio) mainw->whentostop=STOP_ON_AUD_END;
  else if (cfile->opening_audio) mainw->whentostop=STOP_ON_VID_END;
  else if (mainw->loop_cont) mainw->whentostop=NEVER_STOP;
  else if (mainw->loop&&cfile->achans>0&&(mainw->audio_end/cfile->fps)<MAX (cfile->laudio_time,cfile->raudio_time)) mainw->whentostop=STOP_ON_AUD_END;
  else mainw->whentostop=STOP_ON_VID_END; // tada...
}

void 
minimise_aspect_delta (gdouble aspect,gint hblock,gint vblock,gint hsize,gint vsize,gint *width,gint *height) {
  // we will use trigonometry to calculate the smallest difference between a given
  // aspect ratio and the actual frame size. If the delta is smaller than current 
  // we set the height and width
  gint cw=width[0];
  gint ch=height[0];

  gint real_width,real_height;
  gint delta,current_delta=(hsize-cw)*(hsize-cw)+(vsize-ch)*(vsize-ch);

  // minimise x^2 - y^2, get approximate values
  gint calc_width=(gint)((gdouble)((vsize-1)/(aspect*aspect-1))+.5);

  int i;

  // use the block size to find the nearest allowed size
  for (i=-1;i<2;i++) {
    real_width=(gint)(calc_width/hblock+i)*hblock;
    real_height=(gint)(real_width*aspect/vblock+.5)*vblock;
    delta=(hsize-real_width)*(hsize-real_width)+(vsize-real_height)*(vsize-real_height);
    if (delta<current_delta) {
      current_delta=delta;
      width[0]=real_width;
      height[0]=real_height;
    }
  }
}

void 
zero_spinbuttons (void) {
  g_signal_handler_block(mainw->spinbutton_start,mainw->spin_start_func);
  gtk_spin_button_set_range(GTK_SPIN_BUTTON(mainw->spinbutton_start),0,0);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_start),0);
  g_signal_handler_unblock(mainw->spinbutton_start,mainw->spin_start_func);
  g_signal_handler_block(mainw->spinbutton_end,mainw->spin_end_func);
  gtk_spin_button_set_range(GTK_SPIN_BUTTON(mainw->spinbutton_end),0,0);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(mainw->spinbutton_end),0);
  g_signal_handler_unblock(mainw->spinbutton_end,mainw->spin_end_func);
}


void 
hide_cursor(GdkWindow *window) {
  unsigned char cursor_bits[] = {0x00};
  unsigned char cursormask_bits[] = {0x00};
  GdkCursor *cursor;
  GdkPixmap *source, *mask;
  GdkColor fg = { 0, 0, 0, 0 };
  GdkColor bg = { 0, 0, 0, 0 };

  if (prefs->video_player!=VID_PLAYER_INT) return;

  //make the cursor invisible in playback windows
  source = gdk_bitmap_create_from_data (NULL, cursor_bits,
					1, 1);
  mask = gdk_bitmap_create_from_data (NULL, cursormask_bits,
				      1, 1);
  cursor = gdk_cursor_new_from_pixmap (source, mask, &fg, &bg, 0, 0);
  gdk_pixmap_unref (source);
  gdk_pixmap_unref (mask);

  gdk_window_set_cursor (window, cursor);
}


void 
unhide_cursor(GdkWindow *window) {
  if (prefs->video_player!=VID_PLAYER_INT) return;
  gdk_window_set_cursor(window,NULL);
}



void
switch_to_int_player(void) {
  // WARNING: this does not set preferences in ~/.lives
  prefs->video_player=VID_PLAYER_INT;
  if (mainw->plug1==NULL) mainw->xwin=0;
  // this will force adding the image to a new plug when we next play
}



void
switch_to_mplayer(void) {
  // WARNING: this does not set preferences in ~/.lives
  int i;

  for (i=1;i<MAX_FILES;i++) {
    if (!(mainw->files[i]==NULL)) {
      if (i!=mainw->current_file&&mainw->files[i]->opening==TRUE) {
	do_error_dialog(_ ("LiVES cannot switch to mplayer whilst clips are loading."));
	return;
      }
    }
  }
  
  prefs->video_player=VID_PLAYER_MPLAYER;
  
  // need to recreate the socket window for mplayer
  gtk_widget_ref (mainw->playarea);

  if (!(mainw->plug1==NULL)) {
    // pull our image from the plug
    gtk_container_remove (GTK_CONTAINER (mainw->plug1), mainw->image274);
  }
  
  // gtk+ complains here about G_IS_OBJECT being FALSE
  // it really doesnt like it...
  // but socket must be removed and recreated or we can't play in it
  gtk_container_remove (GTK_CONTAINER (mainw->playframe), mainw->playarea);

  // the plug is free'd automatically
  mainw->plug1=NULL;

  // create a new playarea and add it in to playframe
  mainw->playarea = gtk_socket_new ();
  gtk_container_add (GTK_CONTAINER (mainw->playframe), mainw->playarea);
  
  // force updating of the window id next time we play
  mainw->xwin=0;
}


void
switch_aud_to_sox(void) {
  prefs->audio_player=AUD_PLAYER_SOX;
  get_pref_default("sox_command",prefs->audio_play_command,256);
  set_pref("audio_player","sox");
  set_pref("audio_play_command",prefs->audio_play_command);
  if (mainw->current_file>0) {
    gtk_widget_set_sensitive (mainw->resample_audio, cfile->achans>0);
  }
}



void
switch_aud_to_mplayer(void) {
  int i;
  for (i=1;i<MAX_FILES;i++) {
      if (!(mainw->files[i]==NULL)) {
	if (i!=mainw->current_file&&mainw->files[i]->opening==TRUE) {
	  do_error_dialog(_ ("LiVES cannot switch to mplayer whilst clips are loading."));
	  return;
	}
      }
    }
  
  prefs->audio_player=AUD_PLAYER_MPLAYER;
  get_pref_default("mplayer_audio_command",prefs->audio_play_command,256);
  set_pref("audio_player","mplayer");
  set_pref("audio_play_command",prefs->audio_play_command);
  gtk_widget_set_sensitive (mainw->resample_audio, FALSE);
}



void
prepare_to_play_foreign(void) {
  // here we are going to 'play' a captured external window
  gint new_file=mainw->first_free_file;

  // create a new 'file' to play into
  if (get_new_handle(new_file,NULL)==FALSE) {
    return;
  }

  mainw->current_file=new_file;

  cfile->hsize=mainw->foreign_width/2+1;
  cfile->vsize=mainw->foreign_height/2+3;

  cfile->fps=prefs->default_fps;

  resize(-2);

  gtk_widget_show (mainw->playframe);
  gtk_widget_show (mainw->playarea);
  while (g_main_context_iteration(NULL,FALSE));

  // size must be exact, must not be larger than play window or we end up with nothing
  mainw->pwidth=mainw->playframe->allocation.width-H_RESIZE_ADJUST+2;
  mainw->pheight=mainw->playframe->allocation.height-V_RESIZE_ADJUST+2;

  cfile->hsize=mainw->pwidth;
  cfile->vsize=mainw->pheight;

  // for some strange reason we now have to do this twice...and now it doesn't
  // always work
  gtk_socket_add_id(GTK_SOCKET(mainw->playarea),mainw->foreign_id);
  gtk_socket_add_id(GTK_SOCKET(mainw->playarea),mainw->foreign_id);
  while (g_main_context_iteration(NULL,FALSE));

  mainw->foreign_map=gdk_pixmap_foreign_new(mainw->foreign_id);
  mainw->foreign_cmap=gdk_colormap_get_system();

  mainw->play_start=1;
  mainw->play_end=prefs->max_record_frames;

  omute=mainw->mute;
  osepwin=mainw->sep_win;
  ofs=mainw->fs;
  ofaded=mainw->faded;
  odouble=mainw->double_size;

  mainw->mute=TRUE;
  mainw->sep_win=FALSE;
  mainw->fs=FALSE;
  mainw->faded=TRUE;
  mainw->double_size=FALSE;

  gtk_widget_hide(mainw->t_double);
  gtk_widget_hide(mainw->t_single);
  gtk_widget_hide(mainw->t_fullscreen);
  gtk_widget_hide(mainw->t_bckground);
  gtk_widget_hide(mainw->t_sepwin);
  gtk_widget_hide(mainw->t_infobutton);
}


gboolean
after_foreign_play(void) {
  // read details from capture file
  int capture_fd;
  gchar *capfile=g_strdup_printf("%s/.capture.%d",prefs->tmpdir,getpid());
  gchar capbuf[256];
  ssize_t length;
  gint new_file;
  gint new_frames=0;
  gint old_file=mainw->current_file;

  gchar *com;
  gchar **array;
  gchar file_name[256];

  // assume for now we only get one clip passed back
  if ((capture_fd=open(capfile,O_RDONLY))>0) {
    memset(capbuf,0,256);
    if ((length=read(capture_fd,capbuf,256))) {
      if (get_token_count (capbuf,'|')>2) {
	array=g_strsplit(capbuf,"|",3);
	new_frames=atoi(array[1]);
	new_file=mainw->first_free_file;
	mainw->current_file=new_file;
	cfile=(file *)(g_malloc(sizeof(file)));
	g_snprintf(cfile->handle,255,"%s",array[0]);
	g_strfreev(array);
	create_cfile();
	g_snprintf(cfile->file_name,256,"Capture %d",mainw->cap_number);
	g_snprintf(cfile->name,256,"Capture %d",mainw->cap_number++);
	g_snprintf(cfile->type,40,"Frames");
	cfile->progress_start=cfile->start=1;
	cfile->progress_end=cfile->frames=cfile->end=new_frames;

	g_snprintf(file_name,256,"%s/%s/",prefs->tmpdir,cfile->handle);
	
	com=g_strdup_printf("smogrify fill_and_redo_frames %s %d %d %d %d",cfile->handle,1,cfile->frames,mainw->foreign_width,mainw->foreign_height);
	unlink(cfile->info_file);
	system(com);
	cfile->nopreview=TRUE;
	if (do_progress_dialog(TRUE,TRUE,_ ("Cleaning up clip"))) {
	  array=g_strsplit(mainw->msg,"|",5);
	  new_frames=atoi(array[1]);
	  cfile->hsize=atoi(array[2]);
	  cfile->vsize=atoi(array[3]);
	  cfile->bpp=atoi(array[4]);
	  g_strfreev(array);
	  
	  if (new_frames>0) {
	    cfile->start=1;
	    cfile->end=cfile->frames=new_frames;
	    get_next_free_file();
	  }
	}
	else {
	  // cancelled cleanup
	  cfile->frames=cfile->start=cfile->end=new_frames=0;
	}
	g_free(com);
      }
    }
    close(capture_fd);
    unlink(capfile);
  }
  else {
    // nothing captured
    g_free(capfile);
    if (mainw->current_file>0) {
      switch_to_file (old_file,old_file);
      cfile->nopreview=FALSE;
    }
    else {
      close_current_file();
    }
    return FALSE;
  }
  cfile->nopreview=FALSE;
  g_free(capfile);

  if (new_frames==0) {
    // failed to get any frames from this clip
    close_current_file();
    return FALSE;
  }

  add_to_winmenu();
  switch_to_file(old_file,mainw->current_file);
  cfile->is_loaded=TRUE;
  cfile->changed=TRUE;
  return TRUE;
}


void
set_menu_text(GtkWidget *menuitem, const gchar *text, gboolean use_mnemonic) {
  GtkWidget *label;
  if (GTK_IS_MENU_ITEM (menuitem)) {
    label=gtk_bin_get_child(GTK_BIN(menuitem));
    if (use_mnemonic) {
      gtk_label_set_text_with_mnemonic(GTK_LABEL(label),text);
    }
    else {
      gtk_label_set_text(GTK_LABEL(label),text);
    }
  }
}


void
get_menu_text(GtkWidget *menuitem, gchar *text) {
  GtkWidget *label=gtk_bin_get_child(GTK_BIN(menuitem));
  g_snprintf(text,255,"%s",gtk_label_get_text(GTK_LABEL(label)));
}



void 
reset_clip_menu (void) {
  // sometimes the clip menu gets messed up, e.g. after reloading a set.
  // This function will clean up the 'x's and so on.

  int i;
  GtkWidget *active_image=NULL;

  for (i=1;i<=MAX_FILES;i++) {
    if (!(mainw->files[i]==NULL)) {
      if (!(active_image==NULL)) {
	active_image=NULL;
      }
      if (mainw->files[i]->opening==TRUE) {
	active_image = gtk_image_new_from_stock ("gtk-no", GTK_ICON_SIZE_MENU);
      }
      else {
	if (i==mainw->current_file) {
	  active_image = gtk_image_new_from_stock ("gtk-close", GTK_ICON_SIZE_MENU);
	}
      }
      if (!(active_image==NULL)) {
	gtk_widget_show (active_image);
      }
      if (mainw->files[i]->menuentry!=NULL) {
	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mainw->files[i]->menuentry), active_image);
      }
    }
  }
}



gboolean
check_file(const gchar *file_name) {
  int check;
  gboolean exists=FALSE;
  gchar *msg;

  // check if file exists
  if (g_file_test (file_name, G_FILE_TEST_EXISTS)) {
    msg=g_strdup_printf (_ ("\n%s\nalready exists.\n\nOverwrite ?\n"),file_name);
    if (!do_warning_dialog(msg)) {
      g_free (msg);
      return FALSE;
    }
    g_free (msg);
    check=open(file_name,O_RDWR);
    exists=TRUE;
  }
  // if not, check if we can write to it
  else {
    check=open(file_name,O_CREAT|O_EXCL|O_RDWR);
  }

  if (check<0) {
    if (errno==EACCES&&mainw!=NULL&&mainw->is_ready) {
      g_snprintf(mainw->msg,512,_ ("\nLiVES was unable to write to the file:\n%s\nPlease check the file permissions and try again."),file_name);
      do_error_dialog(mainw->msg);
    }
    return FALSE;
  }
  if (check) {
    close(check);
    if (!exists) {
      unlink (file_name);
    }
    return TRUE;
  }
  return FALSE;
}


gboolean 
check_dir_access (gchar *dir) {
  // if a directory exists, make sure it is readable and writable
  // otherwise create it and then check
  gboolean exists=g_file_test (dir, G_FILE_TEST_EXISTS);
  gchar *com;
  gchar *testfile;
  gboolean is_OK=FALSE;

  if (!exists) {
    com=g_strdup_printf ("mkdir %s",dir);
    system (com);
    g_free (com);
  }
  if (!g_file_test(dir, G_FILE_TEST_IS_DIR)) return FALSE;

  testfile=g_strdup_printf ("%s/livestst.txt",dir);
  com=g_strdup_printf ("touch %s",testfile);
  system (com);
  g_free (com);
  if ((is_OK=g_file_test(testfile, G_FILE_TEST_EXISTS))) {
    unlink (testfile);
  }
  g_free (testfile);
  if (!exists) {
    com=g_strdup_printf ("rm -r %s",dir);
    system (com);
    g_free (com);
  }
  return is_OK;
}






guint
get_file_size(int fd) {
  // get the size of file fd
  struct stat filestat;
  fstat(fd,&filestat);
  return (guint)(filestat.st_size);
}

guint
sget_file_size(gchar *name) {
  // get the size of file fd
  struct stat filestat;
  int fd;

  if ((fd=open(name,O_RDONLY))>0) {
    close (fd);
  }
  else {
    return (guint)0;
  }
  stat(name,&filestat);
  return (guint)(filestat.st_size);
}


void 
reget_afilesize (file *file) {
  // re-get the audio file size
  // NB. file here is a struct _file
  gchar *afile=g_strdup_printf ("%s/%s/audio",prefs->tmpdir,file->handle);
  if (!(file->afilesize=sget_file_size (afile))) {
    if (file->opening==FALSE) {
      file->arate=file->achans=file->asampsize=file->arps=0;
    }
  }
  g_free (afile);
}


void
quantise_events (gint rec_start)
{ 
  // this function is called immediately after recording a performance

  // when we recorded frame changes, they could have happened at any time,
  // because when the user presses a key, the frame change is immediate
  // and not in time with a normal frame slot

  // therefore, we need quantise the frame changes so that they occur only on real frame timings,
  // otherwise we will have too many events, and the video will play too slow

  // what we will do is run through all the events, and then move them to the nearest actual frame timing
  // if there are multiple frame events for one real frame, we will use the nearest one to the frame slot

  //e.g: if we have the following before quantisation (with an fps of 25)

  //  index 'frame'     actual frame                 actual time        played at
  //            1           10                       0.02                0.00
  //            2           11                       0.03                0.04
  //            3           14                       0.037               0.08

  // we would end up with:

  //            1           10                       0.00               0.00
  //            2           14                       0.04               0.04

  // note that we also need to do this with other types of events - e.g. fps changes

  // However, as we go forward we must keep track of the fps changes so that we always know when the frames should fall.
  // we also need to fill in any gaps of zeros we find (can be caused by skipped frames); we will use 'last_frame' and 'last_fps' for this


  // Note on events:
  // Frames can also be skipped on playback; so we should repeat an event if we want to make sure the associated events are acted on

  // Here, we make runs of fps changes for that reason.



  // Note 2: we make 3 arrays.
  // 1) The first array which is recorded, and it goes from 0 to cfile->perf_end-1.
  //    cfile->perf_end here just holds the size of our array
  //    It has event values and relative time (since the start of play).

  // 2) We reorder this array to a new one, putting in the events on the frame changes, and filling in any zeros.
  //    It goes from 0 (frame 1) upwards and is the same size as array 1

  //  cfile->perf_end is now set to the frame number of the last event

  // 3) The third array is created, which goes from cfile->perf_start to cfile->perf_end


  // Note also that we actually store fps*1000


  int i,j,framec;
  event *temp_frames,*temp_fps;
  gint frame;

  long double reltime=cfile->ev_origticks,abstime,last_abstime=0l;
  long double offset_frames, delta;
  gint qframe=rec_start+1;  // our quantised frame number
  gint diff_frame,last_frame=0;
  gdouble curr_fps=cfile->fps;
  long double t_stamp=0.;
  gint fps=0;
  guint last_fps=0;
  int last_fps_index=0;
  gint real_perf_end=0;
  gboolean alloc=TRUE;
  gint event_space,event_overkill;
  gboolean backfill_to_start=FALSE;
  gboolean reset_qframe=FALSE;

  // for resampling, we want to backfill to the start
  if (rec_start<0) {
    rec_start=-rec_start;
    qframe=rec_start;
    backfill_to_start=TRUE;
  }

  // first we reference the current events
  temp_frames=cfile->events[EVENT_FRAMES];
  temp_fps=cfile->events[EVENT_FPS];

  cfile->events[EVENT_FRAMES]=NULL;
  cfile->events[EVENT_FPS]=NULL;

  // now make a new event space
  
  // TODO - use predetermined size
  event_overkill=(gint)(FPS_MAX*10/(cfile->fps+1));

  if (create_event_space((cfile->perf_end-cfile->perf_start+1)*event_overkill,CREATE_EV_FRAMES|(CREATE_EV_FPS*(temp_fps!=NULL)))==FALSE) {
    cfile->perf_counter=cfile->perf_start=cfile->perf_end=0;
    return;
  }
  j=cfile->ev_fps_start-1;
  
  for (i=cfile->perf_start-1;i<cfile->perf_end;i++) {
    // walk thru frame events
    frame=(temp_frames+i)->value;
    reltime=(temp_frames+i)->reltime;
    fps=0;

    // if we had an fps change before this frame change, deal with that first
    if (temp_fps!=NULL) {
      if (j<cfile->ev_fps_end) {
      // before or simultaneous with frame change
      // fps events with a reltime of zero are ignored
	if ((temp_fps+j)->reltime<=reltime&&(temp_fps+j)->reltime!=0l&&(temp_fps+j)->value) {
	  fps=(temp_fps+j)->value;
	  // reverse play is not needed now, since we have the frames recorded
	  if (fps<0) fps=-fps;
	  i--;
	  frame=0;
	  reltime=(temp_fps+j)->reltime;
	}
	// next fps change is after the frame change, come back to it later
	else {
	  if ((temp_fps+j)->reltime>0l&&(temp_fps+j)->value) {
	    j--;
	  }
	}
      }
      j++;
    }

    // get the absolute time of our event
    abstime=reltime-mainw->origticks;

    // find our quantised frame at the current fps
    offset_frames=(abstime-t_stamp)/1000000.*curr_fps;
    diff_frame=(gint)(offset_frames);
    delta=offset_frames-(long double)diff_frame;
    
      if (reset_qframe&&frame!=0) {
	qframe+=diff_frame;
	diff_frame=0;
	t_stamp=abstime;
	curr_fps=last_fps/1000.;
	reset_qframe=FALSE;
      }

    // round to nearest whole frame
    if (delta>.5) {
      delta=1.-delta;
      diff_frame++;
    }
    // ignore any events before perf_start (e.g. when we resample)
    if ((frame!=0||fps!=0)&&qframe+diff_frame>=cfile->perf_start) {
      real_perf_end=qframe+diff_frame;

      // OK, we have an event !
      diff_frame-=cfile->perf_start-1;
	  
      if (frame!=0) {
	last_abstime=abstime;
	// fill any zero gaps (can be caused by skipped frames in playback)
	if (last_frame!=0||backfill_to_start==TRUE) {
	  if (prefs->q_type==Q_FILL) {
	    // fill quantisation
	    for (framec=qframe+diff_frame-1;framec>0&&(cfile->events[EVENT_FRAMES]+framec-1)->value==0; framec--) {
	      (cfile->events[EVENT_FRAMES]+framec-1)->value=frame;
	    }
	  }
	  else {
	    // smooth quantisation
	    gint length=1,prev_frame=frame;
	    for (framec=qframe+diff_frame-1;framec>0&&(cfile->events[EVENT_FRAMES]+framec-1)->value==0; framec--) {
	      length++;
	    }
	    if (framec>0) {
	      prev_frame=(cfile->events[EVENT_FRAMES]+framec-1)->value;
	    }
	    for (framec=qframe+diff_frame-1;framec>0&&(cfile->events[EVENT_FRAMES]+framec-1)->value==0; framec--) {
	      (cfile->events[EVENT_FRAMES]+framec-1)->value=frame+(gint)((prev_frame-frame)/length*(framec-qframe-diff_frame+2));
	    }
	  }
	}
	else {
	  // adjust perf_start to the real perf_start
	  cfile->perf_start+=qframe+diff_frame-1;
	}
	// write new frame into our quantised time slot
	// we will cunningly use the reltime field now to record our delta time,
	// that way we can find the nearest event

	if ((cfile->events[EVENT_FRAMES]+qframe+diff_frame-1)->reltime>delta||(cfile->events[EVENT_FRAMES]+qframe+diff_frame-1)->reltime==0) {
	  (cfile->events[EVENT_FRAMES]+qframe+diff_frame-1)->value=last_frame=frame;
	  (cfile->events[EVENT_FRAMES]+qframe+diff_frame-1)->reltime=delta;
	}
      }

      if (fps!=0) {
	// write fps event
	(cfile->events[EVENT_FPS]+qframe+diff_frame-1)->value=fps;
	// we also need to fill in any zeros, as we might skip the frame with the actual fps change...
	if (last_fps!=0) {
	  for (framec=qframe+diff_frame-1;framec>last_fps_index&&framec>0&&(cfile->events[EVENT_FPS]+framec-1)->value==0; framec--) {
	    (cfile->events[EVENT_FPS]+framec-1)->value=last_fps;
	    (cfile->events[EVENT_FPS]+framec-1)->reltime=reltime;
	  }
	}
	last_fps=fps;
	last_fps_index=qframe+diff_frame;
	// fps change - make this our new baseline for frame counting, update the virtual fps
	// we attach any fps changes to the last frame event
	// this is not 100% accurate for variable fps
	reset_qframe=TRUE;
      }
    } // end event
  } // end for()

  if (last_fps==0) {
    // no fps changes were recorded, we can drop event_fps
    if (cfile->events[EVENT_FPS]!=NULL) {
      g_free(cfile->events[EVENT_FPS]);
      cfile->events[EVENT_FPS]=NULL;
    }
  }
  else {
    // fill to end with fps - the event_pending system will ensure our fps is reset to the default once we reach the performance end
    for (i=qframe+1;i<real_perf_end;i++) {
      (cfile->events[EVENT_FPS]+i-1)->value=last_fps;
    }
  }

  if (last_frame==0) {
    g_free(cfile->events[EVENT_FRAMES]);
    cfile->events[EVENT_FRAMES]=NULL;
  }


  // now we can free temp (our record array)
  g_free(temp_frames);
  g_free(temp_fps);

  if (last_fps==0&&last_frame==0) {
    d_print(_ ("nothing recorded.\n"));
    cfile->perf_start=cfile->perf_end=cfile->perf_counter=0;
    return;
  }

  // pass 1 done
  cfile->perf_end=real_perf_end;

  // finally, we will make our real event spaces, 

  // last_frame should never be zero, but just in case, we check
  if (last_frame) {
    temp_frames=cfile->events[EVENT_FRAMES];
    cfile->events[EVENT_FRAMES]=NULL;
  }
  if (last_fps) {
    temp_fps=cfile->events[EVENT_FPS];
    cfile->events[EVENT_FPS]=NULL;
  }

  event_space=cfile->perf_end-cfile->perf_start+1;

  // make the correctly sized event spaces
  if (last_frame) {
    // frames
    if (create_event_space (event_space,CREATE_EV_FRAMES)==FALSE) {
      g_free(temp_frames);
      cfile->events[EVENT_FPS]=NULL;
      if (last_fps&&temp_fps!=NULL) {
	g_free(temp_fps);
	cfile->events[EVENT_FPS]=NULL;
      }
      alloc=FALSE;
    }
  }

  if (alloc==TRUE) {
    // fps
    if (last_fps) {
      if (create_event_space (event_space,CREATE_EV_FPS)==FALSE) {
	g_free(temp_fps);
	cfile->events[EVENT_FPS]=NULL;
	if (!(cfile->events[EVENT_FRAMES]==NULL)) {
	  g_free(temp_frames);
	  cfile->events[EVENT_FRAMES]=NULL;
	}
	alloc=FALSE;
      }
    }
  }

  if (alloc==FALSE) {
    cfile->perf_start=cfile->perf_end=cfile->perf_counter=0;
    return;
  }

  for (i=0;i<event_space;i++) {
    if (last_frame) {
      (cfile->events[EVENT_FRAMES]+i)->value=(temp_frames+i)->value;
    }
    if (last_fps) {
      (cfile->events[EVENT_FPS]+i)->value=(temp_fps+i)->value;
      (cfile->events[EVENT_FPS]+i)->reltime=(temp_fps+i)->reltime;
    }
  }

  if (last_frame) {
    g_free(temp_frames);
  }
  if (last_fps) {
    g_free(temp_fps);
  }
}





void
colour_equal(GdkColor *c1, const GdkColor *c2) {
  c1->pixel=c2->pixel;
  c1->red=c2->red;
  c1->green=c2->green;
  c1->blue=c2->blue;
}


gboolean
create_event_space(gint length, guint64 what_to_create) {
  // try to create desired events
  // if we run out of memory, all events requested are freed, and we return FALSE
  // otherwise we return TRUE

  int j;
  guint64 x=1;

  for (j=0;j<=LAST_EVENT;j++) {
    if (what_to_create&x) {
      if (cfile->events[j]!=NULL) {
	g_free(cfile->events[j]);
      }
      if ((cfile->events[j]=(event *)(g_try_malloc(sizeof(event)*length)))==NULL) {
	// memory overflow
	x=1;
	for (j=0;j<=LAST_EVENT;j++) {
	  if (what_to_create&x) {
	    if (cfile->events[j]!=NULL) {
	      g_free(cfile->events[j]);
	      cfile->events[j]=NULL;
	    }
	  }
	  x*=2;
	}
	return FALSE;
      }
      memset(cfile->events[j],0,length*sizeof(event));
    }
    x*=2;
  }
  return TRUE;
}




void 
add_to_recent(const gchar *filename, gdouble start, gint frames) {
  gchar buff[256];
  gchar *file;

  if (frames>0) {
    file=g_strdup_printf ("%s|%.2f|%d",filename,start,frames);
  }
  else {
    file=g_strdup (filename);
  }


  get_menu_text(mainw->recent1,buff);
  if (strcmp(file,buff)) {
    get_menu_text(mainw->recent2,buff);
    if (strcmp(file,buff)) {
      get_menu_text(mainw->recent3,buff);
      if (strcmp(file,buff)) {
	// not in list, or at pos 4
	get_menu_text(mainw->recent3,buff);
	set_menu_text(mainw->recent4,buff,FALSE);
	set_pref("recent4",buff);
	
	get_menu_text(mainw->recent2,buff);
	set_menu_text(mainw->recent3,buff,FALSE);
	set_pref("recent3",buff);
	
	get_menu_text(mainw->recent1,buff);
	set_menu_text(mainw->recent2,buff,FALSE);
	set_pref("recent2",buff);
	
	set_menu_text(mainw->recent1,file,FALSE);
	set_pref("recent1",file);
      }
      else {
	// #3 in list
	get_menu_text(mainw->recent2,buff);
	set_menu_text(mainw->recent3,buff,FALSE);
	set_pref("recent3",buff);
	
	get_menu_text(mainw->recent1,buff);
	set_menu_text(mainw->recent2,buff,FALSE);
	set_pref("recent2",buff);
	
	set_menu_text(mainw->recent1,file,FALSE);
	set_pref("recent1",filename);
      }
    }
    else {
      // #2 in list
	get_menu_text(mainw->recent1,buff);
	set_menu_text(mainw->recent2,buff,FALSE);
	set_pref("recent2",buff);
	
	set_menu_text(mainw->recent1,file,FALSE);
	set_pref("recent1",filename);
    }
  }
  else {
    // I'm number 1, so why change ;-)
  }

    get_menu_text(mainw->recent1,buff);
    if (strlen(buff)) {
      gtk_widget_show (mainw->recent1);
    }
    get_menu_text(mainw->recent2,buff);
    if (strlen(buff)) {
      gtk_widget_show (mainw->recent2);
    }
    get_menu_text(mainw->recent3,buff);
    if (strlen(buff)) {
      gtk_widget_show (mainw->recent3);
    }
    get_menu_text(mainw->recent4,buff);
    if (strlen(buff)) {
      gtk_widget_show (mainw->recent4);
    }
}


gint 
verhash (gchar *version) {
  gchar *s;
  gint major=0;
  gint minor=0;
  gint micro=0;

  if (!(strlen(version))) return 0;

  s=strtok (version,".");
  if (!(s==NULL)) {
    major=atoi (s);
    s=strtok (NULL,".");
    if (!(s==NULL)) {
      minor=atoi (s);
      s=strtok (NULL,".");
      if (!(s==NULL)) {
	micro=atoi (s);
      }
    }
  }
  return major*1000000+minor*1000+micro;
}



// TODO - move into undo.c
void 
set_undoable (const gchar *what, gboolean sensitive) {
  if (mainw->current_file>-1) {
    cfile->redoable=FALSE;
    cfile->undoable=sensitive;
    if (!(what==NULL)) {
      gchar *what_safe=g_strdelimit (g_strdup (what),"_",' ');
      g_snprintf(cfile->undo_text,32,_ ("_Undo %s"),what_safe);
      g_snprintf(cfile->redo_text,32,_ ("_Redo %s"),what_safe);
      g_free (what_safe);
    }
    else {
      cfile->undoable=FALSE;
      cfile->undo_action=UNDO_NONE;
      g_snprintf(cfile->undo_text,32,_ ("_Undo"));
      g_snprintf(cfile->redo_text,32,_ ("_Redo"));
    }
    set_menu_text(mainw->undo,cfile->undo_text,TRUE);
    set_menu_text(mainw->redo,cfile->redo_text,TRUE);
  }

  gtk_widget_hide(mainw->redo);
  gtk_widget_show(mainw->undo);
  gtk_widget_set_sensitive (mainw->undo,sensitive);
}

void 
set_redoable (const gchar *what, gboolean sensitive) {
  if (mainw->current_file>-1) {
    cfile->undoable=FALSE;
    cfile->redoable=sensitive;
    if (!(what==NULL)) {
      gchar *what_safe=g_strdelimit (g_strdup (what),"_",' ');
      g_snprintf(cfile->undo_text,32,_ ("_Undo %s"),what_safe);
      g_snprintf(cfile->redo_text,32,_ ("_Redo %s"),what_safe);
      g_free (what_safe);
    }
    else {
      cfile->redoable=FALSE;
      cfile->undo_action=UNDO_NONE;
      g_snprintf(cfile->undo_text,32,_ ("_Undo"));
      g_snprintf(cfile->redo_text,32,_ ("_Redo"));
    }
    set_menu_text(mainw->undo,cfile->undo_text,TRUE);
    set_menu_text(mainw->redo,cfile->redo_text,TRUE);
  }

  gtk_widget_hide(mainw->undo);
  gtk_widget_show(mainw->redo);
  gtk_widget_set_sensitive (mainw->redo,sensitive);
}


void 
set_sel_label (void) {
  gchar *tstr,*frstr;

  if (mainw->current_file==-1||!cfile->frames) {
    gtk_label_set_text(GTK_LABEL(mainw->sel_label),_ ("-------------Selection------------"));
  }
  else {
    tstr=g_strdup_printf ("%.2f",calc_variable_time (cfile->end+1)-calc_variable_time (cfile->start));
    frstr=g_strdup_printf ("%d",cfile->end-cfile->start+1);
    gtk_label_set_text(GTK_LABEL(mainw->sel_label),g_strconcat (_ ("---------- [ "),tstr,_(" sec ] ----------Selection---------- [ "),frstr,_ (" frames ] ----------"),NULL));
    g_free (frstr);
    g_free (tstr);
  }
  gtk_widget_queue_draw (mainw->sel_label);
}




guint 
get_signed_endian (int asigned, int endian) {
  if (asigned==1) {
    if (endian==1) {
      return 0;
    }
    if (endian==0) {
      return AFORM_BIG_ENDIAN;
    }
  }
  else {
    if (asigned==0) { 
      if (endian==1) {
	return AFORM_UNSIGNED;
      }
      if (endian==0) {
	return AFORM_UNSIGNED|AFORM_BIG_ENDIAN;
      }
    }
  }
  return AFORM_UNKNOWN;
}



inline void 
clear_mainw_msg (void) {
  memset (mainw->msg,0,512);
}


gint 
get_token_count (gchar *string, int delim) {
  gint pieces=1;
  if (string==NULL) return 0;
  if (delim<=0||delim>255) return 1;

  while ((string=index (string,delim))!=NULL) {
    pieces++;
    string++;
  }
  return pieces;
}





void combo_set_popdown_strings (GtkCombo *combo, GList *list) {
  // this avoids an assert in some versions of GTK+ when list==NULL
  GList *empty_list=NULL;
  empty_list=g_list_append (empty_list,"");
  if (list==NULL) gtk_combo_set_popdown_strings (combo,empty_list);
  else gtk_combo_set_popdown_strings (combo,list);
  g_list_free (empty_list);
}


void 
get_border_size (GtkWidget *win, gint *bx, gint *by) {
  GdkRectangle rect;
  gint wx,wy;
  gdk_window_get_frame_extents (GDK_WINDOW (win->window),&rect);
  gdk_window_get_origin (GDK_WINDOW (win->window), &wx, &wy);
  bx[0]=wx-rect.x;
  by[0]=wy-rect.y;
}



gint hextodec (const gchar *string) {
  int i;
  gint tot=0;
  gchar test[2];

  memset (test+1,0,1);

  for (i=0;i<strlen (string);i++) {
    tot*=16;
    memcpy (test,&string[i],1);
    tot+=get_hex_digit (test);
  }
  return tot;
}

gint get_hex_digit (gchar *c) {
  if (!strcmp (c,"a")||!strcmp (c,"A")) return 10;
  if (!strcmp (c,"b")||!strcmp (c,"B")) return 11;
  if (!strcmp (c,"c")||!strcmp (c,"C")) return 12;
  if (!strcmp (c,"d")||!strcmp (c,"D")) return 13;
  if (!strcmp (c,"e")||!strcmp (c,"E")) return 14;
  if (!strcmp (c,"f")||!strcmp (c,"F")) return 15;
  return(atoi (c));
}





static guint32 fastrand_val;

inline guint32 fastrand(void)
{
  return (fastrand_val=fastrand_val*1073741789+32749);
}

void fastsrand(guint32 seed)
{
	fastrand_val = seed;
}



GdkPixmap*
gdk_pixmap_copy (GdkPixmap *pixmap)
{
  GdkPixmap *pixmap_out;
  GdkGC *gc;
  gint width, height, depth;
  
  g_return_val_if_fail (pixmap != NULL, NULL);
  
  gdk_drawable_get_size (pixmap, &width, &height);
  depth = gdk_drawable_get_depth (pixmap);
  pixmap_out = gdk_pixmap_new (NULL, width, height, depth);
  gc = gdk_gc_new (pixmap);
  gdk_draw_drawable (pixmap_out, gc, pixmap, 0, 0, 0, 0, width, height);
  g_object_unref (gc);
  return pixmap_out;
}




void set_alwaysontop(GtkWidget *window, gboolean ontop) {
  XEvent ev;
  Display *display = GDK_WINDOW_XDISPLAY( window->window );
  Atom net_wm_state = gdk_x11_get_xatom_by_name( "_NET_WM_STATE" );
  Atom net_wm_state_above = gdk_x11_get_xatom_by_name( "_NET_WM_STATE_ABOVE" );
  
  ev.type = ClientMessage;
  ev.xclient.window = GDK_WINDOW_XWINDOW( window->window );
  ev.xclient.message_type = net_wm_state;
  ev.xclient.format = 32;
  ev.xclient.data.l[ 0 ] = ontop==TRUE ? 1 : 0;
  ev.xclient.data.l[ 1 ] = net_wm_state_above;
  ev.xclient.data.l[ 2 ] = 0;

  XSendEvent( display, DefaultRootWindow( display ), False,
	      SubstructureNotifyMask|SubstructureRedirectMask, &ev );
  
}
